@@ -3,13 +3,13 @@
  *
  * Unit Cell utility functions
  *
- * Copyright © 2012 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Lorenzo Galli
  *
  * Authors:
- *   2009-2012 Thomas White <taw@physics.org>
- *   2012      Lorenzo Galli
+ *   2009-2012,2014-2015 Thomas White <taw@physics.org>
+ *   2012                Lorenzo Galli
  *
  * This file is part of CrystFEL.
  *
@@ -128,6 +128,43 @@ LatticeType lattice_from_str(const char *s)
 }
 
 
+static int check_centering(char cen)
+{
+	switch ( cen ) {
+
+		case 'P' :
+		case 'A' :
+		case 'B' :
+		case 'C' :
+		case 'I' :
+		case 'F' :
+		case 'R' :
+		case 'H' :
+		return 0;
+
+		default:
+		return 1;
+
+	}
+}
+
+
+static int check_unique_axis(char ua)
+{
+	switch ( ua ) {
+
+		case 'a' :
+		case 'b' :
+		case 'c' :
+		return 0;
+
+		default:
+		return 1;
+
+	}
+}
+
+
 int right_handed(UnitCell *cell)
 {
 	double asx, asy, asz;
@@ -201,39 +238,46 @@ void cell_print(UnitCell *cell)
 		STATUS(", unique axis %c", cell_get_unique_axis(cell));
 	}
 
-	if ( right_handed(cell) ) {
-		STATUS(", right handed");
+	if ( cell_has_parameters(cell) ) {
+		if ( right_handed(cell) ) {
+			STATUS(", right handed.\n");
+		} else {
+			STATUS(", left handed.\n");
+		}
 	} else {
-		STATUS(", left handed");
+		STATUS(".\n");
 	}
 
-	STATUS(", point group '%s'.\n", cell_get_pointgroup(cell));
+	if ( cell_has_parameters(cell) ) {
+		cell_get_parameters(cell, &a, &b, &c, &alpha, &beta, &gamma);
 
-	cell_get_parameters(cell, &a, &b, &c, &alpha, &beta, &gamma);
+		STATUS("a      b      c            alpha   beta  gamma\n");
+		STATUS("%6.2f %6.2f %6.2f A    %6.2f %6.2f %6.2f deg\n",
+		       a*1e10, b*1e10, c*1e10,
+		       rad2deg(alpha), rad2deg(beta), rad2deg(gamma));
 
-	STATUS("a      b      c            alpha   beta  gamma\n");
-	STATUS("%6.2f %6.2f %6.2f A    %6.2f %6.2f %6.2f deg\n",
-	       a*1e10, b*1e10, c*1e10,
-	       rad2deg(alpha), rad2deg(beta), rad2deg(gamma));
+		cell_get_cartesian(cell, &ax, &ay, &az, &bx, &by, &bz, &cx, &cy, &cz);
 
-	cell_get_cartesian(cell, &ax, &ay, &az, &bx, &by, &bz, &cx, &cy, &cz);
+		STATUS("a = %10.3e %10.3e %10.3e m\n", ax, ay, az);
+		STATUS("b = %10.3e %10.3e %10.3e m\n", bx, by, bz);
+		STATUS("c = %10.3e %10.3e %10.3e m\n", cx, cy, cz);
 
-	STATUS("a = %10.3e %10.3e %10.3e m\n", ax, ay, az);
-	STATUS("b = %10.3e %10.3e %10.3e m\n", bx, by, bz);
-	STATUS("c = %10.3e %10.3e %10.3e m\n", cx, cy, cz);
+		cell_get_reciprocal(cell, &asx, &asy, &asz,
+			                  &bsx, &bsy, &bsz,
+			                  &csx, &csy, &csz);
 
-	cell_get_reciprocal(cell, &asx, &asy, &asz,
-	                          &bsx, &bsy, &bsz,
-	                          &csx, &csy, &csz);
+		STATUS("a* = %10.3e %10.3e %10.3e m^-1 (modulus %10.3e m^-1)\n",
+		       asx, asy, asz, modulus(asx, asy, asz));
+		STATUS("b* = %10.3e %10.3e %10.3e m^-1 (modulus %10.3e m^-1)\n",
+		       bsx, bsy, bsz, modulus(bsx, bsy, bsz));
+		STATUS("c* = %10.3e %10.3e %10.3e m^-1 (modulus %10.3e m^-1)\n",
+		       csx, csy, csz, modulus(csx, csy, csz));
 
-	STATUS("astar = %10.3e %10.3e %10.3e m^-1 (modulus = %10.3e m^-1)\n",
-	                             asx, asy, asz, modulus(asx, asy, asz));
-	STATUS("bstar = %10.3e %10.3e %10.3e m^-1 (modulus = %10.3e m^-1)\n",
-	                             bsx, bsy, bsz, modulus(bsx, bsy, bsz));
-	STATUS("cstar = %10.3e %10.3e %10.3e m^-1 (modulus = %10.3e m^-1)\n",
-	                             csx, csy, csz, modulus(csx, csy, csz));
+		STATUS("Cell representation is %s.\n", cell_rep(cell));
 
-	STATUS("Cell representation is %s.\n", cell_rep(cell));
+	} else {
+		STATUS("Unit cell parameters are not specified.\n");
+	}
 }
 
 
@@ -562,10 +606,12 @@ UnitCell *match_cell(UnitCell *cell_in, UnitCell *template_in, int verbose,
 
 	/* "Un-center" the template unit cell to make the comparison easier */
 	template = uncenter_cell(template_in, &uncentering);
+	if ( template == NULL ) return NULL;
 
 	/* The candidate cell is also uncentered, because it might be centered
 	 * if it came from (e.g.) MOSFLM */
 	cell = uncenter_cell(cell_in, NULL);
+	if ( cell == NULL ) return NULL;
 
 	if ( cell_get_reciprocal(template, &asx, &asy, &asz,
 	                         &bsx, &bsy, &bsz,
@@ -1044,6 +1090,15 @@ static void determine_lattice(UnitCell *cell,
 }
 
 
+/**
+ * load_cell_from_pdb:
+ * @filename: The filename from which to load the cell
+ *
+ * Loads a unit cell from the CRYST1 line of a PDB file.
+ *
+ * Returns: a newly allocated %UnitCell.
+ *
+ */
 UnitCell *load_cell_from_pdb(const char *filename)
 {
 	FILE *fh;
@@ -1096,10 +1151,9 @@ UnitCell *load_cell_from_pdb(const char *filename)
 
 			determine_lattice(cell, as, bs, cs, als, bes, gas);
 
-			if ( strlen(line) > 65 ) {
+			if ( strlen(line) > 55 ) {
 				cell_set_centering(cell, line[55]);
 			} else {
-				cell_set_pointgroup(cell, "1");
 				ERROR("CRYST1 line without centering.\n");
 			}
 
@@ -1110,6 +1164,229 @@ UnitCell *load_cell_from_pdb(const char *filename)
 
 	fclose(fh);
 
+	if ( cell != NULL ) {
+		validate_cell(cell);
+	} else {
+		ERROR("Failed to load cell from %s\n", filename);
+	}
+
+
+	return cell;
+}
+
+
+static int get_length_m(char **bits, int nbits, double *pl)
+{
+	char *rval;
+
+	if ( nbits < 4 ) {
+		ERROR("No units specified for '%s'\n", bits[0]);
+		return 1;
+	}
+
+	*pl = strtod(bits[2], &rval);
+	if ( *rval != '\0' ) {
+		ERROR("Invalid value '%s'.\n", bits[2]);
+		return 1;
+	}
+
+	if ( strcmp(bits[3], "nm") == 0 ) {
+		*pl *= 1e-9;
+	} else if ( strcmp(bits[3], "A") == 0 ) {
+		*pl *= 1e-10;
+	} else {
+		ERROR("Unrecognised length units '%s'\n", bits[3]);
+		return 1;
+	}
+
+	return 0;
+}
+
+
+static int get_angle_rad(char **bits, int nbits, double *pl)
+{
+	char *rval;
+
+	if ( nbits < 4 ) {
+		ERROR("No units specified for '%s'\n", bits[0]);
+		return 1;
+	}
+
+	*pl = strtod(bits[2], &rval);
+	if ( *rval != '\0' ) {
+		ERROR("Invalid value '%s'.\n", bits[2]);
+		return 1;
+	}
+
+	if ( strcmp(bits[3], "rad") == 0 ) {
+		/* Do nothing, already in rad */
+	} else if ( strcmp(bits[3], "deg") == 0 ) {
+		*pl = deg2rad(*pl);
+	} else {
+		ERROR("Unrecognised angle units '%s'\n", bits[3]);
+		return 1;
+	}
+
+	return 0;
+}
+
+
+/**
+ * load_cell_from_file:
+ * @filename: The filename from which to load the cell
+ *
+ * Loads a unit cell from a file of any type (PDB or CrystFEL format)
+ *
+ * Returns: a newly allocated %UnitCell.
+ *
+ */
+UnitCell *load_cell_from_file(const char *filename)
+{
+	FILE *fh;
+	char *rval;
+	char line[1024];
+	UnitCell *cell;
+	int have_a = 0, have_b = 0, have_c = 0;
+	int have_al = 0,  have_be = 0,  have_ga = 0;
+	double a, b, c, al, be, ga;
+
+	fh = fopen(filename, "r");
+	if ( fh == NULL ) {
+		ERROR("Couldn't open '%s'\n", filename);
+		return NULL;
+	}
+
+	rval = fgets(line, 1023, fh);
+	chomp(line);
+
+	if ( strcmp(line, "CrystFEL unit cell file version 1.0") != 0 ) {
+		fclose(fh);
+		return load_cell_from_pdb(filename);
+	}
+
+	cell = cell_new();
+
+	do {
+
+		char line[1024];
+		int n1;
+		int i;
+		char **bits;
+
+		rval = fgets(line, 1023, fh);
+		chomp(line);
+
+		n1 = assplode(line, " \t", &bits, ASSPLODE_NONE);
+		if ( n1 < 3 ) {
+			for ( i=0; i<n1; i++ ) free(bits[i]);
+			free(bits);
+			continue;
+		}
+
+		if ( bits[0][0] == ';' ) {
+			for ( i=0; i<n1; i++ ) free(bits[i]);
+			free(bits);
+			continue;
+		}
+
+		if ( strcmp(bits[0], "lattice_type") == 0 ) {
+			LatticeType lt = lattice_from_str(bits[2]);
+			cell_set_lattice_type(cell, lt);
+
+		} else if ( strcmp(bits[0], "centering") == 0 ) {
+			char cen = bits[2][0];
+			if ( !check_centering(cen) ) {
+				cell_set_centering(cell, cen);
+			} else {
+				ERROR("Unrecognised centering '%c'\n", cen);
+			}
+
+		} else if ( strcmp(bits[0], "unique_axis") == 0 ) {
+			char ua = bits[2][0];
+			if ( !check_unique_axis(ua) ) {
+				cell_set_unique_axis(cell, ua);
+			} else {
+				ERROR("Unrecognised unique axis '%c'\n", ua);
+			}
+
+		} else if ( strcmp(bits[0], "a") == 0 ) {
+			if ( !get_length_m(bits, n1, &a) ) {
+				have_a = 1;
+			}
+
+		} else if ( strcmp(bits[0], "b") == 0 ) {
+			if ( !get_length_m(bits, n1, &b) ) {
+				have_b = 1;
+			}
+
+		} else if ( strcmp(bits[0], "c") == 0 ) {
+			if ( !get_length_m(bits, n1, &c) ) {
+				have_c = 1;
+			}
+
+		} else if ( strcmp(bits[0], "al") == 0 ) {
+			if ( !get_angle_rad(bits, n1, &al) ) {
+				have_al = 1;
+			}
+
+		} else if ( strcmp(bits[0], "be") == 0 ) {
+			if ( !get_angle_rad(bits, n1, &be) ) {
+				have_be = 1;
+			}
+
+		} else if ( strcmp(bits[0], "ga") == 0 ) {
+			if ( !get_angle_rad(bits, n1, &ga) ) {
+				have_ga = 1;
+			}
+
+		} else {
+			ERROR("Unrecognised field '%s'\n", bits[0]);
+		}
+
+		for ( i=0; i<n1; i++ ) free(bits[i]);
+		free(bits);
+
+	} while ( rval != NULL );
+
+	fclose(fh);
+
+	if ( have_a && have_b && have_c && have_al && have_be && have_ga ) {
+		cell_set_parameters(cell, a, b, c, al, be, ga);
+	}
+
+	switch ( cell_get_lattice_type(cell) ) {
+
+		case L_TRICLINIC :
+		case L_ORTHORHOMBIC :
+		case L_CUBIC :
+		case L_RHOMBOHEDRAL :
+		if ( (cell_get_unique_axis(cell) != '?')
+		  && (cell_get_unique_axis(cell) != '*') ) {
+			ERROR("WARNING: Unique axis '%c' doesn't make sense "
+			      "for lattice type %s.\n",
+			      cell_get_unique_axis(cell),
+			      str_lattice(cell_get_lattice_type(cell)));
+		}
+		break;
+
+		case L_MONOCLINIC :
+		case L_TETRAGONAL :
+		case L_HEXAGONAL :
+		if ( (cell_get_unique_axis(cell) == '?')
+		  || (cell_get_unique_axis(cell) == '*') ) {
+			ERROR("You must specify the unique axis for lattice "
+			      "type %s.\n",
+			      str_lattice(cell_get_lattice_type(cell)));
+			return NULL;
+		}
+		break;
+
+		default :
+		ERROR("Unrecognised lattice type %i\n",
+		      cell_get_lattice_type(cell));
+		break;
+	}
+
 	validate_cell(cell);
 
 	return cell;
@@ -1231,7 +1508,7 @@ int validate_cell(UnitCell *cell)
 	int err = 0;
 	char cen, ua;
 
-	if ( !cell_is_sensible(cell) ) {
+	if ( cell_has_parameters(cell) && !cell_is_sensible(cell) ) {
 		ERROR("WARNING: Unit cell parameters are not sensible.\n");
 		err = 1;
 	}
@@ -1242,7 +1519,7 @@ int validate_cell(UnitCell *cell)
 		err = 1;
 	}
 
-	if ( !right_handed(cell) ) {
+	if ( cell_has_parameters(cell) && !right_handed(cell) ) {
 		ERROR("WARNING: Unit cell is not right handed.\n");
 		err = 1;
 	}
@@ -1301,3 +1578,31 @@ int forbidden_reflection(UnitCell *cell,
 
 	return 0;
 }
+
+
+/* Returns cell volume in A^3 */
+double cell_get_volume(UnitCell *cell)
+{
+	double asx, asy, asz;
+	double bsx, bsy, bsz;
+	double csx, csy, csz;
+	struct rvec aCb;
+	double rec_volume;
+
+	if ( cell_get_reciprocal(cell, &asx, &asy, &asz,
+	                               &bsx, &bsy, &bsz,
+	                               &csx, &csy, &csz) ) {
+		ERROR("Couldn't get reciprocal cell.\n");
+		return 0;
+	}
+
+	/* "a" cross "b" */
+	aCb.u = asy*bsz - asz*bsy;
+	aCb.v = - (asx*bsz - asz*bsx);
+	aCb.w = asx*bsy - asy*bsx;
+
+	/* "a cross b" dot "c" */
+	rec_volume = (aCb.u*csx + aCb.v*csy + aCb.w*csz)/1e30;
+
+	return 1/rec_volume;
+}
