@@ -3,15 +3,15 @@
  *
  * A class representing a unit cell
  *
- * Copyright © 2012 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  * Copyright © 2012 Lorenzo Galli
  *
  * Authors:
- *   2009-2012 Thomas White <taw@physics.org>
- *   2010,2012 Richard Kirian
- *   2012      Lorenzo Galli
+ *   2009-2012,2014 Thomas White <taw@physics.org>
+ *   2010,2012      Richard Kirian
+ *   2012           Lorenzo Galli
  *
  * This file is part of CrystFEL.
  *
@@ -117,6 +117,8 @@ extern UnitCell *cell_new_from_reciprocal_axes(struct rvec as, struct rvec bs,
 extern UnitCell *cell_new_from_direct_axes(struct rvec as, struct rvec bs,
                                            struct rvec cs);
 
+extern int cell_has_parameters(UnitCell *cell);
+
 extern void cell_set_cartesian(UnitCell *cell,
                                double ax, double ay, double az,
                                double bx, double by, double bz,
@@ -125,12 +127,6 @@ extern void cell_set_cartesian(UnitCell *cell,
 extern void cell_set_parameters(UnitCell *cell, double a, double b, double c,
 				double alpha, double beta, double gamma);
 
-extern void cell_set_cartesian_a(UnitCell *cell, double ax, double ay, double az);
-extern void cell_set_cartesian_b(UnitCell *cell, double bx, double by, double bz);
-extern void cell_set_cartesian_c(UnitCell *cell, double cx, double cy, double cz);
-extern void cell_set_pointgroup(UnitCell *cell, const char *sym);
-
-
 extern int cell_get_parameters(UnitCell *cell, double *a, double *b, double *c,
                                double *alpha, double *beta, double *gamma);
 
@@ -149,8 +145,6 @@ extern void cell_set_reciprocal(UnitCell *cell,
                                 double bsx, double bsy, double bsz,
                                 double csx, double csy, double csz);
 
-extern const char *cell_get_pointgroup(UnitCell *cell);
-
 extern LatticeType cell_get_lattice_type(UnitCell *cell);
 extern void cell_set_lattice_type(UnitCell *cell, LatticeType lattice_type);
 
