@@ -3,11 +3,11 @@
  *
  * A class representing a single crystal
  *
- * Copyright © 2013 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2013-2015 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2013 Thomas White <taw@physics.org>
+ *   2013-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -59,10 +59,13 @@ extern double crystal_get_profile_radius(Crystal *cryst);
 extern RefList *crystal_get_reflections(Crystal *cryst);
 extern double crystal_get_resolution_limit(Crystal *cryst);
 extern long long int crystal_get_num_saturated_reflections(Crystal *cryst);
+extern long long int crystal_get_num_implausible_reflections(Crystal *cryst);
 extern int crystal_get_user_flag(Crystal *cryst);
 extern double crystal_get_osf(Crystal *cryst);
+extern double crystal_get_Bfac(Crystal *cryst);
 extern struct image *crystal_get_image(Crystal *cryst);
 extern double crystal_get_mosaicity(Crystal *cryst);
+extern const char *crystal_get_notes(Crystal *cryst);
 
 extern void crystal_set_cell(Crystal *cryst, UnitCell *cell);
 extern void crystal_set_profile_radius(Crystal *cryst, double r);
@@ -70,10 +73,16 @@ extern void crystal_set_reflections(Crystal *cryst, RefList *reflist);
 extern void crystal_set_resolution_limit(Crystal *cryst, double res);
 extern void crystal_set_num_saturated_reflections(Crystal *cryst,
                                                   long long int n);
+extern void crystal_set_num_implausible_reflections(Crystal *cryst,
+                                                    long long int n);
 extern void crystal_set_user_flag(Crystal *cryst, int flag);
 extern void crystal_set_osf(Crystal *cryst, double osf);
+extern void crystal_set_Bfac(Crystal *cryst, double B);
 extern void crystal_set_image(Crystal *cryst, struct image *image);
 extern void crystal_set_mosaicity(Crystal *cryst, double m);
+extern void crystal_set_notes(Crystal *cryst, const char *notes);
+
+extern void crystal_add_notes(Crystal *cryst, const char *notes_add);
 
 #ifdef __cplusplus
 }
