@@ -3,12 +3,13 @@
  *
  * Detector properties
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  *
  * Authors:
- *   2009-2014 Thomas White <taw@physics.org>
+ *   2009-2015 Thomas White <taw@physics.org>
+ *   2014      Valerio Mariani
  *   2014      Kenneth Beyerlein <kenneth.beyerlein@desy.de>
  *   2011      Andrew Aquila
  *   2011      Richard Kirian <rkirian@asu.edu>
@@ -42,7 +43,6 @@
 #include "image.h"
 #include "utils.h"
 #include "detector.h"
-#include "beam-parameters.h"
 #include "hdf5-file.h"
 
 
@@ -59,6 +59,19 @@
  */
 
 
+
+struct rg_definition {
+	char *name;
+	char *pns;
+};
+
+
+struct rgc_definition {
+	char *name;
+	char *rgs;
+};
+
+
 static int atob(const char *a)
 {
 	if ( strcasecmp(a, "true") == 0 ) return 1;
@@ -203,6 +216,57 @@ static int dir_conv(const char *a, double *sx, double *sy)
 }
 
 
+static int count_trailing_spaces(const char *string) {
+
+	int i;
+
+	for ( i=0; i<strlen(string); i++ ) {
+		if ( !isspace(string[i]) ) {
+			return i;
+		}
+	}
+
+	return -1;
+}
+
+
+static void build_output_line(const char *line, char *new_line,
+                              const char *string_to_write)
+{
+	int nsp, i, w, ww;
+	int trailsp;
+	int n_bits;
+	char **bits;
+
+	n_bits = assplode(line, "=", &bits, ASSPLODE_NONE);
+	trailsp = count_trailing_spaces(bits[1]);
+
+	strcat(new_line, bits[0]);
+	strcat(new_line, "=");
+
+	for ( nsp=0; nsp < trailsp; nsp++ ) {
+		strcat(new_line, " ");
+	}
+	strcat(new_line, string_to_write);
+
+	for ( w=0; w<strlen(line); w++ ) {
+		if ( strncmp(&line[w],";",1) == 0 ) {
+			for ( ww=w-1; ww>=0; ww-- ) {
+				if ( !isspace(line[ww])) {
+					strcat(new_line, &line[ww]);
+					break;
+				}
+			}
+			break;
+		}
+	}
+	if ( w == strlen(line) ) strcat(new_line, "\n");
+
+	for ( i=0; i<n_bits; i++) free(bits[i]);
+	free(bits);
+}
+
+
 struct rvec get_q_for_panel(struct panel *p, double fs, double ss,
                             double *ttp, double k)
 {
@@ -275,16 +339,30 @@ int in_bad_region(struct detector *det, double fs, double ss)
 
 		struct badregion *b = &det->bad[i];
 
+		if ( (b->panel != NULL)
+		  && (strcmp(b->panel, p->name) != 0) ) continue;
+
 		if ( b->is_fsss ) {
-			if ( fs < b->min_fs ) continue;
-			if ( fs > b->max_fs ) continue;
-			if ( ss < b->min_ss ) continue;
-			if ( ss > b->max_ss ) continue;
+
+			int nfs, nss;
+
+			/* fs/ss bad regions are specified according to the
+			 * original coordinates */
+			nfs = (fs-p->min_fs) + p->orig_min_fs;
+			nss = (ss-p->min_ss) + p->orig_min_ss;
+
+			if ( nfs < b->min_fs ) continue;
+			if ( nfs > b->max_fs ) continue;
+			if ( nss < b->min_ss ) continue;
+			if ( nss > b->max_ss ) continue;
+
 		} else {
+
 			if ( rx < b->min_x ) continue;
 			if ( rx > b->max_x ) continue;
 			if ( ry < b->min_y ) continue;
 			if ( ry > b->max_y ) continue;
+
 		}
 
 		return 1;
@@ -322,8 +400,20 @@ double get_tt(struct image *image, double fs, double ss, int *err)
 }
 
 
-void record_image(struct image *image, int do_poisson, int background,
-                  gsl_rng *rng)
+int detector_has_clen_references(struct detector *det)
+{
+	int i;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+		if ( det->panels[i].clen_from != NULL ) return 1;
+	}
+
+	return 0;
+}
+
+
+void record_image(struct image *image, int do_poisson, double background,
+                  gsl_rng *rng, double beam_radius, double nphotons)
 {
 	int x, y;
 	double total_energy, energy_density;
@@ -338,14 +428,14 @@ void record_image(struct image *image, int do_poisson, int background,
 	int n_nan2 = 0;
 
 	/* How many photons are scattered per electron? */
-	area = M_PI*pow(image->beam->beam_radius, 2.0);
-	total_energy = image->beam->fluence * ph_lambda_to_en(image->lambda);
+	area = M_PI*pow(beam_radius, 2.0);
+	total_energy = nphotons * ph_lambda_to_en(image->lambda);
 	energy_density = total_energy / area;
-	ph_per_e = (image->beam->fluence /area) * pow(THOMSON_LENGTH, 2.0);
+	ph_per_e = (nphotons /area) * pow(THOMSON_LENGTH, 2.0);
 	STATUS("Fluence = %8.2e photons, "
 	       "Energy density = %5.3f kJ/cm^2, "
 	       "Total energy = %5.3f microJ\n",
-	       image->beam->fluence, energy_density/1e7, total_energy*1e6);
+	       nphotons, energy_density/1e7, total_energy*1e6);
 
 	for ( x=0; x<image->width; x++ ) {
 	for ( y=0; y<image->height; y++ ) {
@@ -396,6 +486,9 @@ void record_image(struct image *image, int do_poisson, int background,
 		/* Convert to ADU */
 		dval *= p->adu_per_eV * ph_lambda_to_eV(image->lambda);
 
+		/* Saturation */
+		if ( dval > p->max_adu ) dval = p->max_adu;
+
 		image->data[x + image->width*y] = dval;
 
 		/* Sanity checks */
@@ -461,7 +554,27 @@ struct panel *find_panel(struct detector *det, double fs, double ss)
 }
 
 
-void fill_in_values(struct detector *det, struct hdfile *f)
+/* Like find_panel(), but uses the original panel bounds, i.e. referring to
+ * what's in the HDF5 file */
+struct panel *find_orig_panel(struct detector *det, double fs, double ss)
+{
+	int p;
+
+	for ( p=0; p<det->n_panels; p++ ) {
+		if ( (fs >= det->panels[p].orig_min_fs)
+		  && (fs < det->panels[p].orig_max_fs+1)
+		  && (ss >= det->panels[p].orig_min_ss)
+		  && (ss < det->panels[p].orig_max_ss+1) )
+		{
+			return &det->panels[p];
+		}
+	}
+
+	return NULL;
+}
+
+
+void fill_in_values(struct detector *det, struct hdfile *f, struct event* ev)
 {
 	int i;
 
@@ -470,7 +583,18 @@ void fill_in_values(struct detector *det, struct hdfile *f)
 		struct panel *p = &det->panels[i];
 
 		if ( p->clen_from != NULL ) {
-			p->clen = get_value(f, p->clen_from) * 1.0e-3;
+
+			double val;
+			int r;
+
+			r = hdfile_get_value(f, p->clen_from, ev, &val,
+			                     H5T_NATIVE_DOUBLE);
+			if ( r ) {
+				ERROR("Failed to read '%s'\n", p->clen_from);
+			} else {
+				p->clen = val * 1.0e-3;
+			}
+
 		}
 
 		p->clen += p->coffset;
@@ -479,6 +603,51 @@ void fill_in_values(struct detector *det, struct hdfile *f)
 }
 
 
+int panel_is_in_rigid_group(const struct rigid_group *rg, struct panel *p)
+{
+	int i;
+
+	for ( i=0; i<rg->n_panels; i++ ) {
+		if ( rg->panels[i] == p ) {
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+
+int rigid_group_is_in_collection(struct rg_collection *c,
+                                 struct rigid_group *rg)
+{
+	int i;
+
+	for ( i=0; i<c->n_rigid_groups; i++ ) {
+		if ( c->rigid_groups[i] == rg  ) {
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+
+struct rg_collection *find_rigid_group_collection_by_name(struct detector *det,
+                                                          const char *name)
+{
+	int i;
+
+	for ( i=0; i<det->n_rg_collections; i++ ) {
+		if ( strcmp(det->rigid_group_collections[i]->name,
+		            name) == 0 ) {
+			return det->rigid_group_collections[i];
+		}
+	}
+
+	return NULL;
+}
+
+
 static struct panel *new_panel(struct detector *det, const char *name)
 {
 	struct panel *new;
@@ -496,6 +665,32 @@ static struct panel *new_panel(struct detector *det, const char *name)
 		new->clen_from = strdup(new->clen_from);
 	}
 
+	/* Create a new copy of the data location if needed */
+	if ( new->data != NULL ) {
+		new->data = strdup(new->data);
+	}
+
+	/* Create a new copy of the dim_structure if needed */
+	if ( new->dim_structure != NULL ) {
+
+		struct dim_structure *dim_copy;
+		int di;
+
+		dim_copy = initialize_dim_structure();
+		dim_copy->num_dims = new->dim_structure->num_dims;
+		dim_copy->dims = malloc(dim_copy->num_dims*sizeof(int));
+		for ( di=0; di<dim_copy->num_dims; di++ ) {
+			dim_copy->dims[di] = new->dim_structure->dims[di];
+		}
+
+		new->dim_structure = dim_copy;
+	}
+
+	/* Create a new copy of the bad pixel mask location */
+	if ( new->mask != NULL ) {
+		new->mask = strdup(new->mask);
+	}
+
 	return new;
 }
 
@@ -516,7 +711,8 @@ static struct badregion *new_bad_region(struct detector *det, const char *name)
 	new->max_fs = 0;
 	new->min_ss = 0;
 	new->max_ss = 0;
-	new->is_fsss = 0;
+	new->is_fsss = 99; /* Slightly nasty: means "unassigned" */
+	new->panel = NULL;
 	strcpy(new->name, name);
 
 	return new;
@@ -587,6 +783,39 @@ static struct rigid_group *find_or_add_rg(struct detector *det,
 }
 
 
+static struct rg_collection *find_or_add_rg_coll(struct detector *det,
+                                                 const char *name)
+{
+	int i;
+	struct rg_collection **new;
+	struct rg_collection *rgc;
+
+	for ( i=0; i<det->n_rg_collections; i++ ) {
+		if ( strcmp(det->rigid_group_collections[i]->name, name) == 0 )
+		{
+			return det->rigid_group_collections[i];
+		}
+	}
+
+	new = realloc(det->rigid_group_collections,
+	              (1+det->n_rg_collections)*sizeof(struct rg_collection *));
+	if ( new == NULL ) return NULL;
+
+	det->rigid_group_collections = new;
+
+	rgc = malloc(sizeof(struct rg_collection));
+	if ( rgc == NULL ) return NULL;
+
+	det->rigid_group_collections[det->n_rg_collections++] = rgc;
+
+	rgc->name = strdup(name);
+	rgc->rigid_groups = NULL;
+	rgc->n_rigid_groups = 0;
+
+	return rgc;
+}
+
+
 static void add_to_rigid_group(struct rigid_group *rg, struct panel *p)
 {
 	struct panel **pn;
@@ -597,13 +826,30 @@ static void add_to_rigid_group(struct rigid_group *rg, struct panel *p)
 		return;
 	}
 
-	assert(p->rigid_group == rg);
 	rg->panels = pn;
 	rg->panels[rg->n_panels++] = p;
 }
 
 
-static void rigid_groups_free(struct detector *det)
+static void add_to_rigid_group_coll(struct rg_collection *rgc,
+                                    struct rigid_group *rg)
+{
+	struct rigid_group **r;
+
+	r = realloc(rgc->rigid_groups, (1+rgc->n_rigid_groups)*
+	            sizeof(struct rigid_group *));
+	if ( r == NULL ) {
+		ERROR("Couldn't add rigid group to collection.\n");
+		return;
+	}
+
+	rgc->rigid_groups = r;
+	rgc->rigid_groups[rgc->n_rigid_groups++] = rg;
+}
+
+
+/* Free all rigid groups in detector */
+static void free_all_rigid_groups(struct detector *det)
 {
 	int i;
 
@@ -617,18 +863,33 @@ static void rigid_groups_free(struct detector *det)
 }
 
 
-static void fix_up_rigid_groups(struct detector *det)
+/* Free all rigid groups in detector */
+static void free_all_rigid_group_collections(struct detector *det)
 {
 	int i;
 
-	for ( i=0; i<det->n_panels; i++ ) {
-
-		struct panel *p = &det->panels[i];
-		if ( p->rigid_group != NULL ) {
-			add_to_rigid_group(p->rigid_group, p);
-		}
-
+	if ( det->rigid_group_collections == NULL ) return;
+	for ( i=0; i<det->n_rg_collections; i++ ) {
+		free(det->rigid_group_collections[i]->name);
+		free(det->rigid_group_collections[i]->rigid_groups);
+		free(det->rigid_group_collections[i]);
 	}
+	free(det->rigid_group_collections);
+}
+
+
+static struct rigid_group *find_rigid_group_by_name(struct detector *det,
+                                                    char *name)
+{
+	int i;
+
+	for ( i=0; i<det->n_rigid_groups; i++ ) {
+		if ( strcmp(det->rigid_groups[i]->name, name) == 0 ) {
+			return det->rigid_groups[i];
+		}
+	}
+
+	return NULL;
 }
 
 
@@ -652,7 +913,7 @@ static int parse_field_for_panel(struct panel *panel, const char *key,
 	} else if ( strcmp(key, "adu_per_eV") == 0 ) {
 		panel->adu_per_eV = atof(val);
 	} else if ( strcmp(key, "rigid_group") == 0 ) {
-		panel->rigid_group = find_or_add_rg(det, val);
+		add_to_rigid_group(find_or_add_rg(det, val), panel);
 	} else if ( strcmp(key, "clen") == 0 ) {
 
 		char *end;
@@ -666,6 +927,23 @@ static int parse_field_for_panel(struct panel *panel, const char *key,
 			panel->clen_from = NULL;
 		}
 
+	} else if ( strcmp(key, "data") == 0 ) {
+		if ( strncmp(val,"/",1) != 0 ) {
+			ERROR("Invalid data location '%s'\n", val);
+			reject = -1;
+		}
+		panel->data = strdup(val);
+
+	} else if ( strcmp(key, "mask") == 0 ) {
+		if ( strncmp(val,"/",1) != 0 ) {
+			ERROR("Invalid mask location '%s'\n", val);
+			reject = -1;
+		}
+		panel->mask = strdup(val);
+
+	} else if ( strcmp(key, "mask_file") == 0 ) {
+		panel->mask_file = strdup(val);
+
 	} else if ( strcmp(key, "coffset") == 0) {
 		panel->coffset = atof(val);
 	} else if ( strcmp(key, "res") == 0 ) {
@@ -695,6 +973,11 @@ static int parse_field_for_panel(struct panel *panel, const char *key,
 			ERROR("Invalid slow scan direction '%s'\n", val);
 			reject = 1;
 		}
+	} else if ( strncmp(key, "dim", 3) == 0) {
+		if  ( panel->dim_structure == NULL ) {
+			panel->dim_structure = initialize_dim_structure();
+		}
+		set_dim_structure_entry(panel->dim_structure, key, val);
 	} else {
 		ERROR("Unrecognised field '%s'\n", key);
 	}
@@ -703,43 +986,54 @@ static int parse_field_for_panel(struct panel *panel, const char *key,
 }
 
 
-static int parse_field_bad(struct badregion *panel, const char *key,
-                                  const char *val)
+static int check_badr_fsss(struct badregion *badr, int is_fsss)
+{
+	/* First assignment? */
+	if ( badr->is_fsss == 99 ) {
+		badr->is_fsss = is_fsss;
+		return 0;
+	}
+
+	if ( is_fsss != badr->is_fsss ) {
+		ERROR("You can't mix x/y and fs/ss in a bad region.\n");
+		return 1;
+	}
+
+	return 0;
+}
+
+
+static int parse_field_bad(struct badregion *badr, const char *key,
+                           const char *val)
 {
 	int reject = 0;
 
 	if ( strcmp(key, "min_x") == 0 ) {
-		panel->min_x = atof(val);
-		if ( panel->is_fsss ) {
-			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
-		}
+		badr->min_x = atof(val);
+		reject = check_badr_fsss(badr, 0);
 	} else if ( strcmp(key, "max_x") == 0 ) {
-		panel->max_x = atof(val);
-		if ( panel->is_fsss ) {
-			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
-		}
+		badr->max_x = atof(val);
+		reject = check_badr_fsss(badr, 0);
 	} else if ( strcmp(key, "min_y") == 0 ) {
-		panel->min_y = atof(val);
-		if ( panel->is_fsss ) {
-			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
-		}
+		badr->min_y = atof(val);
+		reject = check_badr_fsss(badr, 0);
 	} else if ( strcmp(key, "max_y") == 0 ) {
-		panel->max_y = atof(val);
-		if ( panel->is_fsss ) {
-			ERROR("You can't mix x/y and fs/ss in a bad region.\n");
-		}
+		badr->max_y = atof(val);
+		reject = check_badr_fsss(badr, 0);
 	} else if ( strcmp(key, "min_fs") == 0 ) {
-		panel->min_fs = atof(val);
-		panel->is_fsss = 1;
+		badr->min_fs = atof(val);
+		reject = check_badr_fsss(badr, 1);
 	} else if ( strcmp(key, "max_fs") == 0 ) {
-		panel->max_fs = atof(val);
-		panel->is_fsss = 1;
+		badr->max_fs = atof(val);
+		reject = check_badr_fsss(badr, 1);
 	} else if ( strcmp(key, "min_ss") == 0 ) {
-		panel->min_ss = atof(val);
-		panel->is_fsss = 1;
+		badr->min_ss = atof(val);
+		reject = check_badr_fsss(badr, 1);
 	} else if ( strcmp(key, "max_ss") == 0 ) {
-		panel->max_ss = atof(val);
-		panel->is_fsss = 1;
+		badr->max_ss = atof(val);
+		reject = check_badr_fsss(badr, 1);
+	} else if ( strcmp(key, "panel") == 0 ) {
+		badr->panel = strdup(val);
 	} else {
 		ERROR("Unrecognised field '%s'\n", key);
 	}
@@ -748,14 +1042,14 @@ static int parse_field_bad(struct badregion *panel, const char *key,
 }
 
 
-static void parse_toplevel(struct detector *det, const char *key,
-                           const char *val)
+static void parse_toplevel(struct detector *det, struct beam_params *beam,
+                           const char *key, const char *val,
+                           struct rg_definition ***rg_defl,
+                           struct rgc_definition ***rgc_defl, int *n_rg_defs,
+                           int *n_rgc_defs)
 {
-	if ( strcmp(key, "mask") == 0 ) {
 
-		det->mask = strdup(val);
-
-	} else if ( strcmp(key, "mask_bad") == 0 ) {
+	if ( strcmp(key, "mask_bad") == 0 ) {
 
 		char *end;
 		double v = strtod(val, &end);
@@ -775,6 +1069,51 @@ static void parse_toplevel(struct detector *det, const char *key,
 
 	} else if ( strcmp(key, "coffset") == 0 ) {
 		det->defaults.coffset = atof(val);
+
+	} else if ( strcmp(key, "photon_energy") == 0 ) {
+		if ( beam != NULL ) {
+			if ( strncmp(val, "/", 1) == 0 ) {
+				beam->photon_energy = 0.0;
+				beam->photon_energy_from = strdup(val);
+			} else {
+				beam->photon_energy = atof(val);
+				beam->photon_energy_from = NULL;
+			}
+		}
+
+	} else if ( strcmp(key, "photon_energy_scale") == 0 ) {
+		if ( beam != NULL ) {
+			beam->photon_energy_scale = atof(val);
+		}
+
+	} else if (strncmp(key, "rigid_group", 11) == 0
+	        && strncmp(key, "rigid_group_collection", 22) != 0 ) {
+
+		struct rg_definition **new;
+
+		new = realloc(*rg_defl,
+		             ((*n_rg_defs)+1)*sizeof(struct rg_definition*));
+		*rg_defl = new;
+
+		(*rg_defl)[*n_rg_defs] = malloc(sizeof(struct rg_definition));
+		(*rg_defl)[*n_rg_defs]->name = strdup(key+12);
+		(*rg_defl)[*n_rg_defs]->pns = strdup(val);
+		*n_rg_defs = *n_rg_defs+1;
+
+	} else if ( strncmp(key, "rigid_group_collection", 22) == 0 ) {
+
+		struct rgc_definition **new;
+
+		new = realloc(*rgc_defl, ((*n_rgc_defs)+1)*
+		      sizeof(struct rgc_definition*));
+		*rgc_defl = new;
+
+		(*rgc_defl)[*n_rgc_defs] =
+		                   malloc(sizeof(struct rgc_definition));
+		(*rgc_defl)[*n_rgc_defs]->name = strdup(key+23);
+		(*rgc_defl)[*n_rgc_defs]->rgs = strdup(val);
+		*n_rgc_defs = *n_rgc_defs+1;
+
 	} else if ( parse_field_for_panel(&det->defaults, key, val, det) ) {
 		ERROR("Unrecognised top level field '%s'\n", key);
 	}
@@ -839,15 +1178,26 @@ static void find_min_max_d(struct detector *det)
 }
 
 
-struct detector *get_detector_geometry(const char *filename)
+struct detector *get_detector_geometry(const char *filename,
+                                       struct beam_params *beam)
 {
 	FILE *fh;
 	struct detector *det;
 	char *rval;
 	char **bits;
 	int i;
+	int rgi, rgci;
 	int reject = 0;
+	int path_dim;
+	int dim_dim;
+	int curr_ss;
 	int x, y, max_fs, max_ss;
+	int dim_reject = 0;
+	int dim_dim_reject = 0;
+	struct rg_definition **rg_defl = NULL;
+	struct rgc_definition **rgc_defl = NULL;
+	int n_rg_definitions = 0;
+	int n_rgc_definitions = 0;
 
 	fh = fopen(filename, "r");
 	if ( fh == NULL ) return NULL;
@@ -858,24 +1208,37 @@ struct detector *get_detector_geometry(const char *filename)
 		return NULL;
 	}
 
+	if ( beam != NULL ) {
+		beam->photon_energy = 0.0;
+		beam->photon_energy_from = NULL;
+		beam->photon_energy_scale = 1.0;
+	}
+
 	det->n_panels = 0;
 	det->panels = NULL;
 	det->n_bad = 0;
 	det->bad = NULL;
 	det->mask_good = 0;
 	det->mask_bad = 0;
-	det->mask = NULL;
 	det->n_rigid_groups = 0;
 	det->rigid_groups = NULL;
+	det->path_dim = 0;
+	det->dim_dim = 0;
+	det->n_rg_collections = 0;
+	det->rigid_group_collections = NULL;
 
 	/* The default defaults... */
 	det->defaults.min_fs = -1;
 	det->defaults.min_ss = -1;
 	det->defaults.max_fs = -1;
 	det->defaults.max_ss = -1;
+	det->defaults.orig_min_fs = -1;
+	det->defaults.orig_min_ss = -1;
+	det->defaults.orig_max_fs = -1;
+	det->defaults.orig_max_ss = -1;
 	det->defaults.cnx = NAN;
 	det->defaults.cny = NAN;
-	det->defaults.clen = -1.0;
+	det->defaults.clen = NAN;
 	det->defaults.coffset = 0.0;
 	det->defaults.res = -1.0;
 	det->defaults.badrow = '-';
@@ -884,9 +1247,12 @@ struct detector *get_detector_geometry(const char *filename)
 	det->defaults.fsy = 0.0;
 	det->defaults.ssx = 0.0;
 	det->defaults.ssy = 1.0;
-	det->defaults.rigid_group = NULL;
 	det->defaults.adu_per_eV = NAN;
 	det->defaults.max_adu = +INFINITY;
+	det->defaults.mask = NULL;
+	det->defaults.mask_file = NULL;
+	det->defaults.data = NULL;
+	det->defaults.dim_structure = NULL;
 	strncpy(det->defaults.name, "", 1023);
 
 	do {
@@ -926,8 +1292,11 @@ struct detector *get_detector_geometry(const char *filename)
 
 		n2 = assplode(bits[0], "/\\.", &path, ASSPLODE_NONE);
 		if ( n2 < 2 ) {
+
 			/* This was a top-level option, not handled above. */
-			parse_toplevel(det, bits[0], bits[2]);
+			parse_toplevel(det, beam, bits[0], bits[2], &rg_defl,
+			               &rgc_defl, &n_rg_definitions,
+				       &n_rgc_definitions);
 			for ( i=0; i<n1; i++ ) free(bits[i]);
 			free(bits);
 			for ( i=0; i<n2; i++ ) free(path[i]);
@@ -975,9 +1344,145 @@ struct detector *get_detector_geometry(const char *filename)
 
 	max_fs = 0;
 	max_ss = 0;
+
+	path_dim = -1;
+	dim_reject = 0;
+
 	for ( i=0; i<det->n_panels; i++ ) {
 
-		if ( det->panels[i].min_fs < 0 ) {
+		int panel_dim = 0;
+		char *next_instance;
+
+		next_instance = det->panels[i].data;
+
+		while ( next_instance ) {
+			next_instance = strstr(next_instance, "%");
+			if ( next_instance != NULL ) {
+				next_instance += 1*sizeof(char);
+				panel_dim += 1;
+			}
+		}
+
+		if ( path_dim == -1 ) {
+			path_dim = panel_dim;
+		} else {
+			if ( panel_dim != path_dim ) {
+				dim_reject = 1;
+			}
+		}
+
+	}
+
+	for ( i=0; i<det->n_panels; i++ ) {
+
+		int panel_mask_dim = 0;
+		char *next_instance;
+
+		if ( det->panels[i].mask != NULL ) {
+
+			next_instance = det->panels[i].mask;
+
+			while(next_instance)
+			{
+				next_instance = strstr(next_instance, "%");
+				if ( next_instance != NULL ) {
+					next_instance += 1*sizeof(char);
+					panel_mask_dim += 1;
+				}
+			}
+
+			if ( panel_mask_dim != path_dim ) {
+				dim_reject = 1;
+			}
+		}
+	}
+
+	if ( dim_reject ==  1) {
+		ERROR("All panels' data and mask entries must have the same "
+		      "number of placeholders\n");
+		reject = 1;
+	}
+
+	det->path_dim = path_dim;
+
+	dim_dim_reject = 0;
+	dim_dim = -1;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+
+		int di;
+		int found_ss = 0;
+		int found_fs = 0;
+		int panel_dim_dim = 0;
+
+		if ( det->panels[i].dim_structure == NULL ) {
+			det->panels[i].dim_structure = default_dim_structure();
+		}
+
+		for ( di=0; di<det->panels[i].dim_structure->num_dims; di++ ) {
+
+			if ( det->panels[i].dim_structure->dims[di] ==
+			                                   HYSL_UNDEFINED  ) {
+				dim_dim_reject = 1;
+			}
+			if ( det->panels[i].dim_structure->dims[di] ==
+			                                   HYSL_PLACEHOLDER  ) {
+				panel_dim_dim += 1;
+			}
+			if ( det->panels[i].dim_structure->dims[di] ==
+			                                   HYSL_SS  ) {
+				found_ss += 1;
+			}
+			if ( det->panels[i].dim_structure->dims[di] ==
+			                                   HYSL_FS  ) {
+				found_fs += 1;
+			}
+
+		}
+
+		if ( found_ss != 1 ) {
+			ERROR("Only one slow scan dim coordinate is allowed\n");
+			dim_dim_reject = 1;
+		}
+
+		if ( found_fs != 1 ) {
+			ERROR("Only one fast scan dim coordinate is allowed\n");
+			dim_dim_reject = 1;
+		}
+
+		if ( panel_dim_dim > 1 ) {
+			ERROR("Maximum one placeholder dim coordinate is "
+			      "allowed\n");
+			dim_dim_reject = 1;
+		}
+
+		if ( dim_dim == -1 ) {
+			dim_dim = panel_dim_dim;
+		} else {
+			if ( panel_dim_dim != dim_dim ) {
+				dim_dim_reject = 1;
+			}
+		}
+
+	}
+
+	if ( dim_dim_reject ==  1) {
+		reject = 1;
+	}
+
+	det->dim_dim = dim_dim;
+
+	curr_ss = 0;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+
+		if ( det->panels[i].max_fs-det->panels[i].min_fs+1 !=
+			det->panels[0].max_fs-det->panels[0].min_fs+1 ) {
+			ERROR("All panels should have the same fs extent\n");
+			reject = 1;
+		}
+
+		if ( det->panels[i ].min_fs < 0 ) {
 			ERROR("Please specify the minimum FS coordinate for"
 			      " panel %s\n", det->panels[i].name);
 			reject = 1;
@@ -1007,7 +1512,7 @@ struct detector *get_detector_geometry(const char *filename)
 			      " panel %s\n", det->panels[i].name);
 			reject = 1;
 		}
-		if ( (det->panels[i].clen < 0.0)
+		if ( isnan(det->panels[i].clen)
 		  && (det->panels[i].clen_from == NULL) ) {
 			ERROR("Please specify the camera length for"
 			      " panel %s\n", det->panels[i].name);
@@ -1023,10 +1528,27 @@ struct detector *get_detector_geometry(const char *filename)
 			      " panel %s\n", det->panels[i].name);
 			reject = 1;
 		}
+
 		/* It's OK if the badrow direction is '0' */
 		/* It's not a problem if "no_index" is still zero */
 		/* The default transformation matrix is at least valid */
 
+		det->panels[i].orig_max_fs = det->panels[i].max_fs;
+		det->panels[i].orig_min_fs = det->panels[i].min_fs;
+		det->panels[i].orig_max_ss = det->panels[i].max_ss;
+		det->panels[i].orig_min_ss = det->panels[i].min_ss;
+
+		det->panels[i].w =
+		                 det->panels[i].max_fs-det->panels[i].min_fs+1;
+		det->panels[i].h =
+		                 det->panels[i].max_ss-det->panels[i].min_ss+1;
+
+		det->panels[i].min_fs = 0;
+		det->panels[i].max_fs = det->panels[i].w-1;
+		det->panels[i].min_ss = curr_ss;
+		det->panels[i].max_ss = curr_ss+det->panels[i].h-1;
+		curr_ss += det->panels[i].h;
+
 		if ( det->panels[i].max_fs > max_fs ) {
 			max_fs = det->panels[i].max_fs;
 		}
@@ -1037,24 +1559,8 @@ struct detector *get_detector_geometry(const char *filename)
 	}
 
 	for ( i=0; i<det->n_bad; i++ ) {
-
-		if ( !det->bad[i].is_fsss && isnan(det->bad[i].min_x) ) {
-			ERROR("Please specify the minimum x coordinate for"
-			      " bad region %s\n", det->bad[i].name);
-			reject = 1;
-		}
-		if ( !det->bad[i].is_fsss && isnan(det->bad[i].min_y) ) {
-			ERROR("Please specify the minimum y coordinate for"
-			      " bad region %s\n", det->bad[i].name);
-			reject = 1;
-		}
-		if ( !det->bad[i].is_fsss && isnan(det->bad[i].max_x) ) {
-			ERROR("Please specify the maximum x coordinate for"
-			      " bad region %s\n", det->bad[i].name);
-			reject = 1;
-		}
-		if ( !det->bad[i].is_fsss && isnan(det->bad[i].max_y) ) {
-			ERROR("Please specify the maximum y coordinate for"
+		if ( det->bad[i].is_fsss == 99 ) {
+			ERROR("Please specify the coordinate ranges for"
 			      " bad region %s\n", det->bad[i].name);
 			reject = 1;
 		}
@@ -1075,6 +1581,96 @@ out:
 	det->max_ss = max_ss;
 
 	free(det->defaults.clen_from);
+	free(det->defaults.data);
+	free(det->defaults.mask);
+
+	for ( rgi=0; rgi<n_rg_definitions; rgi++) {
+
+		int pi, n1;
+		struct rigid_group *rigidgroup = NULL;
+
+		rigidgroup = find_or_add_rg(det, rg_defl[rgi]->name);
+
+		n1 = assplode(rg_defl[rgi]->pns, ",", &bits, ASSPLODE_NONE);
+
+		for ( pi=0; pi<n1; pi++ ) {
+
+			struct panel *p;
+
+			p = find_panel_by_name(det, bits[pi]);
+			if ( p == NULL ) {
+				ERROR("Cannot add panel to rigid group\n");
+				ERROR("Panel not found: %s\n", bits[pi]);
+				return NULL;
+			}
+			add_to_rigid_group(rigidgroup, p);
+			free(bits[pi]);
+		}
+		free(bits);
+		free(rg_defl[rgi]->name);
+		free(rg_defl[rgi]->pns);
+		free(rg_defl[rgi]);
+	}
+	free(rg_defl);
+
+	for ( rgci=0; rgci<n_rgc_definitions; rgci++ ) {
+
+		int rgi, n2;
+		struct rg_collection *rgcollection = NULL;
+
+		rgcollection = find_or_add_rg_coll(det, rgc_defl[rgci]->name);
+
+		n2 = assplode(rgc_defl[rgci]->rgs, ",", &bits, ASSPLODE_NONE);
+
+		for ( rgi=0; rgi<n2; rgi++ ) {
+
+			struct rigid_group *r;
+
+			r = find_rigid_group_by_name(det, bits[rgi]);
+			if ( r == NULL ) {
+				ERROR("Cannot add rigid group to collection\n");
+				ERROR("Rigid group not found: %s\n", bits[rgi]);
+				return NULL;
+			}
+			add_to_rigid_group_coll(rgcollection, r);
+			free(bits[rgi]);
+		}
+		free(bits);
+		free(rgc_defl[rgci]->name);
+		free(rgc_defl[rgci]->rgs);
+		free(rgc_defl[rgci]);
+
+	}
+	free(rgc_defl);
+
+	if ( n_rg_definitions == 0 ) {
+
+		int pi;
+
+		for ( pi=0; pi<det->n_panels; pi++ ) {
+
+			struct rigid_group *rigidgroup = NULL;
+
+			rigidgroup = find_or_add_rg(det, det->panels[pi].name);
+			add_to_rigid_group(rigidgroup, &det->panels[pi]);
+
+		}
+	}
+
+	if ( n_rgc_definitions == 0 ) {
+
+		int rgi;
+		struct rg_collection *rgcollection = NULL;
+
+		rgcollection = find_or_add_rg_coll(det, "default");
+
+		for ( rgi=0; rgi<det->n_rigid_groups; rgi++ ) {
+
+			add_to_rigid_group_coll(rgcollection,
+			                        det->rigid_groups[rgi]);
+
+		}
+	}
 
 	/* Calculate matrix inverses and other stuff */
 	for ( i=0; i<det->n_panels; i++ ) {
@@ -1086,7 +1682,6 @@ out:
 
 		if ( p->fsx*p->ssy == p->ssx*p->fsy ) {
 			ERROR("Panel %i transformation singular.\n", i);
-			reject = 1;
 		}
 
 		d = (double)p->fsx*p->ssy - p->ssx*p->fsy;
@@ -1098,16 +1693,8 @@ out:
 		p->w = p->max_fs - p->min_fs + 1;
 		p->h = p->max_ss - p->min_ss + 1;
 
-		if ( p->rigid_group == NULL ) {
-			p->rigid_group = find_or_add_rg(det, p->name);
-		}
-
 	}
 
-	/* Fix up rigid group panel pointers now that the panels and RGs have
-	 * stopped being realloc()ed */
-	fix_up_rigid_groups(det);
-
 	find_min_max_d(det);
 
 	if ( reject ) return NULL;
@@ -1122,15 +1709,16 @@ void free_detector_geometry(struct detector *det)
 {
 	int i;
 
-	rigid_groups_free(det);
+	free_all_rigid_groups(det);
+	free_all_rigid_group_collections(det);
 
 	for ( i=0; i<det->n_panels; i++ ) {
 		free(det->panels[i].clen_from);
+		free_dim_structure(det->panels[i].dim_structure);
 	}
 
 	free(det->panels);
 	free(det->bad);
-	free(det->mask);
 	free(det);
 }
 
@@ -1143,12 +1731,6 @@ struct detector *copy_geom(const struct detector *in)
 	out = malloc(sizeof(struct detector));
 	memcpy(out, in, sizeof(struct detector));
 
-	if ( in->mask != NULL ) {
-		out->mask = strdup(in->mask);
-	} else {
-		out->mask = NULL;  /* = in->mask */
-	}
-
 	out->panels = malloc(out->n_panels * sizeof(struct panel));
 	memcpy(out->panels, in->panels, out->n_panels * sizeof(struct panel));
 
@@ -1157,6 +1739,8 @@ struct detector *copy_geom(const struct detector *in)
 
 	out->n_rigid_groups = 0;
 	out->rigid_groups = NULL;
+	out->n_rg_collections = 0;
+	out->rigid_group_collections = NULL;
 
 	for ( i=0; i<out->n_panels; i++ ) {
 
@@ -1170,20 +1754,77 @@ struct detector *copy_geom(const struct detector *in)
 			p->clen_from = strdup(p->clen_from);
 		}
 
+		if ( p->data != NULL ) {
+			/* Make a copy of the data fields unique to this
+			 * copy of the structure. */
+			p->data = strdup(p->data);
+		}
+
+		if ( p->dim_structure != NULL ) {
+			/* Make a copy of the dim_structure fields unique to this
+			 * copy of the structure. */
+
+			struct dim_structure *dim_new;
+			int di;
+
+			dim_new = initialize_dim_structure();
+			dim_new->num_dims = p->dim_structure->num_dims;
+			dim_new->dims = malloc(dim_new->num_dims*sizeof(int));
+			for ( di=0; di<dim_new->num_dims; di++ ) {
+				dim_new->dims[di] = p->dim_structure->dims[di];
+			}
+
+			p->dim_structure = dim_new;
+
+		}
+
 	}
 
 	for ( i=0; i<in->n_panels; i++ ) {
 
-		struct rigid_group *rg;
+		int rgi;
 
-		rg = in->panels[i].rigid_group;
-		if ( rg == NULL ) continue;
+		for ( rgi=0; rgi<in->n_rigid_groups; rgi++ ) {
+
+			if ( panel_is_in_rigid_group(in->rigid_groups[rgi],
+			                             &in->panels[i]) )
+			{
+				struct rigid_group *g;
+				g = find_or_add_rg(out,
+				                   in->rigid_groups[rgi]->name);
+				add_to_rigid_group(g, &out->panels[i]);
+			}
+		}
+
+		if ( &in->panels[i] == in->furthest_out_panel ) {
+			out->furthest_out_panel = &out->panels[i];
+		}
+		if ( &in->panels[i] == in->furthest_in_panel ) {
+			out->furthest_in_panel = &out->panels[i];
+		}
 
-		out->panels[i].rigid_group = find_or_add_rg(out, rg->name);
 
 	}
 
-	fix_up_rigid_groups(out);
+	for ( i=0; i<in->n_rigid_groups; i++ ) {
+
+		int rgci;
+
+		for ( rgci=0; rgci<in->n_rg_collections; rgci++ ) {
+
+			const char *n = in->rigid_group_collections[rgci]->name;
+
+			if ( rigid_group_is_in_collection(
+			                      in->rigid_group_collections[rgci],
+			                      in->rigid_groups[i]) )
+			{
+				struct rg_collection *rgcoll;
+				rgcoll = find_or_add_rg_coll(out, n);
+				add_to_rigid_group_coll(rgcoll,
+				                        out->rigid_groups[i]);
+			}
+		}
+	}
 
 	return out;
 }
@@ -1202,10 +1843,17 @@ struct detector *simple_geometry(const struct image *image)
 	geom->panels[0].max_fs = image->width-1;
 	geom->panels[0].min_ss = 0;
 	geom->panels[0].max_ss = image->height-1;
+	geom->panels[0].orig_min_fs = 0;
+	geom->panels[0].orig_max_fs = image->width-1;
+	geom->panels[0].orig_min_ss = 0;
+	geom->panels[0].orig_max_ss = image->height-1;
 	geom->panels[0].cnx = -image->width / 2.0;
 	geom->panels[0].cny = -image->height / 2.0;
-	geom->panels[0].rigid_group = NULL;
 	geom->panels[0].max_adu = INFINITY;
+	geom->panels[0].orig_min_fs = -1;
+	geom->panels[0].orig_max_fs = -1;
+	geom->panels[0].orig_min_ss = -1;
+	geom->panels[0].orig_max_ss = -1;
 
 	geom->panels[0].fsx = 1;
 	geom->panels[0].fsy = 0;
@@ -1220,25 +1868,15 @@ struct detector *simple_geometry(const struct image *image)
 	geom->panels[0].w = image->width;
 	geom->panels[0].h = image->height;
 
+	geom->panels[0].mask = NULL;
+	geom->panels[0].data = NULL;
+
 	find_min_max_d(geom);
 
 	return geom;
 }
 
 
-void twod_mapping(double fs, double ss, double *px, double *py,
-                  struct panel *p)
-{
-	double xs, ys;
-
-	xs = fs*p->fsx + ss*p->ssx;
-	ys = fs*p->fsy + ss*p->ssy;
-
-	*px = xs + p->cnx;
-	*py = ys + p->cny;
-}
-
-
 int reverse_2d_mapping(double x, double y, double *pfs, double *pss,
                        struct detector *det)
 {
@@ -1291,6 +1929,72 @@ static void check_extents(struct panel p, double *min_x, double *min_y,
 }
 
 
+static void process_panel_fields(const struct panel *p, char *line,
+                                     FILE *fh, char **bits,
+				     int write_panel_coffset)
+{
+	char new_line[1024];
+	char string_to_write[512];
+
+	strcpy(new_line,"\0");
+	strcpy(string_to_write,"\0");
+
+	if(strstr(bits[1], "fs") != NULL &&
+	   strstr(bits[1], "min_fs") == NULL &&
+	   strstr(bits[1], "max_fs") == NULL &&
+	   strstr(bits[1], "offset") == NULL   ) {
+
+		sprintf(string_to_write, "%+fx %+fy",
+			p->fsx, p->fsy);
+		build_output_line(line, new_line,
+		                  string_to_write);
+		fputs(new_line, fh);
+		return;
+
+	} else if ( strstr(bits[1], "ss") != NULL &&
+		    strstr(bits[1], "min_ss") == NULL &&
+		    strstr(bits[1], "max_ss") == NULL) {
+
+		sprintf(string_to_write, "%+fx %+fy",
+			p->ssx, p->ssy);
+		build_output_line(line, new_line,
+		                  string_to_write);
+		fputs(new_line, fh);
+		return;
+
+	} else if ( strstr(bits[1], "corner_x") != NULL) {
+
+		sprintf(string_to_write, "%g",
+			p->cnx);
+		build_output_line(line, new_line,
+		                  string_to_write);
+		fputs(new_line, fh);
+		return;
+
+	} else if ( strstr(bits[1], "corner_y") != NULL) {
+
+		sprintf(string_to_write, "%g",
+			p->cny);
+		build_output_line(line, new_line,
+		                  string_to_write);
+		fputs(new_line, fh);
+		return;
+
+	} else if ( strstr(bits[1], "coffset") != NULL) {
+
+		if ( write_panel_coffset ) {
+			return;
+		} else {
+			fputs(line, fh);
+			return;
+		}
+
+	} else {
+		fputs(line, fh);
+	}
+}
+
+
 double largest_q(struct image *image)
 {
 	struct rvec q;
@@ -1357,56 +2061,118 @@ void get_pixel_extents(struct detector *det,
 }
 
 
-int write_detector_geometry(const char *filename, struct detector *det)
+int write_detector_geometry_2(const char *geometry_filename,
+                              const char *output_filename, struct detector *det,
+                              const char *additional_comment,
+                              int write_panel_coffset)
 {
-	struct panel *p;
-	int pi;
+	FILE *ifh;
 	FILE *fh;
 
-	if ( filename == NULL ) return 2;
+	if ( geometry_filename == NULL ) return 2;
+	if ( output_filename == NULL ) return 2;
 	if ( det->n_panels < 1 ) return 3;
 
-	fh = fopen(filename, "w");
-	if ( fh == NULL ) return 1;
+	ifh = fopen(geometry_filename, "r");
+	if ( ifh == NULL ) return 1;
 
-	for ( pi=0; pi<det->n_panels; pi++) {
+	fh = fopen(output_filename, "w");
+	if ( fh == NULL ) {
+		return 1;
+	}
 
-		p = &(det->panels[pi]);
+	if ( additional_comment != NULL ) {
 
-		if ( p == NULL ) return 4;
+		char additional_comment_tmp[1024];
 
-		if ( pi > 0 ) fprintf(fh, "\n");
-
-		fprintf(fh, "%s/min_fs = %d\n", p->name, p->min_fs);
-		fprintf(fh, "%s/min_ss = %d\n", p->name, p->min_ss);
-		fprintf(fh, "%s/max_fs = %d\n", p->name, p->max_fs);
-		fprintf(fh, "%s/max_ss = %d\n", p->name, p->max_ss);
-		fprintf(fh, "%s/badrow_direction = %C\n", p->name, p->badrow);
-		fprintf(fh, "%s/res = %g\n", p->name, p->res);
-		fprintf(fh, "%s/clen = %s\n", p->name, p->clen_from);
-		fprintf(fh, "%s/fs = %+fx %+fy\n", p->name, p->fsx, p->fsy);
-		fprintf(fh, "%s/ss = %+fx %+fy\n", p->name, p->ssx, p->ssy);
-		fprintf(fh, "%s/corner_x = %g\n", p->name, p->cnx);
-		fprintf(fh, "%s/corner_y = %g\n", p->name, p->cny);
-		fprintf(fh, "%s/adu_per_eV = %g\n", p->name, p->adu_per_eV);
-		fprintf(fh, "%s/max_adu = %g\n", p->name, p->max_adu);
-
-		if ( p->no_index ) {
-			fprintf(fh, "%s/no_index = 1\n", p->name);
-		} /* else don't clutter up the file */
-
-		if ( p->rigid_group != NULL ) {
-			fprintf(fh, "%s/rigid_group = %s\n",
-			        p->name, p->rigid_group->name);
+		strcpy(additional_comment_tmp, "; ");
+		strncat(additional_comment_tmp, additional_comment, 1021);
+		strcat(additional_comment_tmp, "\n");
+		if ( strlen(additional_comment) > 1021 ) {
+			strncpy(&additional_comment_tmp[1023], "\0", 1);
 		}
+		fputs(additional_comment_tmp, fh);
+	}
+	if  ( write_panel_coffset ) {
+
+		char new_line[1024];
+		strcpy(new_line,"; Optimized panel offsets can be found at the "
+		                "end of the file\n");
+		fputs(new_line, fh);
 
 	}
+
+	do {
+
+		char *rval;
+		char line[1024];
+
+		int n_bits;
+		char **bits;
+
+		int i;
+		struct panel *p;
+
+		rval = fgets(line, 1023, ifh);
+		if ( rval == NULL ) break;
+
+		n_bits = assplode(line, "/=", &bits, ASSPLODE_NONE);
+
+		if ( n_bits != 3 ) {
+			if ( strstr(bits[0], "coffset" ) != NULL &&
+			     write_panel_coffset ) {
+				continue;
+			} else {
+				fputs(line, fh);
+			}
+		} else {
+
+			p = find_panel_by_name(det, bits[0]);
+
+			if ( p != NULL ) {
+				process_panel_fields(p, line, fh, bits,
+						     write_panel_coffset);
+
+			} else {
+				fputs(line, fh);
+			}
+		}
+
+		for ( i=0; i<n_bits; i++ ) free(bits[i]);
+
+	} while ( 1 );
+
+	if ( write_panel_coffset ) {
+
+		int pi;
+		char new_line[1024];
+
+		strcpy(new_line, "\n");
+		fputs(new_line, fh);
+
+		for ( pi=0; pi<det->n_panels; pi++ ) {
+			sprintf(new_line, "%s/coffset = %f\n",
+			        det->panels[pi].name, det->panels[pi].coffset);
+			fputs(new_line, fh);
+		}
+	}
+
+
+	fclose(ifh);
 	fclose(fh);
 
 	return 0;
 }
 
 
+int write_detector_geometry(const char *geometry_filename,
+                            const char *output_filename, struct detector *det)
+{
+	return write_detector_geometry_2(geometry_filename, output_filename,
+	                                 det, NULL, 0);
+}
+
+
 /**
  * mark_resolution_range_as_bad:
  * @image: An image structure
@@ -1442,3 +2208,43 @@ void mark_resolution_range_as_bad(struct image *image,
 
 	}
 }
+
+
+extern int single_panel_data_source(struct detector *det, const char *element)
+{
+	int pi;
+	char *first_datafrom = NULL;
+	char *curr_datafrom = NULL;
+
+	if ( det->panels[0].data == NULL ) {
+		if ( element != NULL ) {
+			first_datafrom = strdup(element);
+		} else {
+			first_datafrom = strdup("/data/data");
+		}
+	} else {
+		first_datafrom = strdup(det->panels[0].data);
+	}
+
+	for ( pi=1;pi<det->n_panels;pi++ ) {
+
+		if ( det->panels[pi].data == NULL ) {
+			if ( element != NULL ) {
+				curr_datafrom = strdup(element);
+			} else {
+				curr_datafrom = strdup("/data/data");
+			}
+		} else {
+			curr_datafrom = strdup(det->panels[pi].data);
+		}
+
+		if ( strcmp(curr_datafrom, first_datafrom) != 0 ) {
+			return 0;
+		}
+	}
+
+	free(first_datafrom);
+	free(curr_datafrom);
+
+	return 1;
+}
