@@ -7,7 +7,9 @@
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2009-2014 Thomas White <taw@physics.org>
+ *   2009-2013 Thomas White <taw@physics.org>
+ *   2014      Valerio Mariani
+ *
  *
  * This file is part of CrystFEL.
  *
@@ -33,6 +35,8 @@
 #ifndef IMAGE_H
 #define IMAGE_H
 
+struct detector;
+
 #include <stdint.h>
 #include <complex.h>
 #include <sys/types.h>
@@ -69,6 +73,7 @@ struct imagefeature {
 	struct image                    *parent;
 	double                          fs;
 	double                          ss;
+	char                            *pn;
 	double                          intensity;
 
 	/* Reciprocal space coordinates (m^-1 of course) of this feature */
@@ -93,6 +98,15 @@ struct sample
 };
 
 
+struct beam_params
+{
+	double photon_energy;  /* eV per photon */
+	char  *photon_energy_from; /* HDF5 dataset name */
+	double photon_energy_scale;  /* Scale factor for photon energy, if the
+	                              * energy is to be from the HDF5 file */
+};
+
+
 /**
  * image:
  *
@@ -164,10 +178,14 @@ struct image {
 	struct detector         *det;
 	struct beam_params      *beam;  /* The nominal beam parameters */
 	char                    *filename;
+	struct event            *event;
 	const struct copy_hdf5_field *copyme;
+	struct stuff_from_stream *stuff_from_stream;
 
 	int                     id;   /* ID number of the thread
 	                               * handling this image */
+	int                     serial;  /* Monotonically ascending serial
+	                                  * number for this image */
 
 	struct sample           *spectrum;
 	int                     nsamples; /* Number of wavelengths */
@@ -207,6 +225,10 @@ extern struct imagefeature *image_feature_closest(ImageFeatureList *flist,
                                                   double fs, double ss,
                                                   double *d, int *idx,
                                                   struct detector *det);
+extern Reflection *image_reflection_closest(RefList *rlist,
+                                            double fs, double ss,
+                                            struct detector *det,
+                                            double *d);
 
 extern int image_feature_count(ImageFeatureList *flist);
 extern struct imagefeature *image_get_feature(ImageFeatureList *flist, int idx);
