@@ -54,94 +54,6 @@
 #include "integration.h"
 
 
-static void check_eigen(gsl_vector *e_val)
-{
-	int i;
-	double vmax, vmin;
-	const int n = e_val->size;
-	const double max_condition = 1e6;
-	const int verbose = 0;
-
-	if ( verbose ) STATUS("Eigenvalues:\n");
-	vmin = +INFINITY;
-	vmax = 0.0;
-	for ( i=0; i<n; i++ ) {
-		double val = gsl_vector_get(e_val, i);
-		if ( verbose ) STATUS("%i: %e\n", i, val);
-		if ( val > vmax ) vmax = val;
-		if ( val < vmin ) vmin = val;
-	}
-
-	for ( i=0; i<n; i++ ) {
-		double val = gsl_vector_get(e_val, i);
-		if ( val < vmax/max_condition ) {
-			gsl_vector_set(e_val, i, 0.0);
-		}
-	}
-
-	vmin = +INFINITY;
-	vmax = 0.0;
-	for ( i=0; i<n; i++ ) {
-		double val = gsl_vector_get(e_val, i);
-		if ( val == 0.0 ) continue;
-		if ( val > vmax ) vmax = val;
-		if ( val < vmin ) vmin = val;
-	}
-	if ( verbose ) {
-		STATUS("Condition number: %e / %e = %5.2f\n",
-		       vmax, vmin, vmax/vmin);
-	}
-}
-
-
-static gsl_vector *solve_svd(gsl_vector *v, gsl_matrix *Mp)
-{
-	gsl_matrix *s_vec;
-	gsl_vector *s_val;
-	int err, n;
-	gsl_vector *shifts;
-	gsl_matrix *M;
-
-	n = v->size;
-	if ( v->size != Mp->size1 ) return NULL;
-	if ( v->size != Mp->size2 ) return NULL;
-
-	M = gsl_matrix_alloc(n, n);
-	if ( M == NULL ) return NULL;
-	gsl_matrix_memcpy(M, Mp);
-
-	s_val = gsl_vector_calloc(n);
-	s_vec = gsl_matrix_calloc(n, n);
-
-	err = gsl_linalg_SV_decomp_jacobi(M, s_vec, s_val);
-	if ( err ) {
-		ERROR("SVD failed: %s\n", gsl_strerror(err));
-		gsl_matrix_free(s_vec);
-		gsl_vector_free(s_val);
-		return NULL;
-	}
-	/* "M" is now "U" */
-
-	check_eigen(s_val);
-
-	shifts = gsl_vector_calloc(n);
-	err = gsl_linalg_SV_solve(M, s_vec, s_val, v, shifts);
-	if ( err ) {
-		ERROR("Matrix solution failed: %s\n", gsl_strerror(err));
-		gsl_matrix_free(s_vec);
-		gsl_vector_free(s_val);
-		gsl_vector_free(shifts);
-		return NULL;
-	}
-
-	gsl_matrix_free(s_vec);
-	gsl_vector_free(s_val);
-	gsl_matrix_free(M);
-
-	return shifts;
-}
-
-
 enum boxmask_val
 {
 	BM_IG,  /* "Soft" ignore */
@@ -340,14 +252,14 @@ static void show_reference_profile(struct intcontext *ic, int i)
 
 
 static void show_peak_box(struct intcontext *ic, struct peak_box *bx,
-                          int results_pipe)
+                          pthread_mutex_t *term_lock)
 {
 #ifdef HAVE_CURSES_COLOR
 	int q;
 	signed int h, k, l;
 	double fs, ss;
 
-	if ( results_pipe != 0 ) write(results_pipe, "SUSPEND\n", 8);
+	if ( term_lock != NULL ) pthread_mutex_lock(term_lock);
 
 	initscr();
 	clear();
@@ -358,8 +270,11 @@ static void show_peak_box(struct intcontext *ic, struct peak_box *bx,
 
 	get_indices(bx->refl, &h, &k, &l);
 	get_detector_pos(bx->refl, &fs, &ss);
-	printw("Indices %i %i %i\nPosition fs = %.1f, ss = %.1f\n\n",
-	       h, k, l, fs, ss);
+	/* Convert coordinates to match arrangement of panels in HDF5 file */
+	fs = fs - bx->p->min_fs + bx->p->orig_min_fs;
+	ss = ss - bx->p->min_ss + bx->p->orig_min_ss;
+	printw("Indices %i %i %i\nPanel %s\nPosition fs = %.1f, ss = %.1f\n\n",
+	       h, k, l, bx->p->name, fs, ss);
 
 	printw("Pixel values:\n");
 	for ( q=ic->w-1; q>=0; q-- ) {
@@ -408,7 +323,10 @@ static void show_peak_box(struct intcontext *ic, struct peak_box *bx,
 	getch();
 	endwin();
 
-	if ( results_pipe != 0 ) write(results_pipe, "RELEASE\n", 8);
+	if ( term_lock != NULL ) pthread_mutex_unlock(term_lock);
+#else
+	STATUS("Not showing peak box because CrystFEL was compiled without "
+	       "ncurses.\n");
 #endif
 }
 
@@ -439,8 +357,8 @@ static void fit_bg(struct intcontext *ic, struct peak_box *bx)
 	}
 	}
 
-	/* SVD is massive overkill here */
-	ans = solve_svd(v, bx->bgm);
+	/* FIXME: SVD is massive overkill here */
+	ans = solve_svd(v, bx->bgm, NULL, 0);
 	gsl_vector_free(v);
 
 	bx->a = gsl_vector_get(ans, 0);
@@ -497,7 +415,7 @@ static int init_intcontext(struct intcontext *ic)
 	}
 
 	/* How many reference profiles? */
-	ic->n_reference_profiles = ic->image->det->n_panels;
+	ic->n_reference_profiles = 1;
 	ic->reference_profiles = calloc(ic->n_reference_profiles,
 	                                sizeof(double *));
 	if ( ic->reference_profiles == NULL ) return 1;
@@ -641,6 +559,7 @@ static void delete_box(struct intcontext *ic, struct peak_box *bx)
 	}
 
 	free(bx->bm);
+	gsl_matrix_free(bx->bgm);
 
 	memmove(&ic->boxes[i], &ic->boxes[i+1],
 	        (ic->n_boxes-i-1)*sizeof(struct peak_box));
@@ -1205,133 +1124,6 @@ static int suitable_reference(struct intcontext *ic, struct peak_box *bx)
 }
 
 
-static void add_to_rg_matrix(struct intcontext *ic, struct panel *p,
-                             gsl_matrix *M, gsl_vector *vx, gsl_vector *vy,
-                             int *pn)
-{
-	int i;
-
-	for ( i=0; i<ic->n_boxes; i++ ) {
-
-		double x, y, w;
-		double fs, ss;
-		double offs_x, offs_y;
-
-		if ( ic->boxes[i].p != p ) continue;
-
-		fs = ic->boxes[i].cfs + ic->halfw;
-		ss = ic->boxes[i].css + ic->halfw;
-
-		twod_mapping(fs, ss, &x, &y, ic->boxes[i].p);
-
-		w = ic->boxes[i].intensity;
-
-		addm(M, 0, 0, w*fs*fs);
-		addm(M, 0, 1, w*fs*ss);
-		addm(M, 0, 2, w*fs);
-		addm(M, 1, 0, w*fs*ss);
-		addm(M, 1, 1, w*ss*ss);
-		addm(M, 1, 2, w*ss);
-		addm(M, 2, 0, w*fs);
-		addm(M, 2, 1, w*ss);
-		addm(M, 2, 2, w);
-
-		/* Offsets in lab coordinate system */
-		offs_x = p->fsx * ic->boxes[i].offs_fs
-		       + p->ssx * ic->boxes[i].offs_ss;
-
-		offs_y = p->fsy * ic->boxes[i].offs_fs
-		       + p->ssy * ic->boxes[i].offs_ss;
-
-		addv(vx, 0, w*offs_x*fs);
-		addv(vx, 1, w*offs_x*ss);
-		addv(vx, 2, w*offs_x);
-
-		addv(vy, 0, w*offs_y*fs);
-		addv(vy, 1, w*offs_y*ss);
-		addv(vy, 2, w*offs_y);
-
-		(*pn)++;
-	}
-}
-
-
-static void UNUSED refine_rigid_groups(struct intcontext *ic)
-{
-	int i;
-
-	for ( i=0; i<ic->image->det->n_rigid_groups; i++ ) {
-
-		struct rigid_group *rg;
-		gsl_matrix *M;
-		gsl_vector *vx;
-		gsl_vector *vy;
-		gsl_vector *dq1;
-		gsl_vector *dq2;
-		int j;
-		int n;
-
-		M = gsl_matrix_calloc(3, 3);
-		if ( M == NULL ) {
-			ERROR("Failed to allocate matrix\n");
-			return;
-		}
-
-		vx = gsl_vector_calloc(3);
-		if ( vx == NULL ) {
-			ERROR("Failed to allocate vector\n");
-			return;
-		}
-
-		vy = gsl_vector_calloc(3);
-		if ( vy == NULL ) {
-			ERROR("Failed to allocate vector\n");
-			return;
-		}
-
-		rg = ic->image->det->rigid_groups[i];
-
-		n = 0;
-		for ( j=0; j<rg->n_panels; j++ ) {
-			add_to_rg_matrix(ic, rg->panels[j], M, vx, vy, &n);
-		}
-
-		if ( n > 10 ) {
-
-			dq1 = solve_svd(vx, M);
-			dq2 = solve_svd(vy, M);
-
-			rg->d_fsx = gsl_vector_get(dq1, 0);
-			rg->d_ssx = gsl_vector_get(dq1, 1);
-			rg->d_cnx = gsl_vector_get(dq1, 2);
-			rg->d_fsy = gsl_vector_get(dq2, 0);
-			rg->d_ssy = gsl_vector_get(dq2, 1);
-			rg->d_cny = gsl_vector_get(dq2, 2);
-			rg->have_deltas = 1;
-
-			gsl_vector_free(dq1);
-			gsl_vector_free(dq2);
-
-		} else {
-
-			rg->d_fsx = 0.0;
-			rg->d_ssx = 0.0;
-			rg->d_cnx = 0.0;
-			rg->d_fsy = 0.0;
-			rg->d_ssy = 0.0;
-			rg->d_cny = 0.0;
-			rg->have_deltas = 0;
-
-		}
-
-		gsl_vector_free(vx);
-		gsl_vector_free(vy);
-		gsl_matrix_free(M);
-
-	}
-}
-
-
 static int get_int_diag(struct intcontext *ic, Reflection *refl)
 {
 	if ( ic->int_diag == INTDIAG_NONE ) return 0;
@@ -1378,7 +1170,7 @@ static int get_int_diag(struct intcontext *ic, Reflection *refl)
 
 
 static void integrate_prof2d_once(struct intcontext *ic, struct peak_box *bx,
-                                  int results_pipe)
+                                  pthread_mutex_t *term_lock)
 {
 	bx->intensity = fit_intensity(ic, bx);
 	bx->sigma = calc_sigma(ic, bx);
@@ -1401,15 +1193,16 @@ static void integrate_prof2d_once(struct intcontext *ic, struct peak_box *bx,
 		get_detector_pos(bx->refl, &pfs, &pss);
 		pfs += bx->offs_fs;
 		pss += bx->offs_ss;
-		set_detector_pos(bx->refl, 0.0, pfs, pss);
+		set_detector_pos(bx->refl, pfs, pss);
 
 		if ( bx->intensity < -5.0*bx->sigma ) {
 			ic->n_implausible++;
 			set_redundancy(bx->refl, 0);
 		}
 
-		if ( get_int_diag(ic, bx->refl) ) show_peak_box(ic, bx,
-		                                                results_pipe);
+		if ( get_int_diag(ic, bx->refl) ) {
+			show_peak_box(ic, bx, term_lock);
+		}
 
 	} else {
 
@@ -1502,7 +1295,7 @@ static void integrate_prof2d(IntegrationMethod meth,
                              Crystal *cr, struct image *image, IntDiag int_diag,
                              signed int idh, signed int idk, signed int idl,
                              double ir_inn, double ir_mid, double ir_out,
-                             int results_pipe, int **masks)
+                             pthread_mutex_t *term_lock, int **masks)
 {
 	RefList *list;
 	UnitCell *cell;
@@ -1546,7 +1339,7 @@ static void integrate_prof2d(IntegrationMethod meth,
 	for ( i=0; i<ic.n_boxes; i++ ) {
 		struct peak_box *bx;
 		bx = &ic.boxes[i];
-		integrate_prof2d_once(&ic, bx, results_pipe);
+		integrate_prof2d_once(&ic, bx, term_lock);
 	}
 
 	//refine_rigid_groups(&ic);
@@ -1559,7 +1352,7 @@ static void integrate_prof2d(IntegrationMethod meth,
 
 static void integrate_rings_once(Reflection *refl, struct image *image,
                                  struct intcontext *ic, UnitCell *cell,
-                                 int results_pipe)
+                                 pthread_mutex_t *term_lock)
 {
 	double pfs, pss;
 	struct peak_box *bx;
@@ -1605,6 +1398,7 @@ static void integrate_rings_once(Reflection *refl, struct image *image,
 		r = check_box(ic, bx, &saturated);
 		if ( !r ) {
 			fit_bg(ic, bx);
+			if ( !bg_ok(bx) ) r = 1;
 		}
 		bx->offs_fs = 0.0;
 		bx->offs_ss = 0.0;
@@ -1629,9 +1423,10 @@ static void integrate_rings_once(Reflection *refl, struct image *image,
 	sig2_poisson = aduph * intensity;
 
 	/* If intensity is within one photon of nothing, set the Poisson
-	 * error to be one photon */
+	 * error to be one photon.  Pretend I = 1 photon = 1*aduph,
+	 * then sig2_posson = aduph*intensity = aduph^2. */
 	if ( fabs(intensity / aduph) < 1.0 ) {
-		sig2_poisson = aduph;
+		sig2_poisson = aduph * aduph;
 	}
 
 	/* If intensity is negative by more than one photon, assume that
@@ -1654,9 +1449,9 @@ static void integrate_rings_once(Reflection *refl, struct image *image,
 	/* Update position */
 	pfs += bx->offs_fs;
 	pss += bx->offs_ss;
-	set_detector_pos(refl, 0.0, pfs, pss);
+	set_detector_pos(refl, pfs, pss);
 
-	if ( get_int_diag(ic, refl) ) show_peak_box(ic, bx, results_pipe);
+	if ( get_int_diag(ic, refl) ) show_peak_box(ic, bx, term_lock);
 
 	if ( intensity < -5.0*sigma ) {
 		ic->n_implausible++;
@@ -1738,6 +1533,7 @@ static double estimate_resolution(UnitCell *cell, ImageFeatureList *flist)
 
 	if ( n_acc < 3 ) {
 		STATUS("WARNING: Too few peaks to estimate resolution.\n");
+		free(acc);
 		return 0.0;
 	}
 
@@ -1756,7 +1552,7 @@ static void integrate_rings(IntegrationMethod meth,
                             Crystal *cr, struct image *image, IntDiag int_diag,
                             signed int idh, signed int idk, signed int idl,
                             double ir_inn, double ir_mid, double ir_out,
-                            int results_pipe, int **masks)
+                            pthread_mutex_t *term_lock, int **masks)
 {
 	RefList *list;
 	Reflection *refl;
@@ -1792,7 +1588,7 @@ static void integrate_rings(IntegrationMethod meth,
 	      refl != NULL;
 	      refl = next_refl(refl, iter) )
 	{
-		integrate_rings_once(refl, image, &ic, cell, results_pipe);
+		integrate_rings_once(refl, image, &ic, cell, term_lock);
 	}
 
 	//refine_rigid_groups(&ic);
@@ -1800,32 +1596,7 @@ static void integrate_rings(IntegrationMethod meth,
 	free_intcontext(&ic);
 
 	crystal_set_num_saturated_reflections(cr, ic.n_saturated);
-
-	if ( ic.n_implausible ) {
-		STATUS("Warning: %i implausibly negative reflection%s.\n",
-		       ic.n_implausible, ic.n_implausible>1?"s":"");
-	}
-}
-
-
-static void apply_resolution_cutoff(Crystal *cr, double res)
-{
-	Reflection *refl;
-	RefListIterator *iter;
-	UnitCell *cell;
-
-	cell = crystal_get_cell(cr);
-
-	for ( refl = first_refl(crystal_get_reflections(cr), &iter);
-	      refl != NULL;
-	      refl = next_refl(refl, iter) )
-	{
-		signed int h, k, l;
-		get_indices(refl, &h, &k, &l);
-		if ( 2.0*resolution(cell, h, k, l) > res ) {
-			set_redundancy(refl, 0);
-		}
-	}
+	crystal_set_num_implausible_reflections(cr, ic.n_implausible);
 }
 
 
@@ -1834,17 +1605,27 @@ void integrate_all_4(struct image *image, IntegrationMethod meth,
                      double ir_inn, double ir_mid, double ir_out,
                      IntDiag int_diag,
                      signed int idh, signed int idk, signed int idl,
-                     int results_pipe)
+                     pthread_mutex_t *term_lock)
 {
 	int i;
 	int *masks[image->det->n_panels];
 
+	if ( !(meth & INTEGRATION_RESCUT) ) push_res = +INFINITY;
+
 	/* Predict all reflections */
 	for ( i=0; i<image->n_crystals; i++ ) {
+
 		RefList *list;
-		list = find_intersections(image, image->crystals[i],
-		                          PMODEL_SCSPHERE);
+		double res;
+
+		res = estimate_resolution(crystal_get_cell(image->crystals[i]),
+		                          image->features);
+		crystal_set_resolution_limit(image->crystals[i], res);
+
+		list = find_intersections_to_res(image, image->crystals[i],
+		                                 pmodel, res+push_res);
 		crystal_set_reflections(image->crystals[i], list);
+
 	}
 
 	for ( i=0; i<image->det->n_panels; i++ ) {
@@ -1853,7 +1634,6 @@ void integrate_all_4(struct image *image, IntegrationMethod meth,
 
 	for ( i=0; i<image->n_crystals; i++ ) {
 
-		double res = INFINITY;
 		Crystal *cr = image->crystals[i];
 
 		switch ( meth & INTEGRATION_METHOD_MASK ) {
@@ -1865,18 +1645,14 @@ void integrate_all_4(struct image *image, IntegrationMethod meth,
 			integrate_rings(meth, cr, image,
 			                int_diag, idh, idk, idl,
 			                ir_inn, ir_mid, ir_out,
-			                results_pipe, masks);
-			res = estimate_resolution(crystal_get_cell(cr),
-			                          image->features);
+			                term_lock, masks);
 			break;
 
 			case INTEGRATION_PROF2D :
 			integrate_prof2d(meth, cr, image,
 			                 int_diag, idh, idk, idl,
 			                 ir_inn, ir_mid, ir_out,
-			                 results_pipe, masks);
-			res = estimate_resolution(crystal_get_cell(cr),
-			                          image->features);
+			                 term_lock, masks);
 			break;
 
 			default :
@@ -1885,11 +1661,6 @@ void integrate_all_4(struct image *image, IntegrationMethod meth,
 
 		}
 
-		crystal_set_resolution_limit(cr, res);
-		if ( meth & INTEGRATION_RESCUT ) {
-			apply_resolution_cutoff(cr, res+push_res);
-		}
-
 	}
 
 	for ( i=0; i<image->det->n_panels; i++ ) {
@@ -1915,7 +1686,8 @@ void integrate_all_2(struct image *image, IntegrationMethod meth,
                      IntDiag int_diag,
                      signed int idh, signed int idk, signed int idl)
 {
-	integrate_all_3(image, meth, PMODEL_SPHERE, 0.0, ir_inn, ir_mid, ir_out,
+	integrate_all_3(image, meth, PMODEL_SCSPHERE, 0.0,
+	                ir_inn, ir_mid, ir_out,
 	                int_diag, idh, idk, idl);
 }
 
