@@ -3,13 +3,14 @@
  *
  * Invoke the DPS auto-indexing algorithm through MOSFLM
  *
- * Copyright © 2012-2013 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  *
  * Authors:
  *   2010      Richard Kirian <rkirian@asu.edu>
- *   2010-2013 Thomas White <taw@physics.org>
+ *   2010-2014 Thomas White <taw@physics.org>
+ *   2014      Takanori Nakane <nakane.t@gmail.com>
  *
  * This file is part of CrystFEL.
  *
@@ -200,18 +201,18 @@ static int check_cell(struct mosflm_private *mp, struct image *image,
 static void mosflm_parseline(const char *line, struct image *image,
                              struct mosflm_data *dirax)
 {
-	#if MOSFLM_VERBOSE
-	char *copy;
-	int i;
+	if ( MOSFLM_VERBOSE || (strncmp(line, "Invocation:", 11) == 0) ) {
+		char *copy;
+		int i;
 
-	copy = strdup(line);
-	for ( i=0; i<strlen(copy); i++ ) {
-		if ( copy[i] == '\r' ) copy[i]='r';
-		if ( copy[i] == '\n' ) copy[i]='\0';
+		copy = strdup(line);
+		for ( i=0; i<strlen(copy); i++ ) {
+			if ( copy[i] == '\r' ) copy[i]='r';
+			if ( copy[i] == '\n' ) copy[i]='\0';
+		}
+		STATUS("MOSFLM: %s\n", copy);
+		free(copy);
 	}
-	STATUS("MOSFLM: %s\n", copy);
-	free(copy);
-	#endif
 }
 
 
@@ -576,14 +577,29 @@ static void mosflm_send_next(struct image *image, struct mosflm_data *mosflm)
 		break;
 
 		case 8 :
-		snprintf(tmp, 255, "AUTOINDEX DPS FILE %s"
-		                   " IMAGE 1 MAXCELL 1000 REFINE\n",
-		         mosflm->sptfile);
+		if ( mosflm->mp->indm & INDEXING_USE_CELL_PARAMETERS ) {
 
-		/* "This option is only available if you e-mail Andrew Leslie
-		 * and ask for it." - MOSFLM
-		 * snprintf(tmp, 255, "AUTOINDEX NODISPLAY IMAGE 1 FILE %s\n",
-		 *         mosflm->sptfile); */
+			char cen;
+			double a, b, c, alpha, beta, gamma;
+
+			cell_get_parameters(mosflm->mp->template,
+			                    &a, &b, &c, &alpha, &beta, &gamma);
+			cen = cell_get_centering(mosflm->mp->template);
+
+			snprintf(tmp, 255, "AUTOINDEX DPS FILE %s IMAGE 1 "
+			         "MAXCELL 1000 REFINE "
+			         "CELL %.1f %.1f %.1f %.1f %.1f %.1f "
+			         "CENTERING %c\n",
+				 mosflm->sptfile, a*1e10, b*1e10, c*1e10,
+				 rad2deg(alpha), rad2deg(beta), rad2deg(gamma),
+				 cen);
+
+                } else {
+		        snprintf(tmp, 255, "AUTOINDEX DPS FILE %s"
+			                   " IMAGE 1 MAXCELL 1000 REFINE\n",
+			                   mosflm->sptfile);
+
+		}
 		mosflm_sendline(tmp, mosflm);
 		break;
 
@@ -753,8 +769,10 @@ int run_mosflm(struct image *image, IndexingPrivate *ipriv)
 		t.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
 		tcsetattr(STDIN_FILENO, TCSANOW, &t);
 
+		execlp("mosflm", "", (char *)NULL);
 		execlp("ipmosflm", "", (char *)NULL);
-		ERROR("Failed to invoke MOSFLM.\n");
+		ERROR("Invocation: Failed to invoke MOSFLM: %s\n",
+		      strerror(errno));
 		_exit(0);
 
 	}
@@ -831,39 +849,59 @@ int run_mosflm(struct image *image, IndexingPrivate *ipriv)
 
 
 IndexingPrivate *mosflm_prepare(IndexingMethod *indm, UnitCell *cell,
-                                struct detector *det, struct beam_params *beam,
-                                float *ltl)
+                                struct detector *det, float *ltl)
 {
 	struct mosflm_private *mp;
 	int need_cell = 0;
 
+	/* Check if cell parameters are needed/provided */
 	if ( *indm & INDEXING_CHECK_CELL_COMBINATIONS ) need_cell = 1;
 	if ( *indm & INDEXING_CHECK_CELL_AXES ) need_cell = 1;
-	if ( *indm & INDEXING_USE_LATTICE_TYPE ) need_cell = 1;
-
-	if ( need_cell && (cell == NULL) ) {
-		ERROR("Altering your MOSFLM flags because no PDB file was"
-		      " provided.\n");
+	if ( *indm & INDEXING_USE_CELL_PARAMETERS ) need_cell = 1;
+	if ( need_cell && !cell_has_parameters(cell) ) {
+		ERROR("Altering your MOSFLM flags because cell parameters were"
+		      " not provided.\n");
 		*indm &= ~INDEXING_CHECK_CELL_COMBINATIONS;
 		*indm &= ~INDEXING_CHECK_CELL_AXES;
+		*indm &= ~INDEXING_USE_CELL_PARAMETERS;
+	}
+
+	/* Check if lattice type information is needed/provided */
+	if ( (*indm & INDEXING_USE_LATTICE_TYPE) && (cell == NULL) ) {
+		ERROR("Altering your MOSFLM flags because lattice type "
+		      "information was not provided.\n");
 		*indm &= ~INDEXING_USE_LATTICE_TYPE;
 	}
 
 	/* Flags that MOSFLM knows about */
 	*indm &= INDEXING_METHOD_MASK | INDEXING_CHECK_CELL_COMBINATIONS
 	       | INDEXING_CHECK_CELL_AXES | INDEXING_CHECK_PEAKS
-	       | INDEXING_USE_LATTICE_TYPE;
+	       | INDEXING_USE_LATTICE_TYPE| INDEXING_USE_CELL_PARAMETERS;
 
-	if ( *indm & INDEXING_USE_LATTICE_TYPE ) {
-		if ( !((*indm & INDEXING_CHECK_CELL_COMBINATIONS)
-		    || (*indm & INDEXING_CHECK_CELL_AXES)) ) {
-			ERROR("WARNING: The unit cell from %s might have had "
-			      "its axes permuted from the unit cell you gave.\n"
-			      "If this is a problem, consider using "
-			      "mosflm-axes-latt or mosflm-comb-latt instead of "
-			      "mosflm-raw-latt.\n", indexer_str(*indm));
-		}
+	if ( (*indm & INDEXING_USE_LATTICE_TYPE)
+	   && !((*indm & INDEXING_CHECK_CELL_COMBINATIONS)
+	       || (*indm & INDEXING_CHECK_CELL_AXES))
+	   && (cell_has_parameters(cell)
+	       || (cell_get_unique_axis(cell) == 'a')
+	       || (cell_get_unique_axis(cell) == 'b')
+	       || (cell_get_unique_axis(cell) == 'c')) )
+	{
+		ERROR("WARNING: The unit cell from %s might have had "
+		      "its axes permuted from the unit cell you gave.\n"
+		      "If this is a problem, consider using "
+		      "mosflm-axes-latt or mosflm-comb-latt instead of "
+		      "mosflm-raw-latt.\n", indexer_str(*indm));
+		/* It'll be OK if INDEXING_CHECK_CELL_PARAMETERS and
+		 * MOSFLM 7.2.0 or later, but that's getting a bit too
+		 * complicated to explain to the user. */
+	}
 
+	if ( (*indm & INDEXING_USE_CELL_PARAMETERS)
+	  && !((*indm & INDEXING_CHECK_CELL_COMBINATIONS)
+	    || (*indm & INDEXING_CHECK_CELL_AXES)) )
+	{
+		ERROR("WARNING: Using 'mosflm-raw' but providing cell "
+		      "parameters as prior information to mosflm.\n");
 	}
 
 	mp = malloc(sizeof(struct mosflm_private));
