@@ -67,20 +67,15 @@ struct _refldata {
 	signed int ls;
 
 	/* Partiality and related geometrical stuff */
-	double r1;  /* First excitation error */
-	double r2;  /* Second excitation error */
+	double rlow;  /* Low excitation error */
+	double rhigh;  /* High excitation error */
 	double p;   /* Partiality */
 	double L;   /* Lorentz factor */
-	int clamp1; /* Clamp status for r1 */
-	int clamp2; /* Clamp status for r2 */
 
 	/* Location in image */
 	double fs;
 	double ss;
-
-	/* The distance from the exact Bragg position to the coordinates
-	 * given above. */
-	double excitation_error;
+	struct panel *panel;
 
 	/* Non-zero if this reflection can be used for scaling */
 	int scalable;
@@ -107,6 +102,7 @@ struct _refldata {
 	/* User-specified temporary values */
 	double temp1;
 	double temp2;
+	int flag;
 };
 
 
@@ -318,17 +314,6 @@ Reflection *next_found_refl(Reflection *refl)
 
 /********************************** Getters ***********************************/
 
-/**
- * get_excitation_error:
- * @refl: A %Reflection
- *
- * Returns: The excitation error for the reflection.
- **/
-double get_excitation_error(const Reflection *refl)
-{
-	return refl->data.excitation_error;
-}
-
 
 /**
  * get_detector_pos:
@@ -344,6 +329,19 @@ void get_detector_pos(const Reflection *refl, double *fs, double *ss)
 }
 
 
+/**
+ * get_panel:
+ * @refl: A %Reflection
+ *
+ * Returns: the panel which the reflection appears on
+ *
+ **/
+struct panel *get_panel(const Reflection *refl)
+{
+	return refl->data.panel;
+}
+
+
 /**
  * get_indices:
  * @refl: A %Reflection
@@ -424,24 +422,20 @@ double get_intensity(const Reflection *refl)
 /**
  * get_partial:
  * @refl: A %Reflection
- * @r1: Location at which to store the first excitation error
- * @r2: Location at which to store the second excitation error
+ * @rlow: Location at which to store the "low" excitation error
+ * @rhigh: Location at which to store the "high" excitation error
  * @p: Location at which to store the partiality
- * @clamp_low: Location at which to store the first clamp status
- * @clamp_high: Location at which to store the second clamp status
  *
  * This function is used during post refinement (in conjunction with
  * set_partial()) to get access to the details of the partiality calculation.
  *
  **/
-void get_partial(const Reflection *refl, double *r1, double *r2, double *p,
-                 int *clamp_low, int *clamp_high)
+void get_partial(const Reflection *refl, double *rlow, double *rhigh,
+                 double *p)
 {
-	*r1 = refl->data.r1;
-	*r2 = refl->data.r2;
+	*rlow = refl->data.rlow;
+	*rhigh = refl->data.rhigh;
 	*p = get_partiality(refl);
-	*clamp_low = refl->data.clamp1;
-	*clamp_high = refl->data.clamp2;
 }
 
 
@@ -553,6 +547,22 @@ double get_temp2(const Reflection *refl)
 }
 
 
+/**
+ * get_flag:
+ * @refl: A %Reflection
+ *
+ * The integer flag value can be used according to the needs of the calling
+ * program.
+ *
+ * Returns: the flag for this reflection.
+ *
+ **/
+int get_flag(const Reflection *refl)
+{
+	return refl->data.flag;
+}
+
+
 /********************************** Setters ***********************************/
 
 /**
@@ -576,40 +586,47 @@ void copy_data(Reflection *to, const Reflection *from)
 /**
  * set_detector_pos:
  * @refl: A %Reflection
- * @exerr: The excitation error for this reflection
  * @fs: The fast scan offset of the reflection
  * @ss: The slow scan offset of the reflection
  *
  **/
-void set_detector_pos(Reflection *refl, double exerr, double fs, double ss)
+void set_detector_pos(Reflection *refl, double fs, double ss)
 {
-	refl->data.excitation_error = exerr;
 	refl->data.fs = fs;
 	refl->data.ss = ss;
 }
 
 
+/**
+ * set_panel:
+ * @refl: A %Reflection
+ * @p: Pointer to the panel structure on which the reflection appears
+ *
+ * Note that the pointer will be stored, not the contents of the structure.
+ *
+ **/
+void set_panel(Reflection *refl, struct panel *p)
+{
+	refl->data.panel = p;
+}
+
+
 /**
  * set_partial:
  * @refl: A %Reflection
- * @r1: The first excitation error
- * @r2: The second excitation error
+ * @rlow: The "low" excitation error
+ * @rhigh: The "high" excitation error
  * @p: The partiality
- * @clamp_low: The first clamp status
- * @clamp_high: The second clamp status
  *
  * This function is used during post refinement (in conjunction with
  * get_partial()) to get access to the details of the partiality calculation.
  *
  **/
-void set_partial(Reflection *refl, double r1, double r2, double p,
-                 double clamp_low, double clamp_high)
+void set_partial(Reflection *refl, double rlow, double rhigh, double p)
 {
-	refl->data.r1 = r1;
-	refl->data.r2 = r2;
+	refl->data.rlow = rlow;
+	refl->data.rhigh = rhigh;
 	refl->data.p = p;
-	refl->data.clamp1 = clamp_low;
-	refl->data.clamp2 = clamp_high;
 }
 
 
@@ -772,6 +789,21 @@ void set_temp2(Reflection *refl, double temp)
 }
 
 
+/**
+ * set_flag
+ * @refl: A %Reflection
+ * @flag: New flag value
+ *
+ * @flag is an integer value which can be used according to the needs of the
+ * calling program.
+ *
+ **/
+void set_flag(Reflection *refl, int flag)
+{
+	refl->data.flag = flag;
+}
+
+
 /********************************* Insertion **********************************/
 
 static Reflection *rotate_once(Reflection *refl, int dir)
