@@ -3,11 +3,11 @@
  *
  * Fast reflection/peak list
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2011-2014 Thomas White <taw@physics.org>
+ *   2011-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -84,8 +84,8 @@ extern Reflection *find_refl(const RefList *list, signed int h, signed int k, si
 extern Reflection *next_found_refl(Reflection *refl);
 
 /* Get */
-extern double get_excitation_error(const Reflection *refl);
 extern void get_detector_pos(const Reflection *refl, double *fs, double *ss);
+extern struct panel *get_panel(const Reflection *refl);
 extern double get_partiality(const Reflection *refl);
 extern double get_lorentz(const Reflection *refl);
 extern void get_indices(const Reflection *refl,
@@ -94,8 +94,8 @@ extern void get_symmetric_indices(const Reflection *refl,
                                   signed int *hs, signed int *ks,
                                   signed int *ls);
 extern double get_intensity(const Reflection *refl);
-extern void get_partial(const Reflection *refl, double *r1, double *r2,
-                        double *p, int *clamp_low, int *clamp_high);
+extern void get_partial(const Reflection *refl, double *rlow, double *rhigh,
+                        double *p);
 extern int get_redundancy(const Reflection *refl);
 extern double get_temp1(const Reflection *refl);
 extern double get_temp2(const Reflection *refl);
@@ -103,13 +103,13 @@ extern double get_esd_intensity(const Reflection *refl);
 extern double get_phase(const Reflection *refl, int *have_phase);
 extern double get_peak(const Reflection *refl);
 extern double get_mean_bg(const Reflection *refl);
+extern int get_flag(const Reflection *refl);
 
 /* Set */
 extern void copy_data(Reflection *to, const Reflection *from);
-extern void set_detector_pos(Reflection *refl, double exerr,
-                             double fs, double ss);
-extern void set_partial(Reflection *refl, double r1, double r2, double p,
-                        double clamp_low, double clamp_high);
+extern void set_detector_pos(Reflection *refl, double fs, double ss);
+extern void set_panel(Reflection *refl, struct panel *p);
+extern void set_partial(Reflection *refl, double rlow, double rhigh, double p);
 extern void set_partiality(Reflection *refl, double p);
 extern void set_lorentz(Reflection *refl, double L);
 extern void set_intensity(Reflection *refl, double intensity);
@@ -122,6 +122,7 @@ extern void set_peak(Reflection *refl, double peak);
 extern void set_mean_bg(Reflection *refl, double mean_bg);
 extern void set_symmetric_indices(Reflection *refl,
                                   signed int hs, signed int ks, signed int ls);
+extern void set_flag(Reflection *refl, int flag);
 
 /* Insertion */
 extern Reflection *add_refl(RefList *list,
