@@ -38,7 +38,8 @@ enum {
 	SCALE_COLOUR,
 	SCALE_MONO,
 	SCALE_INVMONO,
-	SCALE_RATIO
+	SCALE_RATIO,
+	SCALE_GEOPTIMISER
 };
 
 #ifdef __cplusplus
