@@ -3,12 +3,13 @@
  *
  * Stream tools
  *
- * Copyright © 2013-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2013-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
+ *   2014      Valerio Mariani
  *   2011      Richard Kirian
  *   2011      Andrew Aquila
  *   2014      Takanori Nakane <nakane.t@gmail.com>
@@ -53,12 +54,11 @@
 #include "reflist-utils.h"
 
 #define LATEST_MAJOR_VERSION (2)
-#define LATEST_MINOR_VERSION (2)
+#define LATEST_MINOR_VERSION (3)
 
 #define AT_LEAST_VERSION(st, a, b) ((st->major_version>=(a)) \
                                     && (st->minor_version>=(b)))
 
-
 struct _stream
 {
 	FILE *fh;
@@ -67,7 +67,6 @@ struct _stream
 	int minor_version;
 };
 
-
 static int read_peaks(FILE *fh, struct image *image)
 {
 	char *rval = NULL;
@@ -80,6 +79,8 @@ static int read_peaks(FILE *fh, struct image *image)
 		char line[1024];
 		float x, y, d, intensity;
 		int r;
+		struct panel *p = NULL;
+		float add_x, add_y;
 
 		rval = fgets(line, 1023, fh);
 		if ( rval == NULL ) continue;
@@ -96,8 +97,26 @@ static int read_peaks(FILE *fh, struct image *image)
 
 		first = 0;
 		if ( r == 4 ) {
-			image_add_feature(image->features, x, y,
-			                  image, intensity, NULL);
+
+			if ( image->det != NULL ) {
+
+				p = find_orig_panel(image->det, x, y);
+				if ( p == NULL ) {
+					ERROR("Panel not found\n");
+					return 1;
+				}
+
+				add_x = x-p->orig_min_fs+p->min_fs;
+				add_y = y-p->orig_min_ss+p->min_ss;
+
+				image_add_feature(image->features, add_x, add_y,
+				                  image, intensity, NULL);
+
+			} else {
+
+				image_add_feature(image->features, x, y,
+				image, intensity, NULL);
+			}
 		}
 
 	} while ( rval != NULL );
@@ -107,7 +126,61 @@ static int read_peaks(FILE *fh, struct image *image)
 }
 
 
-static void write_peaks(struct image *image, FILE *ofh)
+static int read_peaks_2_3(FILE *fh, struct image *image)
+{
+	char *rval = NULL;
+	int first = 1;
+
+	image->features = image_feature_list_new();
+
+	do {
+
+		char line[1024];
+		char pn[32];
+		float x, y, d, intensity;
+		int r;
+		struct panel *p = NULL;
+		float add_x, add_y;
+
+		rval = fgets(line, 1023, fh);
+		if ( rval == NULL ) continue;
+		chomp(line);
+
+		if ( strcmp(line, PEAK_LIST_END_MARKER) == 0 ) return 0;
+
+		r = sscanf(line, "%f %f %f %f %s", &x, &y, &d, &intensity, pn);
+		if ( (r != 5) && (!first) ) {
+			ERROR("Failed to parse peak list line.\n");
+			ERROR("The failed line was: '%s'\n", line);
+			return 1;
+		}
+
+		first = 0;
+
+		if ( r == 5 ) {
+
+			p = find_panel_by_name(image->det, pn);
+			if ( p == NULL ) {
+				ERROR("Panel not found: %s\n", pn);
+				return 1;
+			}
+
+			add_x = x-p->orig_min_fs+p->min_fs;
+			add_y = y-p->orig_min_ss+p->min_ss;
+
+			image_add_feature(image->features, add_x, add_y,
+			                  image, intensity, NULL);
+
+		}
+
+	} while ( rval != NULL );
+
+	/* Got read error of some kind before finding PEAK_LIST_END_MARKER */
+	return 1;
+}
+
+
+static int write_peaks(struct image *image, FILE *ofh)
 {
 	int i;
 
@@ -126,16 +199,143 @@ static void write_peaks(struct image *image, FILE *ofh)
 		r = get_q(image, f->fs, f->ss, NULL, 1.0/image->lambda);
 		q = modulus(r.u, r.v, r.w);
 
-		fprintf(ofh, "%7.2f %7.2f   %10.2f  %10.2f\n",
-		       f->fs, f->ss, q/1.0e9, f->intensity);
+		if ( image->det != NULL ) {
+
+			struct panel *p;
+			double write_fs, write_ss;
+
+			p = find_orig_panel(image->det, f->fs, f->ss);
+			if ( p == NULL ) {
+				ERROR("Panel not found\n");
+				return 1;
+			}
+
+			/* Convert coordinates to match arrangement of panels in
+			 * HDF5 file */
+			write_fs = f->fs - p->min_fs + p->orig_min_fs;
+			write_ss = f->ss - p->min_ss + p->orig_min_ss;
+
+			fprintf(ofh, "%7.2f %7.2f   %10.2f  %10.2f\n",
+			        write_fs, write_ss, q/1.0e9, f->intensity);
+
+		} else {
+
+			fprintf(ofh, "%7.2f %7.2f   %10.2f  %10.2f\n",
+			        f->fs, f->ss, q/1.0e9, f->intensity);
+
+		}
 
 	}
 
 	fprintf(ofh, PEAK_LIST_END_MARKER"\n");
+	return 0;
 }
 
 
-static RefList *read_stream_reflections_2_1(FILE *fh)
+static int write_peaks_2_3(struct image *image, FILE *ofh)
+{
+	int i;
+
+	fprintf(ofh, PEAK_LIST_START_MARKER"\n");
+	fprintf(ofh, "  fs/px   ss/px (1/d)/nm^-1   Intensity  Panel\n");
+
+	for ( i=0; i<image_feature_count(image->features); i++ ) {
+
+		struct imagefeature *f;
+		struct rvec r;
+		double q;
+		struct panel *p;
+		double write_fs, write_ss;
+
+		f = image_get_feature(image->features, i);
+		if ( f == NULL ) continue;
+
+		r = get_q(image, f->fs, f->ss, NULL, 1.0/image->lambda);
+		q = modulus(r.u, r.v, r.w);
+
+		p = find_panel(image->det, f->fs, f->ss);
+		if ( p == NULL ) {
+			ERROR("Panel not found\n");
+			return 1;
+		}
+
+		/* Convert coordinates to match arrangement of panels in HDF5
+		 * file */
+		write_fs = f->fs - p->min_fs + p->orig_min_fs;
+		write_ss = f->ss - p->min_ss + p->orig_min_ss;
+
+		fprintf(ofh, "%7.2f %7.2f %10.2f  %10.2f   %s\n",
+		        write_fs, write_ss, q/1.0e9, f->intensity, p->name);
+
+	}
+
+	fprintf(ofh, PEAK_LIST_END_MARKER"\n");
+	return 0;
+}
+
+
+static RefList *read_stream_reflections_2_3(FILE *fh, struct detector *det)
+{
+	char *rval = NULL;
+	int first = 1;
+	RefList *out;
+
+	out = reflist_new();
+
+	do {
+
+		char line[1024];
+		signed int h, k, l;
+		float intensity, sigma, fs, ss, pk, bg;
+		char pn[32];
+		int r;
+		Reflection *refl;
+
+		rval = fgets(line, 1023, fh);
+		if ( rval == NULL ) continue;
+		chomp(line);
+
+		if ( strcmp(line, REFLECTION_END_MARKER) == 0 ) return out;
+
+		r = sscanf(line, "%i %i %i %f %f %f %f %f %f %s",
+		           &h, &k, &l, &intensity, &sigma, &pk, &bg,
+			   &fs, &ss, pn);
+
+		if ( (r != 10) && (!first) ) {
+			reflist_free(out);
+			return NULL;
+		}
+
+		first = 0;
+
+		if ( r == 10 ) {
+
+			struct panel *p;
+
+			refl = add_refl(out, h, k, l);
+			set_intensity(refl, intensity);
+			if ( det != NULL ) {
+				double write_fs, write_ss;
+				p = find_panel_by_name(det,pn);
+				write_fs = fs - p->orig_min_fs + p->min_fs;
+				write_ss = ss - p->orig_min_ss + p->min_ss;
+				set_detector_pos(refl, write_fs, write_ss);
+				set_panel(refl, p);
+			}
+			set_esd_intensity(refl, sigma);
+			set_peak(refl, pk);
+			set_mean_bg(refl, bg);
+			set_redundancy(refl, 1);
+		}
+
+	} while ( rval != NULL );
+
+	/* Got read error of some kind before finding PEAK_LIST_END_MARKER */
+	return NULL;
+}
+
+
+static RefList *read_stream_reflections_2_1(FILE *fh, struct detector *det)
 {
 	char *rval = NULL;
 	int first = 1;
@@ -160,7 +360,8 @@ static RefList *read_stream_reflections_2_1(FILE *fh)
 		if ( strcmp(line, REFLECTION_END_MARKER) == 0 ) return out;
 
 		r = sscanf(line, "%i %i %i %f %s %f %i %f %f",
-		           &h, &k, &l, &intensity, phs, &sigma, &cts, &fs, &ss);
+		                  &h, &k, &l, &intensity, phs, &sigma, &cts,
+		                   &fs, &ss);
 		if ( (r != 9) && (!first) ) {
 			reflist_free(out);
 			return NULL;
@@ -174,7 +375,22 @@ static RefList *read_stream_reflections_2_1(FILE *fh)
 
 			refl = add_refl(out, h, k, l);
 			set_intensity(refl, intensity);
-			set_detector_pos(refl, 0.0, fs, ss);
+
+			if ( det != NULL ) {
+
+				double write_fs, write_ss;
+				struct panel *p;
+
+				p = find_orig_panel(det, fs, ss);
+				write_fs = fs - p->orig_min_fs + p->min_fs;
+				write_ss = ss - p->orig_min_ss + p->min_ss;
+				set_detector_pos(refl, write_fs, write_ss);
+
+			} else {
+
+				set_detector_pos(refl, fs, ss);
+
+			}
 			set_esd_intensity(refl, sigma);
 			set_redundancy(refl, cts);
 
@@ -190,7 +406,7 @@ static RefList *read_stream_reflections_2_1(FILE *fh)
 }
 
 
-static RefList *read_stream_reflections(FILE *fh)
+static RefList *read_stream_reflections_2_2(FILE *fh, struct detector *det)
 {
 	char *rval = NULL;
 	int first = 1;
@@ -224,7 +440,23 @@ static RefList *read_stream_reflections(FILE *fh)
 
 			refl = add_refl(out, h, k, l);
 			set_intensity(refl, intensity);
-			set_detector_pos(refl, 0.0, fs, ss);
+
+			if ( det != NULL ) {
+
+				double write_fs, write_ss;
+				struct panel *p;
+
+				p = find_orig_panel(det, fs, ss);
+				write_fs = fs - p->orig_min_fs + p->min_fs;
+				write_ss = ss - p->orig_min_ss + p->min_ss;
+				set_detector_pos(refl, write_fs, write_ss);
+
+			} else {
+
+				set_detector_pos(refl, fs, ss);
+
+			}
+
 			set_esd_intensity(refl, sigma);
 			set_redundancy(refl, 1);
 			set_peak(refl, pk);
@@ -234,53 +466,19 @@ static RefList *read_stream_reflections(FILE *fh)
 
 	} while ( rval != NULL );
 
-	/* Got read error of some kind before finding PEAK_LIST_END_MARKER */
+	/* Got read error of some kind before finding REFLECTION_END_MARKER */
 	return NULL;
 }
 
 
-static void write_stream_reflections(FILE *fh, RefList *list)
-{
-	Reflection *refl;
-	RefListIterator *iter;
-
-	fprintf(fh, "   h    k    l          I   sigma(I)       peak background"
-	            "  fs/px  ss/px\n");
-
-	for ( refl = first_refl(list, &iter);
-	      refl != NULL;
-	      refl = next_refl(refl, iter) )
-	{
-
-		signed int h, k, l;
-		double intensity, esd_i, bg, pk;
-		double fs, ss;
-
-		get_indices(refl, &h, &k, &l);
-		get_detector_pos(refl, &fs, &ss);
-		intensity = get_intensity(refl);
-		esd_i = get_esd_intensity(refl);
-		pk = get_peak(refl);
-		bg = get_mean_bg(refl);
-
-		/* Reflections with redundancy = 0 are not written */
-		if ( get_redundancy(refl) == 0 ) continue;
-
-		fprintf(fh,
-		       "%4i %4i %4i %10.2f %10.2f %10.2f %10.2f %6.1f %6.1f\n",
-		       h, k, l, intensity, esd_i, pk, bg, fs, ss);
-
-	}
-}
-
-
-static void write_stream_reflections_2_1(FILE *fh, RefList *list)
+static int write_stream_reflections_2_1(FILE *fh, RefList *list,
+                                        struct image *image)
 {
 	Reflection *refl;
 	RefListIterator *iter;
 
 	fprintf(fh, "  h   k   l          I    phase   sigma(I) "
-		     " counts  fs/px  ss/px\n");
+			 " counts  fs/px  ss/px\n");
 
 	for ( refl = first_refl(list, &iter);
 	      refl != NULL;
@@ -310,11 +508,147 @@ static void write_stream_reflections_2_1(FILE *fh, RefList *list)
 			strncpy(phs, "       -", 15);
 		}
 
+		if ( image->det != NULL ) {
+
+			struct panel *p;
+			double write_fs, write_ss;
+
+			p = find_orig_panel(image->det, fs, ss);
+			if ( p == NULL ) {
+				ERROR("Panel not found\n");
+				return 1;
+			}
+
+			/* Convert coordinates to match arrangement of panels
+			 * in HDF5 file */
+			write_fs = fs - p->min_fs + p->orig_min_fs;
+			write_ss = ss - p->min_ss + p->orig_min_ss;
+
+			fprintf(fh, "%3i %3i %3i %10.2f %s %10.2f %7i "
+			            "%6.1f %6.1f\n",
+			             h, k, l, intensity, phs, esd_i, red,
+				     write_fs, write_ss);
+
+		} else {
+
+			fprintf(fh, "%3i %3i %3i %10.2f %s %10.2f %7i "
+			            "%6.1f %6.1f\n",
+			            h, k, l, intensity, phs, esd_i, red,
+				    fs, ss);
+
+		}
+	}
+	return 0;
+}
+
+
+static int write_stream_reflections_2_2(FILE *fh, RefList *list,
+                                        struct image *image)
+{
+	Reflection *refl;
+	RefListIterator *iter;
+
+	fprintf(fh, "   h    k    l          I   sigma(I)       "
+	            "peak background  fs/px  ss/px\n");
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+
+		signed int h, k, l;
+		double intensity, esd_i, bg, pk;
+		double fs, ss;
+
+		get_indices(refl, &h, &k, &l);
+		get_detector_pos(refl, &fs, &ss);
+		intensity = get_intensity(refl);
+		esd_i = get_esd_intensity(refl);
+		pk = get_peak(refl);
+		bg = get_mean_bg(refl);
+
+		/* Reflections with redundancy = 0 are not written */
+		if ( get_redundancy(refl) == 0 ) continue;
+
+		if ( image->det != NULL ) {
+
+			struct panel *p;
+			double write_fs, write_ss;
+
+			p = find_orig_panel(image->det, fs, ss);
+			if ( p == NULL ) {
+				ERROR("Panel not found\n");
+				return 1;
+			}
+
+			/* Convert coordinates to match arrangement of panels in HDF5
+			 * file */
+			write_fs = fs - p->min_fs + p->orig_min_fs;
+			write_ss = ss - p->min_ss + p->orig_min_ss;
+
+			fprintf(fh, "%4i %4i %4i %10.2f %10.2f %10.2f %10.2f"
+			            " %6.1f %6.1f\n",
+			        h, k, l, intensity, esd_i, pk, bg, write_fs,
+			        write_ss);
+
+		} else {
+
+			fprintf(fh, "%4i %4i %4i %10.2f %10.2f %10.2f %10.2f"
+			            " %6.1f %6.1f\n",
+			        h, k, l, intensity, esd_i, pk, bg, fs, ss);
+		}
+	}
+	return 0;
+}
+
+
+static int write_stream_reflections_2_3(FILE *fh, RefList *list,
+                                        struct image *image)
+{
+	Reflection *refl;
+	RefListIterator *iter;
+
+	fprintf(fh, "   h    k    l          I   sigma(I)       "
+	            "peak background  fs/px  ss/px panel\n");
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+
+		signed int h, k, l;
+		double intensity, esd_i, pk, bg;
+		double fs, ss;
+		double write_fs, write_ss;
+		struct panel *p = NULL;
+
+		get_indices(refl, &h, &k, &l);
+		get_detector_pos(refl, &fs, &ss);
+		intensity = get_intensity(refl);
+		esd_i = get_esd_intensity(refl);
+		pk = get_peak(refl);
+		bg = get_mean_bg(refl);
+
+		/* Reflections with redundancy = 0 are not written */
+		if ( get_redundancy(refl) == 0 ) continue;
+
+		p = find_panel(image->det,fs,ss);
+		if ( p == NULL ) {
+			ERROR("Panel not found\n");
+			return 1;
+		}
+
+		write_fs = fs-p->min_fs+p->orig_min_fs;
+		write_ss = ss-p->min_ss+p->orig_min_ss;
+
 		fprintf(fh,
-		       "%3i %3i %3i %10.2f %s %10.2f %7i %6.1f %6.1f\n",
-		       h, k, l, intensity, phs, esd_i, red,  fs, ss);
+                          "%4i %4i %4i %10.2f %10.2f %10.2f %10.2f "
+                          "%6.1f %6.1f %s\n",
+                           h, k, l, intensity, esd_i, pk, bg,
+                           write_fs, write_ss, p->name);
 
 	}
+	return 0;
 }
 
 
@@ -335,7 +669,7 @@ static int num_integrated_reflections(RefList *list)
 }
 
 
-static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
+static int write_crystal(Stream *st, Crystal *cr, int include_reflections)
 {
 	UnitCell *cell;
 	RefList *reflist;
@@ -344,6 +678,7 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 	double csx, csy, csz;
 	double a, b, c, al, be, ga;
 	double rad;
+	int ret = 0;
 
 	fprintf(st->fh, CRYSTAL_START_MARKER"\n");
 
@@ -352,40 +687,46 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 
 	cell_get_parameters(cell, &a, &b, &c, &al, &be, &ga);
 	fprintf(st->fh, "Cell parameters %7.5f %7.5f %7.5f nm,"
-	                " %7.5f %7.5f %7.5f deg\n",
-	                a*1.0e9, b*1.0e9, c*1.0e9,
-	                rad2deg(al), rad2deg(be), rad2deg(ga));
+			" %7.5f %7.5f %7.5f deg\n",
+			a*1.0e9, b*1.0e9, c*1.0e9,
+			rad2deg(al), rad2deg(be), rad2deg(ga));
 
 	cell_get_reciprocal(cell, &asx, &asy, &asz,
-		                  &bsx, &bsy, &bsz,
-		                  &csx, &csy, &csz);
+				  &bsx, &bsy, &bsz,
+				  &csx, &csy, &csz);
 	fprintf(st->fh, "astar = %+9.7f %+9.7f %+9.7f nm^-1\n",
 	        asx/1e9, asy/1e9, asz/1e9);
 	fprintf(st->fh, "bstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
 	        bsx/1e9, bsy/1e9, bsz/1e9);
 	fprintf(st->fh, "cstar = %+9.7f %+9.7f %+9.7f nm^-1\n",
-	        csx/1e9, csy/1e9, csz/1e9);
+		csx/1e9, csy/1e9, csz/1e9);
 
 	fprintf(st->fh, "lattice_type = %s\n",
-	        str_lattice(cell_get_lattice_type(cell)));
+		str_lattice(cell_get_lattice_type(cell)));
 	fprintf(st->fh, "centering = %c\n", cell_get_centering(cell));
 	fprintf(st->fh, "unique_axis = %c\n", cell_get_unique_axis(cell));
 
 	rad = crystal_get_profile_radius(cr);
 	fprintf(st->fh, "profile_radius = %.5f nm^-1\n", rad/1e9);
 
+	if ( crystal_get_notes(cr) != NULL ) {
+		fprintf(st->fh, "%s\n", crystal_get_notes(cr));
+	}
+
 	reflist = crystal_get_reflections(cr);
 	if ( reflist != NULL ) {
 
 		fprintf(st->fh, "diffraction_resolution_limit"
-		                " = %.2f nm^-1 or %.2f A\n",
-		                crystal_get_resolution_limit(cr)/1e9,
-		                1e10 / crystal_get_resolution_limit(cr));
+				" = %.2f nm^-1 or %.2f A\n",
+				crystal_get_resolution_limit(cr)/1e9,
+				1e10 / crystal_get_resolution_limit(cr));
 
 		fprintf(st->fh, "num_reflections = %i\n",
 		                num_integrated_reflections(reflist));
 		fprintf(st->fh, "num_saturated_reflections = %lli\n",
 		                crystal_get_num_saturated_reflections(cr));
+		fprintf(st->fh, "num_implausible_reflections = %lli\n",
+		                crystal_get_num_implausible_reflections(cr));
 
 	}
 
@@ -393,13 +734,25 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 
 		if ( reflist != NULL ) {
 
+			struct image *image;
+
+			image = crystal_get_image(cr);
+
 			fprintf(st->fh, REFLECTION_START_MARKER"\n");
-			if ( AT_LEAST_VERSION(st, 2, 2) ) {
-				write_stream_reflections(st->fh, reflist);
+			if ( AT_LEAST_VERSION(st, 2, 3) ) {
+				ret = write_stream_reflections_2_3(st->fh,
+				                                   reflist,
+				                                   image);
+			} else if ( AT_LEAST_VERSION(st, 2, 2) ) {
+				ret = write_stream_reflections_2_2(st->fh,
+			                                           reflist,
+			                                           image);
 			} else {
 				/* This function writes like a normal reflection
 				 * list was written in stream 2.1 */
-				write_stream_reflections_2_1(st->fh, reflist);
+				ret = write_stream_reflections_2_1(st->fh,
+				                                   reflist,
+				                                   image);
 			}
 			fprintf(st->fh, REFLECTION_END_MARKER"\n");
 
@@ -411,18 +764,26 @@ static void write_crystal(Stream *st, Crystal *cr, int include_reflections)
 	}
 
 	fprintf(st->fh, CRYSTAL_END_MARKER"\n");
+
+	return ret;
 }
 
 
-void write_chunk(Stream *st, struct image *i, struct hdfile *hdfile,
-                 int include_peaks, int include_reflections)
+int write_chunk(Stream *st, struct image *i, struct hdfile *hdfile,
+                int include_peaks, int include_reflections, struct event* ev)
 {
 	int j;
 	char *indexer;
+	int ret = 0;
 
 	fprintf(st->fh, CHUNK_START_MARKER"\n");
 
 	fprintf(st->fh, "Image filename: %s\n", i->filename);
+	if ( i->event != NULL ) {
+		fprintf(st->fh, "Event: %s\n", get_event_string(i->event));
+	}
+
+	fprintf(st->fh, "Image serial number: %i\n", i->serial);
 
 	indexer = indexer_str(i->indexed_by);
 	fprintf(st->fh, "indexed_by = %s\n", indexer);
@@ -431,10 +792,10 @@ void write_chunk(Stream *st, struct image *i, struct hdfile *hdfile,
 	fprintf(st->fh, "photon_energy_eV = %f\n",
 	        J_to_eV(ph_lambda_to_en(i->lambda)));
 
-	fprintf(st->fh, "beam_divergence = %.5f mrad\n", i->div*1e3);
-	fprintf(st->fh, "beam_bandwidth = %.5f %%\n", i->bw*100.0);
+	fprintf(st->fh, "beam_divergence = %.2e rad\n", i->div);
+	fprintf(st->fh, "beam_bandwidth = %.2e (fraction)\n", i->bw);
 
-	copy_hdf5_fields(hdfile, i->copyme, st->fh);
+	copy_hdf5_fields(hdfile, i->copyme, st->fh, ev);
 
 	if ( i->det != NULL ) {
 
@@ -474,16 +835,25 @@ void write_chunk(Stream *st, struct image *i, struct hdfile *hdfile,
 	fprintf(st->fh, "num_peaks = %lli\n", i->num_peaks);
 	fprintf(st->fh, "num_saturated_peaks = %lli\n", i->num_saturated_peaks);
 	if ( include_peaks ) {
-		write_peaks(i, st->fh);
+		if ( AT_LEAST_VERSION(st, 2, 3) ) {
+			ret = write_peaks_2_3(i, st->fh);
+		} else {
+			ret = write_peaks(i, st->fh);
+		}
 	}
 
 	for ( j=0; j<i->n_crystals; j++ ) {
-		write_crystal(st, i->crystals[j], include_reflections);
+		if ( crystal_get_user_flag(i->crystals[j]) == 0 ) {
+			ret = write_crystal(st, i->crystals[j],
+			                    include_reflections);
+		}
 	}
 
 	fprintf(st->fh, CHUNK_END_MARKER"\n");
 
 	fflush(st->fh);
+
+	return ret;
 }
 
 
@@ -525,6 +895,10 @@ static void read_crystal(Stream *st, struct image *image, StreamReadFlags srf)
 	int n;
 	Crystal **crystals_new;
 
+	as.u = 0.0;  as.v = 0.0;  as.w = 0.0;
+	bs.u = 0.0;  bs.v = 0.0;  bs.w = 0.0;
+	cs.u = 0.0;  cs.v = 0.0;  cs.w = 0.0;
+
 	cr = crystal_new();
 	if ( cr == NULL ) {
 		ERROR("Failed to allocate crystal!\n");
@@ -606,7 +980,7 @@ static void read_crystal(Stream *st, struct image *image, StreamReadFlags srf)
 			crystal_set_resolution_limit(cr, lim*1e9);
 		}
 
-		if ( sscanf(line, "profile_radius = %f nm^-1", &rad) == 1 ) {
+		if ( sscanf(line, "profile_radius = %e nm^-1", &rad) == 1 ) {
 			crystal_set_profile_radius(cr, rad*1e9);
 		}
 
@@ -618,10 +992,15 @@ static void read_crystal(Stream *st, struct image *image, StreamReadFlags srf)
 
 			/* The reflection list format in the stream diverges
 			 * after 2.2 */
-			if ( AT_LEAST_VERSION(st, 2, 2) ) {
-				reflist = read_stream_reflections(st->fh);
+			if ( AT_LEAST_VERSION(st, 2, 3) ) {
+				reflist = read_stream_reflections_2_3(st->fh,
+                                                      image->det);
+			} else if ( AT_LEAST_VERSION(st, 2, 2) ) {
+				reflist = read_stream_reflections_2_2(st->fh,
+				          image->det);
 			} else {
-				reflist = read_stream_reflections_2_1(st->fh);
+				reflist = read_stream_reflections_2_1(st->fh,
+				          image->det);
 			}
 			if ( reflist == NULL ) {
 				ERROR("Failed while reading reflections\n");
@@ -680,6 +1059,39 @@ static void read_crystal(Stream *st, struct image *image, StreamReadFlags srf)
 }
 
 
+static int read_and_store_hdf5_field(struct image *image, const char *line)
+{
+
+	char **new_fields;
+
+	if ( image->stuff_from_stream == NULL ) {
+		image->stuff_from_stream =
+		       malloc(sizeof(struct stuff_from_stream));
+		if ( image->stuff_from_stream == NULL) {
+			ERROR("Failed reading hdf5 entries from "
+			      "stream\n");
+			return 1;
+		}
+		image->stuff_from_stream->fields = NULL;
+		image->stuff_from_stream->n_fields = 0;
+	}
+
+	new_fields = realloc(image->stuff_from_stream->fields,
+			     (1+image->stuff_from_stream->n_fields)*
+			     sizeof(char *));
+	if ( new_fields == NULL ) {
+		ERROR("Failed reading hdf5 entries from stream\n");
+		return 1;
+	}
+	image->stuff_from_stream->fields = new_fields;
+	image->stuff_from_stream->fields[image->stuff_from_stream->n_fields]
+							     = strdup(line);
+	image->stuff_from_stream->n_fields++;
+
+	return 0;
+}
+
+
 /* Read the next chunk from a stream and fill in 'image' */
 int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 {
@@ -694,6 +1106,8 @@ int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 	image->features = NULL;
 	image->crystals = NULL;
 	image->n_crystals = 0;
+	image->event = NULL;
+	image->stuff_from_stream = NULL;
 
 	if ( (srf & STREAM_READ_REFLECTIONS) || (srf & STREAM_READ_UNITCELL) ) {
 		srf |= STREAM_READ_CRYSTALS;
@@ -701,6 +1115,7 @@ int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 
 	do {
 		long long num_peaks;
+		int ser;
 		float div, bw;
 
 		rval = fgets(line, 1023, st->fh);
@@ -715,6 +1130,10 @@ int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 			have_filename = 1;
 		}
 
+		if ( strncmp(line, "Event: ", 7) == 0 ) {
+			image->event = get_event_from_event_string(line+7);
+		}
+
 		if ( strncmp(line, "indexed_by = ", 13) == 0 ) {
 			IndexingMethod *list;
 			list = build_indexer_list(line+13);
@@ -728,18 +1147,22 @@ int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 			have_ev = 1;
 		}
 
-		if ( sscanf(line, "beam_divergence = %f mrad", &div) == 1 ) {
-			image->div = div/1e3;
+		if ( sscanf(line, "beam_divergence = %e rad", &div) == 1 ) {
+			image->div = div;
 		}
 
-		if ( sscanf(line, "beam_bandwidth = %f %%", &bw) == 1 ) {
-			image->bw = bw/100.0;
+		if ( sscanf(line, "beam_bandwidth = %f", &bw) == 1 ) {
+			image->bw = bw;
 		}
 
-		if ( sscanf(line, "num_peaks = %lld %%", &num_peaks) == 1 ) {
+		if ( sscanf(line, "num_peaks = %lld", &num_peaks) == 1 ) {
 			image->num_peaks = num_peaks;
 		}
 
+		if ( sscanf(line, "Image serial number: %i", &ser) == 1 ) {
+			image->serial = ser;
+		}
+
 		if ( strncmp(line, "camera_length_", 14) == 0 ) {
 			if ( image->det != NULL ) {
 
@@ -766,17 +1189,35 @@ int read_chunk_2(Stream *st, struct image *image,  StreamReadFlags srf)
 			}
 		}
 
+		if ( strncmp(line, "hdf5", 3) == 0 ) {
+
+			int fail;
+
+			fail = read_and_store_hdf5_field(image, line);
+			if ( fail ) {
+				ERROR("Failed to read hd5 fields from stream.\n");
+				return 1;
+			}
+		}
+
 		if ( (srf & STREAM_READ_PEAKS)
-		  && (strcmp(line, PEAK_LIST_START_MARKER) == 0) ) {
-			if ( read_peaks(st->fh, image) ) {
+		    && strcmp(line, PEAK_LIST_START_MARKER) == 0 ) {
+
+			int fail;
+
+			if ( AT_LEAST_VERSION(st, 2, 3) ) {
+				fail = read_peaks_2_3(st->fh, image);
+			} else {
+				fail = read_peaks(st->fh, image);
+			}
+			if ( fail ) {
 				ERROR("Failed while reading peaks\n");
 				return 1;
 			}
 		}
 
 		if ( (srf & STREAM_READ_CRYSTALS)
-		  && (strcmp(line, CRYSTAL_START_MARKER) == 0) )
-		{
+		  && (strcmp(line, CRYSTAL_START_MARKER) == 0) ) {
 			read_crystal(st, image, srf);
 		}
 
@@ -827,7 +1268,12 @@ Stream *open_stream_for_read(const char *filename)
 	st = malloc(sizeof(struct _stream));
 	if ( st == NULL ) return NULL;
 
-	st->fh = fopen(filename, "r");
+	if ( strcmp(filename, "-") == 0 ) {
+		st->fh = stdin;
+	} else {
+		st->fh = fopen(filename, "r");
+	}
+
 	if ( st->fh == NULL ) {
 		free(st);
 		return NULL;
@@ -852,6 +1298,9 @@ Stream *open_stream_for_read(const char *filename)
 	} else if ( strncmp(line, "CrystFEL stream format 2.2", 26) == 0 ) {
 		st->major_version = 2;
 		st->minor_version = 2;
+	} else if ( strncmp(line, "CrystFEL stream format 2.3", 26) == 0 ) {
+		st->major_version = 2;
+		st->minor_version = 3;
 	} else {
 		ERROR("Invalid stream, or stream format is too new.\n");
 		close_stream(st);
@@ -1063,8 +1512,11 @@ void write_geometry_file(Stream *st, const char *geom_filename) {
 
 	do {
 		rval = fgets(line, 1023, geom_fh);
-		fputs(line, st->fh);
+		if ( rval != NULL ) fputs(line, st->fh);
 	} while ( rval != NULL );
+
+	fclose(geom_fh);
+
 	fprintf(st->fh, GEOM_END_MARKER"\n");
 	fflush(st->fh);
 }
@@ -1086,3 +1538,4 @@ int rewind_stream(Stream *st)
 {
 	return fseek(st->fh, 0, SEEK_SET);
 }
+
