@@ -3,12 +3,12 @@
  *
  * Invoke xds for crystal autoindexing
  *
- * Copyright © 2013-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2013-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2013 Cornelius Gati
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *        2013 Cornelius Gati <cornelius.gati@cfel.de>
  *
  * This file is part of CrystFEL.
@@ -45,18 +45,11 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 
-#ifdef HAVE_CLOCK_GETTIME
-#include <time.h>
-#else
-#include <sys/time.h>
-#endif
-
 #include "cell.h"
 #include "image.h"
 #include "utils.h"
 #include "peaks.h"
 #include "detector.h"
-#include "beam-parameters.h"
 #include "cell-utils.h"
 
 
@@ -520,11 +513,15 @@ int run_xds(struct image *image, IndexingPrivate *priv)
 
 	if ( write_inp(image, xp) ) {
 		ERROR("Failed to write XDS.INP file for XDS.\n");
+		free(xds);
 		return 0;
 	}
 
 	n = image_feature_count(image->features);
-	if (n < 25) return 0;
+	if ( n < 25 ) {
+		free(xds);
+		return 0;
+	}
 
 	write_spot(image);
 
@@ -619,19 +616,34 @@ int run_xds(struct image *image, IndexingPrivate *priv)
 
 
 IndexingPrivate *xds_prepare(IndexingMethod *indm, UnitCell *cell,
-                             struct detector *det, struct beam_params *beam,
-                             float *ltl)
+                             struct detector *det, float *ltl)
 {
 	struct xds_private *xp;
+	int need_cell = 0;
+
+	/* Check if cell parameters are needed/provided */
+	if ( *indm & INDEXING_CHECK_CELL_COMBINATIONS ) need_cell = 1;
+	if ( *indm & INDEXING_CHECK_CELL_AXES ) need_cell = 1;
+	if ( *indm & INDEXING_USE_CELL_PARAMETERS ) need_cell = 1;
+	if ( need_cell && !cell_has_parameters(cell) ) {
+		ERROR("Altering your XDS flags because cell parameters were"
+		      " not provided.\n");
+		*indm &= ~INDEXING_CHECK_CELL_COMBINATIONS;
+		*indm &= ~INDEXING_CHECK_CELL_AXES;
+		*indm &= ~INDEXING_USE_CELL_PARAMETERS;
+	}
 
 	/* Either cell,latt and cell provided, or nocell-nolatt and no cell
 	 * - complain about anything else.  Could figure this out automatically,
 	 * but we'd have to decide whether the user just forgot the cell, or
 	 * forgot "-nolatt", or whatever. */
 	if ( ((*indm & INDEXING_USE_LATTICE_TYPE)
-	  || (*indm & INDEXING_USE_CELL_PARAMETERS)) && (cell == NULL) ) {
-		ERROR("No cell provided.  If you wanted to use XDS without "
-		      "prior cell information, use xds-nolatt-nocell.\n");
+	  || (*indm & INDEXING_USE_CELL_PARAMETERS))
+	  && !cell_has_parameters(cell) )
+	{
+		ERROR("No cell parameters provided.  If you wanted to use XDS "
+		      "without prior cell information, use "
+		      "xds-nolatt-nocell.\n");
 		return NULL;
 	}
 
