@@ -3,13 +3,13 @@
  *
  * Resolve indexing ambiguities
  *
- * Copyright © 2014 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2014-2015 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  * Copyright © 2014 Wolfgang Brehm
  *
  * Authors:
- *   2014 Thomas White <taw@physics.org>
- *   2014 Wolfgang Brehm <wolfgang.brehm@gmail.com>
+ *   2014-2015 Thomas White <taw@physics.org>
+ *   2014      Wolfgang Brehm <wolfgang.brehm@gmail.com>
  *
  * This file is part of CrystFEL.
  *
@@ -604,7 +604,7 @@ static void detwin(struct cc_list *ccs, int n_crystals, int *assignments,
 	}
 
 	if ( ndud > 0 ) {
-		STATUS("Warning: %i crystals had no correlation\n", ndud);
+		STATUS("WARNING: %i crystals had no correlation\n", ndud);
 	}
 
 	STATUS("Mean f,g = %10f,%10f. Changed %i assignments this time.\n",
@@ -1049,18 +1049,6 @@ int main(int argc, char *argv[])
 
 	}
 
-	if ( argc != (optind+1) ) {
-		ERROR("Please provide exactly one stream filename.\n");
-		return 1;
-	}
-
-	infile = argv[optind++];
-	st = open_stream_for_read(infile);
-	if ( st == NULL ) {
-		ERROR("Failed to open input stream '%s'\n", infile);
-		return 1;
-	}
-
 	if ( s_sym_str == NULL ) {
 		ERROR("You must specify the input symmetry (with -y)\n");
 		return 1;
@@ -1111,6 +1099,19 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	if ( argc != (optind+1) ) {
+		ERROR("Please provide exactly one stream filename.\n");
+		return 1;
+	}
+
+	infile = argv[optind++];
+	st = open_stream_for_read(infile);
+	if ( st == NULL ) {
+		ERROR("Failed to open input stream '%s'\n", infile);
+		return 1;
+	}
+
+
 	crystals = NULL;
 	n_crystals = 0;
 	max_crystals = 0;
@@ -1122,7 +1123,8 @@ int main(int argc, char *argv[])
 
 		cur.det = NULL;
 
-		if ( read_chunk(st, &cur) != 0 ) {
+		if ( read_chunk_2(st, &cur, STREAM_READ_UNITCELL
+		                          | STREAM_READ_REFLECTIONS) != 0 ) {
 			break;
 		}
 
@@ -1306,7 +1308,7 @@ int main(int argc, char *argv[])
 		write_reindexed_stream(infile, outfile, assignments, amb);
 	} else if ( outfile != NULL ) {
 		ERROR("Can only write stream with known ambiguity operator.\n");
-		ERROR("Try again with -w\n");
+		ERROR("Try again with -w or --operator.\n");
 	}
 
 	free(assignments);
