@@ -3,11 +3,11 @@
  *
  * Characterise reflection lists
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -57,7 +57,7 @@ static void show_help(const char *s)
 "  -h, --help                 Display this help message.\n"
 "      --version              Print CrystFEL version number and exit.\n"
 "  -y, --symmetry=<sym>       The symmetry of the input file.\n"
-"  -p, --pdb=<filename>       PDB file to use.\n"
+"  -p, --pdb=<filename>       Unit cell file to use (PDB or CrystFEL format).\n"
 "      --rmin=<res>           Low resolution cutoff (1/d in m^-1).\n"
 "      --rmax=<res>           High resolution cutoff (1/d in m^-1).\n"
 "      --lowres=<n>           Low resolution cutoff in (d in A).\n"
@@ -444,7 +444,7 @@ static void plot_shells(RefList *list, UnitCell *cell, const SymOpList *sym,
 
 	fh = fopen(shell_file, "w");
 	if ( fh == NULL ) {
-		ERROR("Couldn't open 'shells.dat'\n");
+		ERROR("Couldn't open '%s'\n", shell_file);
 		return;
 	}
 
@@ -614,23 +614,24 @@ static void plot_shells(RefList *list, UnitCell *cell, const SymOpList *sym,
 	STATUS("%i reflections in total.\n", nrefl);
 
 	if ( nout ) {
-		STATUS("Warning; %i reflections outside resolution range.\n",
+		STATUS("WARNING; %i reflections outside resolution range.\n",
 		       nout);
 	}
 
 	if ( nsilly ) {
-		STATUS("Warning; %i reflections had infinite or invalid values"
+		STATUS("WARNING; %i reflections had infinite or invalid values"
 		       " of I/sigma(I).\n", nsilly);
 	}
 
-	fprintf(fh, "1/d centre   # refs Possible  Compl       "
-		    "Meas   Red   SNR    Std dev       Mean     d(A)\n");
+	fprintf(fh, "Center 1/nm  # refs Possible  Compl       "
+		    "Meas   Red   SNR    Std dev       Mean     d(A)    "
+		    "Min 1/nm   Max 1/nm\n");
 	for ( i=0; i<nshells; i++ ) {
 
 		double cen;
 		cen = rmins[i] + (rmaxs[i] - rmins[i])/2.0;
 		fprintf(fh, "%10.3f %8i %8i %6.2f %10i %5.1f"
-		            " %5.2f %10.2f %10.2f %8.2f\n",
+		            " %5.2f %10.2f %10.2f %8.2f  %10.3f %10.3f\n",
 		        cen*1.0e-9,
 		        measured[i],
 		        possible[i],
@@ -639,13 +640,14 @@ static void plot_shells(RefList *list, UnitCell *cell, const SymOpList *sym,
 		        (double)measurements[i]/measured[i],
 		        snr[i]/(double)snr_measured[i],
 		        sqrt(var[i]/measured[i]),
-		        mean[i], (1.0/cen)*1e10);
+		        mean[i], (1.0/cen)*1e10,
+			rmins[i]*1.0e-9, rmaxs[i]*1.0e-9);
 
 	}
 
 	fclose(fh);
 
-	STATUS("Resolution shell information written to shells.dat.\n");
+	STATUS("Resolution shell information written to %s.\n", shell_file);
 
 	free(possible);
 	free(measurements);
@@ -692,7 +694,7 @@ int main(int argc, char *argv[])
 	RefList *list;
 	Reflection *refl;
 	RefListIterator *iter;
-	char *pdb = NULL;
+	char *cellfile = NULL;
 	int rej = 0;
 	float rmin_fix = -1.0;
 	float rmax_fix = -1.0;
@@ -751,7 +753,7 @@ int main(int argc, char *argv[])
 			break;
 
 			case 'p' :
-			pdb = strdup(optarg);
+			cellfile = strdup(optarg);
 			break;
 
 			case 0 :
@@ -833,13 +835,16 @@ int main(int argc, char *argv[])
 
 	file = strdup(argv[optind++]);
 
-	if ( pdb == NULL ) {
-		ERROR("You need to provide a PDB file containing"
-		       " the unit cell.\n");
+	if ( cellfile == NULL ) {
+		ERROR("You need to provide a unit cell.\n");
 		return 1;
 	}
-	cell = load_cell_from_pdb(pdb);
-	free(pdb);
+	cell = load_cell_from_file(cellfile);
+	if ( cell == NULL ) {
+		ERROR("Failed to load cell.\n");
+		return 1;
+	}
+	free(cellfile);
 
 	raw_list = read_reflections(file);
 	if ( raw_list == NULL ) {
@@ -854,6 +859,11 @@ int main(int argc, char *argv[])
 	if ( check_list_symmetry(raw_list, sym) ) {
 		ERROR("The input reflection list does not appear to"
 		      " have symmetry %s\n", symmetry_name(sym));
+		if ( cell_get_lattice_type(cell) == L_MONOCLINIC ) {
+			ERROR("You may need to specify the unique axis in your "
+			      "point group.  The default is unique axis c.\n");
+			ERROR("See 'man crystfel' for more details.\n");
+		}
 		return 1;
 	}
 
