@@ -3,11 +3,11 @@
  *
  * Compare reflection lists
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *   2013      Lorenzo Galli <lorenzo.galli@desy.de>
  *
  * This file is part of CrystFEL.
@@ -41,6 +41,7 @@
 #include <assert.h>
 #include <gsl/gsl_errno.h>
 #include <gsl/gsl_statistics.h>
+#include <gsl/gsl_fit.h>
 
 #include "version.h"
 #include "utils.h"
@@ -60,7 +61,9 @@ enum fom
 	FOM_CCANO,
 	FOM_CRDANO,
 	FOM_RANO,
-	FOM_RANORSPLIT
+	FOM_RANORSPLIT,
+	FOM_D1SIG,
+	FOM_D2SIG
 };
 
 
@@ -76,6 +79,8 @@ static enum fom get_fom(const char *s)
 	if ( strcasecmp(s, "crdano") == 0 ) return FOM_CRDANO;
 	if ( strcasecmp(s, "rano") == 0 ) return FOM_RANO;
 	if ( strcasecmp(s, "rano/rsplit") == 0 ) return FOM_RANORSPLIT;
+	if ( strcasecmp(s, "d1sig") == 0 ) return FOM_D1SIG;
+	if ( strcasecmp(s, "d2sig") == 0 ) return FOM_D2SIG;
 
 	ERROR("Unknown figure of merit '%s'.\n", s);
 	exit(1);
@@ -89,10 +94,11 @@ static void show_help(const char *s)
 "Compare intensity lists.\n"
 "\n"
 "  -y, --symmetry=<sym>       The symmetry of both the input files.\n"
-"  -p, --pdb=<filename>       PDB file to use.\n"
+"  -p, --pdb=<filename>       Unit cell file to use.\n"
 "      --fom=<FoM>            Calculate this figure of merit  Choose from:\n"
 "                              R1I, R1F, R2, Rsplit, CC, CCstar,\n"
-"			       CCano, CRDano, Rano, Rano/Rsplit\n"
+"			       CCano, CRDano, Rano, Rano/Rsplit, d1sig,\n"
+"                              d2sig\n"
 "      --nshells=<n>          Use <n> resolution shells.\n"
 "  -u                         Force scale factor to 1.\n"
 "      --shell-file=<file>    Write resolution shells to <file>.\n"
@@ -118,6 +124,7 @@ struct fom_context
 {
 	enum fom fom;
 	int nshells;
+	int *cts;
 
 	/* For R-factors */
 	double *num;
@@ -132,6 +139,9 @@ struct fom_context
 	double **vec2;
 	int *n;
 	int nmax;
+
+	/* For "counting" things e.g. d1sig or d2sig */
+	int *n_within;
 };
 
 
@@ -145,6 +155,10 @@ static struct fom_context *init_fom(enum fom fom, int nmax, int nshells)
 
 	fctx->fom = fom;
 	fctx->nshells = nshells;
+	fctx->cts = malloc(nshells*sizeof(int));
+	for ( i=0; i<nshells; i++ ) {
+		fctx->cts[i] = 0;
+	}
 
 	switch ( fctx->fom ) {
 
@@ -190,10 +204,18 @@ static struct fom_context *init_fom(enum fom fom, int nmax, int nshells)
 		for ( i=0; i<nshells; i++ ) {
 			fctx->n[i] = 0;
 		}
-
 		fctx->nmax = nmax;
 		break;
 
+		case FOM_D1SIG :
+		case FOM_D2SIG :
+		fctx->n_within = malloc(nshells*sizeof(int));
+		if ( fctx->n_within == NULL ) return NULL;
+		for ( i=0; i<nshells; i++ ) {
+			fctx->n_within[i] = 0;
+		}
+		break;
+
 	}
 
 	return fctx;
@@ -201,11 +223,14 @@ static struct fom_context *init_fom(enum fom fom, int nmax, int nshells)
 
 
 static void add_to_fom(struct fom_context *fctx, double i1, double i2,
-                       double i1bij, double i2bij, int bin)
+                       double i1bij, double i2bij, double sig1, double sig2,
+                       int bin)
 {
 	double f1, f2;
 	double im, imbij;
 
+	fctx->cts[bin]++;
+
 	/* Negative intensities have already been weeded out. */
 	f1 = sqrt(i1);
 	f2 = sqrt(i2);
@@ -260,6 +285,17 @@ static void add_to_fom(struct fom_context *fctx, double i1, double i2,
 		fctx->den[bin] += im + imbij;
 		break;
 
+		case FOM_D1SIG :
+		if ( fabs(i1-i2) < (sig1+sig2)/2.0 ) {
+			fctx->n_within[bin]++;
+		}
+		break;
+
+		case FOM_D2SIG :
+		if ( fabs(i1-i2) < sig1+sig2 ) {  /* = 2 * (sig1+sig2)/2 */
+			fctx->n_within[bin]++;
+		}
+		break;
 
 	}
 }
@@ -357,6 +393,16 @@ static double fom_overall(struct fom_context *fctx)
 		free(overall_perpend_diagonal);
 		break;
 
+		case FOM_D1SIG :
+		case FOM_D2SIG :
+		overall_num = 0.0;
+		overall_den = 0.0;
+		for ( i=0; i<fctx->nshells; i++ ) {
+			overall_num += fctx->n_within[i];
+			overall_den += fctx->cts[i];
+		}
+		break;
+
 	}
 
 	switch ( fctx->fom ) {
@@ -386,6 +432,10 @@ static double fom_overall(struct fom_context *fctx)
 		return (2.0*(overall_num/overall_den)) /
 		       (2.0*(overall_num2/overall_den2) / sqrt(2.0));
 
+		case FOM_D1SIG :
+		case FOM_D2SIG :
+		return overall_num/overall_den;
+
 	}
 
 	ERROR("This point is never reached.\n");
@@ -448,6 +498,10 @@ static double fom_shell(struct fom_context *fctx, int i)
 		free(perpend_diagonal);
 		return sqrt(variance_signal / variance_error);
 
+		case FOM_D1SIG :
+		case FOM_D2SIG :
+		return (double)fctx->n_within[i] / fctx->cts[i];
+
 	}
 
 	ERROR("This point is never reached.\n");
@@ -608,39 +662,135 @@ static int get_bin(struct shells *s, Reflection *refl, UnitCell *cell)
 }
 
 
+static int wilson_scale(RefList *list1, RefList *list2, UnitCell *cell)
+{
+	Reflection *refl1;
+	Reflection *refl2;
+	RefListIterator *iter;
+	int max_n = 256;
+	int n = 0;
+	double *x;
+	double *y;
+	int r;
+	double G, B;
+	double c0, c1, cov00, cov01, cov11, chisq;
+
+	x = malloc(max_n*sizeof(double));
+	y = malloc(max_n*sizeof(double));
+	if ( (x==NULL) || (y==NULL) ) {
+		ERROR("Failed to allocate memory for scaling.\n");
+		return 1;
+	}
+
+	for ( refl1 = first_refl(list1, &iter);
+	      refl1 != NULL;
+	      refl1 = next_refl(refl1, iter) )
+	{
+		signed int h, k, l;
+		double Ih1, Ih2;
+		double res;
+
+		get_indices(refl1, &h, &k, &l);
+		res = resolution(cell, h, k, l);
+
+		refl2 = find_refl(list2, h, k, l);
+		assert(refl2 != NULL);
+
+		Ih1 = get_intensity(refl1);
+		Ih2 = get_intensity(refl2);
+
+		if ( (Ih1 <= 0.0) || (Ih2 <= 0.0) ) continue;
+		if ( isnan(Ih1) || isinf(Ih1) ) continue;
+		if ( isnan(Ih2) || isinf(Ih2) ) continue;
+
+		if ( n == max_n ) {
+			max_n *= 2;
+			x = realloc(x, max_n*sizeof(double));
+			y = realloc(y, max_n*sizeof(double));
+			if ( (x==NULL) || (y==NULL) ) {
+				ERROR("Failed to allocate memory for scaling.\n");
+				return 1;
+			}
+		}
+
+		x[n] = res*res;
+		y[n] = log(Ih1/Ih2);
+		n++;
+
+	}
+
+	if ( n < 2 ) {
+		ERROR("Not enough reflections for scaling\n");
+		return 1;
+	}
+
+	r = gsl_fit_linear(x, 1, y, 1, n, &c0, &c1,
+	                     &cov00, &cov01, &cov11, &chisq);
+
+	if ( r ) {
+		ERROR("Scaling failed.\n");
+		return 1;
+	}
+
+	G = exp(c0);
+	B = c1/2.0;
+
+	STATUS("Relative scale factor = %f, relative B factor = %f A^2\n",
+	       G, B*1e20);
+	STATUS("A scale factor greater than 1 means that the second reflection "
+	       "list is weaker than the first.\n");
+	STATUS("A positive relative B factor means that the second reflection "
+	       "list falls off with resolution more quickly than the first.\n");
+
+	free(x);
+	free(y);
+
+	/* Apply the scaling factor */
+	for ( refl2 = first_refl(list2, &iter);
+	      refl2 != NULL;
+	      refl2 = next_refl(refl2, iter) )
+	{
+		signed int h, k, l;
+		double res;
+		double corr;
+
+		get_indices(refl2, &h, &k, &l);
+		res = resolution(cell, h, k, l);
+
+		corr = G * exp(2.0*B*res*res);
+		set_intensity(refl2, get_intensity(refl2)*corr);
+		set_esd_intensity(refl2, get_esd_intensity(refl2)*corr);
+
+	}
+	return 0;
+}
+
+
 static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
                    double rmin, double rmax, enum fom fom,
                    int config_unity, int nshells, const char *filename,
                    int config_intshells, double min_I, double max_I,
                    SymOpList *sym)
 {
-	int *cts;
 	int i;
 	Reflection *refl1;
 	RefListIterator *iter;
 	FILE *fh;
-	double scale;
 	struct fom_context *fctx;
 	struct shells *shells;
 	const char *t1, *t2;
 	int n_out;
 
-	cts = malloc(nshells*sizeof(int));
 	fctx = init_fom(fom, num_reflections(list1), nshells);
 
-	if ( (fctx==NULL) || (cts==NULL) ) {
+	if ( fctx==NULL ) {
 		ERROR("Couldn't allocate memory for resolution shells.\n");
 		return;
 	}
 
-	for ( i=0; i<nshells; i++ ) {
-		cts[i] = 0;
-	}
-
-	if ( config_unity ) {
-		scale = 1.0;
-	} else {
-		scale = stat_scale_intensity(list1, list2);
+	if ( !config_unity && wilson_scale(list1, list2, cell) ) {
+		ERROR("Error with scaling.\n");
+		return;
 	}
 
 	/* Calculate the bins */
@@ -664,6 +814,7 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		int bin;
 		double i1, i2;
 		double i1bij, i2bij;
+		double sig1, sig2;
 		Reflection *refl2;
 
 		get_indices(refl1, &h, &k, &l);
@@ -678,7 +829,9 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		}
 
 		i1 = get_intensity(refl1);
-		i2 = scale * get_intensity(refl2);
+		i2 = get_intensity(refl2);
+		sig1 = get_esd_intensity(refl1);
+		sig2 = get_esd_intensity(refl2);
 
 		if ( (fom == FOM_CCANO) || (fom == FOM_CRDANO)
 		  || (fom == FOM_RANO) || (fom == FOM_RANORSPLIT) )
@@ -704,7 +857,7 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 			assert(refl2_bij != NULL);
 
 			i1bij = get_intensity(refl1_bij);
-			i2bij = scale * get_intensity(refl2_bij);
+			i2bij = get_intensity(refl2_bij);
 
 		} else {
 
@@ -714,11 +867,10 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 
 		}
 
-		add_to_fom(fctx, i1, i2, i1bij, i2bij, bin);
-		cts[bin]++;
+		add_to_fom(fctx, i1, i2, i1bij, i2bij, sig1, sig2, bin);
 	}
 	if ( n_out)  {
-		ERROR("Warning: %i reflection pairs outside range.\n", n_out);
+		ERROR("WARNING: %i reflection pairs outside range.\n", n_out);
 	}
 
 	switch ( fom ) {
@@ -763,6 +915,17 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		STATUS("Overall Rano/Rsplit =  %.7f\n", fom_overall(fctx));
 		break;
 
+		case FOM_D1SIG :
+		STATUS("Fraction of differences less than 1 sigma = %.7f %%\n",
+		       100.0*fom_overall(fctx));
+		break;
+
+		case FOM_D2SIG :
+		STATUS("Fraction of differences less than 2 sigma = %.7f %%\n",
+		       100.0*fom_overall(fctx));
+		break;
+
+
 	}
 
 	fh = fopen(filename, "w");
@@ -776,7 +939,7 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		t2 = "";
 	} else {
 		t1 = "  1/d centre";
-		t2 = "      d / A";
+		t2 = "      d / A   Min 1/nm    Max 1/nm";
 	}
 
 	switch ( fom ) {
@@ -821,6 +984,14 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		fprintf(fh, "%s Rano/Rsplit       nref%s\n", t1, t2);
 		break;
 
+		case FOM_D1SIG :
+		fprintf(fh, "%s D<1sigma/%%     nref%s\n", t1, t2);
+		break;
+
+		case FOM_D2SIG :
+		fprintf(fh, "%s D<2sigma/%%     nref%s\n", t1, t2);
+		break;
+
 	}
 
 	for ( i=0; i<nshells; i++ ) {
@@ -839,10 +1010,14 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		case FOM_RANO :
 		if ( config_intshells ) {
 			fprintf(fh, "%10.3f %10.2f %10i\n",
-				cen, r*100.0, cts[i]);
+				cen, r*100.0, fctx->cts[i]);
 		} else {
-			fprintf(fh, "%10.3f %10.2f %10i %10.2f\n",
-				cen*1.0e-9, r*100.0, cts[i], (1.0/cen)*1e10);
+			fprintf(fh, "%10.3f %10.2f %10i %10.2f "
+			            "%10.3f  %10.3f\n",
+			        cen*1.0e-9, r*100.0, fctx->cts[i],
+			        (1.0/cen)*1e10,
+			        shells->rmins[i]*1.0e-9,
+			        shells->rmaxs[i]*1.0e-9);
 		}
 		break;
 
@@ -852,23 +1027,43 @@ static void do_fom(RefList *list1, RefList *list2, UnitCell *cell,
 		case FOM_CRDANO :
 		if ( config_intshells ) {
 			fprintf(fh, "%10.3f %10.7f %10i\n",
-				cen, r, cts[i]);
+				cen, r, fctx->cts[i]);
 		} else {
-			fprintf(fh, "%10.3f %10.7f %10i %10.2f\n",
-				cen*1.0e-9, r, cts[i], (1.0/cen)*1e10);
+			fprintf(fh, "%10.3f %10.7f %10i %10.2f "
+			            "%10.3f  %10.3f\n",
+			        cen*1.0e-9, r, fctx->cts[i], (1.0/cen)*1e10,
+			        shells->rmins[i]*1.0e-9,
+			        shells->rmaxs[i]*1.0e-9);
 		}
 		break;
 
 		case FOM_RANORSPLIT :
 		if ( config_intshells ) {
 			fprintf(fh, "%10.3f    %10.7f %10i\n",
-				cen, r, cts[i]);
+				cen, r, fctx->cts[i]);
 		} else {
-			fprintf(fh, "%10.3f    %10.7f %10i %10.2f\n",
-				cen*1.0e-9, r, cts[i], (1.0/cen)*1e10);
+			fprintf(fh, "%10.3f    %10.7f %10i %10.2f "
+			            "%10.3f  %10.3f\n",
+			        cen*1.0e-9, r, fctx->cts[i], (1.0/cen)*1e10,
+			        shells->rmins[i]*1.0e-9,
+			        shells->rmaxs[i]*1.0e-9);
 		}
 		break;
 
+		case FOM_D1SIG :
+		case FOM_D2SIG :
+		if ( config_intshells ) {
+			fprintf(fh, "%10.3f %10.2f %10i\n",
+				cen, r*100.0, fctx->cts[i]);
+		} else {
+			fprintf(fh, "%10.3f %10.2f %10i %10.2f "
+			            "%10.3f  %10.3f\n",
+			        cen*1.0e-9, r*100.0, fctx->cts[i],
+			        (1.0/cen)*1e10,
+			        shells->rmins[i]*1.0e-9,
+			        shells->rmaxs[i]*1.0e-9);
+		}
+		break;
 
 	}
 
@@ -916,7 +1111,7 @@ int main(int argc, char *argv[])
 	RefList *list1_raw;
 	RefList *list2_raw;
 	enum fom fom = FOM_R1I;
-	char *pdb = NULL;
+	char *cellfile = NULL;
 	float rmin_fix = -1.0;
 	float rmax_fix = -1.0;
 	double rmin, rmax;
@@ -974,7 +1169,7 @@ int main(int argc, char *argv[])
 			break;
 
 			case 'p' :
-			pdb = strdup(optarg);
+			cellfile = strdup(optarg);
 			break;
 
 			case 'u' :
@@ -1077,6 +1272,8 @@ int main(int argc, char *argv[])
 			case FOM_CRDANO :
 			case FOM_RANO :
 			case FOM_RANORSPLIT :
+			case FOM_D1SIG :
+			case FOM_D2SIG :
 			break;
 		}
 	}
@@ -1096,6 +1293,8 @@ int main(int argc, char *argv[])
 			case FOM_RSPLIT :
 			case FOM_CC :
 			case FOM_CCSTAR :
+			case FOM_D1SIG :
+			case FOM_D2SIG :
 			break;
 
 			case FOM_CCANO :
@@ -1116,15 +1315,18 @@ int main(int argc, char *argv[])
 	afile = strdup(argv[optind++]);
 	bfile = strdup(argv[optind]);
 
-	if ( pdb == NULL ) {
-		ERROR("You must provide a PDB file with the unit cell.\n");
-		exit(1);
-	}
-
 	if ( shell_file == NULL ) shell_file = strdup("shells.dat");
 
-	cell = load_cell_from_pdb(pdb);
-	free(pdb);
+	cell = load_cell_from_file(cellfile);
+	if ( cellfile == NULL ) {
+		ERROR("You must provide a unit cell.\n");
+		exit(1);
+	}
+	if ( cell == NULL ) {
+		ERROR("Failed to load cell.\n");
+		return 1;
+	}
+	free(cellfile);
 
 	list1_raw = read_reflections(afile);
 	if ( list1_raw == NULL ) {
@@ -1142,11 +1344,21 @@ int main(int argc, char *argv[])
 	if ( check_list_symmetry(list1_raw, sym) ) {
 		ERROR("The first input reflection list does not appear to"
 		      " have symmetry %s\n", symmetry_name(sym));
+		if ( cell_get_lattice_type(cell) == L_MONOCLINIC ) {
+			ERROR("You may need to specify the unique axis in your "
+			      "point group.  The default is unique axis c.\n");
+			ERROR("See 'man crystfel' for more details.\n");
+		}
 		return 1;
 	}
 	if ( check_list_symmetry(list2_raw, sym) ) {
 		ERROR("The second input reflection list does not appear to"
 		      " have symmetry %s\n", symmetry_name(sym));
+		if ( cell_get_lattice_type(cell) == L_MONOCLINIC ) {
+			ERROR("You may need to specify the unique axis in your "
+			      "point group.  The default is unique axis c.\n");
+			ERROR("See 'man crystfel' for more details.\n");
+		}
 		return 1;
 	}
 
