@@ -41,7 +41,6 @@
 #include "utils.h"
 #include "cell.h"
 #include "diffraction.h"
-#include "beam-parameters.h"
 #include "symmetry.h"
 #include "pattern_sim.h"
 
@@ -424,6 +423,40 @@ static void diffraction_at_k(struct image *image, const double *intensities,
 }
 
 
+static void normalise_sampled_spectrum(struct sample *spec, int n, int nsamp)
+{
+	int i;
+	double total_w = 0.0;
+	double samp_w = 0.0;
+
+	for ( i=0; i<n; i++ ) {
+		total_w += spec[i].weight;
+	}
+	STATUS("%i energies in spectrum, %i samples requested.\n", n, nsamp);
+
+	for ( i=0; i<nsamp; i++ ) {
+		samp_w += spec[i].weight;
+	}
+
+	if ( samp_w < 0.8 * total_w ) {
+		ERROR("WARNING: Only %.1f %% of the calculated spectrum "
+		      "intensity was sampled.\n", 100.0 * samp_w / total_w);
+		ERROR("I've increased the weighting of the sampled points to "
+		      "keep the final intensity constant, but the spectrum "
+		      "might be very far from accurate.  Consider increasing "
+		      "the number of spectrum samples.\n");
+	} else {
+		STATUS("%.1f %% of calculated spectrum intensity sampled.\n",
+		       100.0 * samp_w / total_w);
+		STATUS("Normalised to keep total intensity constant.\n");
+	}
+
+	for ( i=0; i<n; i++ ) {
+		spec[i].weight /= samp_w;
+	}
+}
+
+
 static int compare_samples(const void *a, const void *b)
 {
 	struct sample *sample1 = (struct sample *)a;
@@ -549,6 +582,7 @@ struct sample *generate_tophat(struct image *image)
 	}
 
 	image->spectrum_size = image->nsamples;
+	/* No need to call normalise_sampled_spectrum() in this case */
 
 	return spectrum;
 }
@@ -557,7 +591,6 @@ struct sample *generate_tophat(struct image *image)
 struct sample *generate_SASE(struct image *image, gsl_rng *rng)
 {
 	struct sample *spectrum;
-	int i;
 	const int spec_size = 1024;
 	double eV_cen;  /* Central photon energy for this spectrum */
 	const double jitter_sigma_eV = 8.0;
@@ -566,9 +599,10 @@ struct sample *generate_SASE(struct image *image, gsl_rng *rng)
 	eV_cen = gaussian_noise(rng, ph_lambda_to_eV(image->lambda),
 	                        jitter_sigma_eV);
 
-	/* Convert FWHM to standard deviation.  Note that bandwidth is taken to
-	 * be "delta E over E" (E = photon energy), not the bandwidth in terms
-	 * of wavelength, but the difference should be very small */
+	/* Convert FWHM to standard deviation.  Note that bandwidth is taken
+	 * here to be "delta E over E" (E = photon energy), not the bandwidth in
+	 * terms of wavelength (as it is everywhere else), but the difference
+	 * should be very small */
 	double sigma = (image->bw*eV_cen) / (2.0*sqrt(2.0*log(2.0)));
 
 	/* The spectrum will be calculated to a resolution which spreads six
@@ -581,21 +615,13 @@ struct sample *generate_SASE(struct image *image, gsl_rng *rng)
 	/* Add SASE-type noise to Gaussian spectrum */
 	add_sase_noise(spectrum, spec_size, rng);
 
-	/* Normalise intensity (before taking restricted number of samples) */
-	double total_weight = 0.0;
-	for ( i=0; i<spec_size; i++ ) {
-		total_weight += spectrum[i].weight;
-	}
-	for ( i=0; i<spec_size; i++ ) {
-		spectrum[i].weight /= total_weight;
-	}
-
 	/* Sort samples in spectrum by weight.  Diffraction calculation will
 	 * take the requested number, starting from the brightest */
 	qsort(spectrum, spec_size, sizeof(struct sample), compare_samples);
 
-	image->spectrum_size = spec_size;
+	normalise_sampled_spectrum(spectrum, spec_size, image->nsamples);
 
+	image->spectrum_size = spec_size;
 	return spectrum;
 }
 
@@ -613,6 +639,10 @@ struct sample *generate_twocolour(struct image *image)
 
 	eV_cen = ph_lambda_to_eV(image->lambda);
 
+	/* Convert FWHM to standard deviation.  Note that bandwidth is taken
+	 * here to be "delta E over E" (E = photon energy), not the bandwidth in
+	 * terms of wavelength (as it is everywhere else), but the difference
+	 * should be very small */
 	double halfwidth = eV_cen*image->bw/2.0; /* eV */
 
 	eV_cen1 = eV_cen - halfwidth;
@@ -658,8 +688,9 @@ struct sample *generate_twocolour(struct image *image)
 	* take the requested number, starting from the brightest */
 	qsort(spectrum, spec_size, sizeof(struct sample), compare_samples);
 
-	image->spectrum_size = spec_size;
+	normalise_sampled_spectrum(spectrum, spec_size, image->nsamples);
 
+	image->spectrum_size = spec_size;
 	return spectrum;
 }
 
