@@ -3,15 +3,15 @@
  *
  * Quick yet non-crappy HDF viewer
  *
- * Copyright © 2012 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  *
  * Authors:
- *   2009-2012 Thomas White <taw@physics.org>
- *   2012      Richard Kirian
+ *   2009-2014 Thomas White <taw@physics.org>
  *   2014      Valerio Mariani
  *   2014      Takanori Nakane <nakane.t@gmail.com>
+ *   2012      Richard Kirian
  *
  * This file is part of CrystFEL.
  *
@@ -30,7 +30,6 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -38,9 +37,10 @@
 #ifndef DISPLAYWINDOW_H
 #define DISPLAYWINDOW_H
 
+#include "events.h"
+#include "image.h"
 #include <gtk/gtk.h>
 
-
 typedef struct {
 	GtkWidget	*window;
 	GtkWidget	*entry;
@@ -59,6 +59,12 @@ typedef struct {
 } RingRadiusDialog;
 
 
+typedef struct {
+	GtkWidget	*window;
+	GtkWidget	*entry;
+} EventDialog;
+
+
 struct numberswindow {
 	GtkWidget *window;
 	GtkWidget *labels[17*17];
@@ -83,6 +89,9 @@ typedef struct {
 	GtkWidget	*scrollarea;
 	GtkUIManager	*ui;
 	GtkActionGroup	*action_group;
+	GtkActionGroup	*calibration_action_group;
+	GtkActionGroup	*events_action_group;
+
 	int             n_pixbufs;
 	GdkPixbuf	**pixbufs;
 	gulong		motion_callback;
@@ -90,16 +99,19 @@ typedef struct {
 
 	int             not_ready_yet;
 
-	struct detector *loaded_geom;
-	struct detector *simple_geom;
+	int             simple;
 
 	struct hdfile	*hdfile;
 	struct image	*image;
 
+	char            *geom_filename;
+	char            *rg_coll_name;
+
 	/* Dialog boxes */
 	BinningDialog  *binning_dialog;
 	BoostIntDialog *boostint_dialog;
 	RingRadiusDialog *ringradius_dialog;
+	EventDialog *event_dialog;
 	struct numberswindow *numbers_window;
 
 	int		width;
@@ -113,7 +125,6 @@ typedef struct {
 	double		boostint;
 	int		noisefilter;	/* Use aggressive noise filter */
 	int             median_filter;
-	int             use_geom;
 	int             show_rings;
 	int		show_peaks;
 	double          ring_radius;
@@ -121,8 +132,9 @@ typedef struct {
 	int             n_rings;
 
 	CalibMode       calib_mode;
-	struct rigid_group *calib_mode_curr_rg;
-	struct panel    *calib_mode_curr_p;
+	struct rg_collection      *rg_coll;
+	struct rigid_group  *calib_mode_curr_rg;
+	struct panel	*calib_mode_curr_p;
 	int             calib_mode_show_focus;
 	GtkWidget       *statusbar;
 
@@ -130,16 +142,25 @@ typedef struct {
 	int		scale;
 	GdkPixbuf	*col_scale;
 
+	int                multi_event;
+	struct event_list  *ev_list;
+	int                curr_event;
+
+	gsl_rng          rng;
+
+
 } DisplayWindow;
 
 /* Open an image display window showing the given filename, or NULL */
-extern DisplayWindow *displaywindow_open(const char *filename,
+extern DisplayWindow *displaywindow_open(char *filename, char *geom_filename,
                                          const char *peaks, double boost,
                                          int binning,
-                                         int noisefilter, int calibmode, int colscale,
-                                         const char *element,
-                                         const char *geometry, const char *beam,
-                                         int show_rings,
+                                         int noisefilter, int calibmode,
+                                         int colscale, const char *element,
+                                         const char *event,
+                                         struct detector *det_geom,
+                                         struct beam_params *beam,
+                                         const char *rgcoll_name, int show_rings,
                                          double *ring_radii, int n_rings,
                                          double ring_size, int median_filter);
 
