@@ -3,11 +3,11 @@
  *
  * Small program to manipulate reflection lists
  *
- * Copyright © 2013-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2013-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2009-2014 Thomas White <taw@physics.org>
+ *   2009-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -42,7 +42,6 @@
 #include "utils.h"
 #include "reflist-utils.h"
 #include "symmetry.h"
-#include "beam-parameters.h"
 #include "cell.h"
 #include "cell-utils.h"
 
@@ -411,8 +410,6 @@ int main(int argc, char *argv[])
 	char *input_file = NULL;
 	char *template = NULL;
 	char *output = NULL;
-	char *beamfile = NULL;
-	struct beam_params *beam = NULL;
 	RefList *input;
 	double adu_per_photon = 0.0;
 	int have_adu_per_photon = 0;
@@ -421,7 +418,7 @@ int main(int argc, char *argv[])
 	char *cutoff_str = NULL;
 	double cutiso = 0.0;
 	float cutn1, cutn2, cutn3;
-	char *pdb = NULL;
+	char *cellfile = NULL;
 	char *reindex_str = NULL;
 	SymOpList *reindex = NULL;
 
@@ -448,7 +445,7 @@ int main(int argc, char *argv[])
 	};
 
 	/* Short options */
-	while ((c = getopt_long(argc, argv, "ht:o:i:w:y:e:b:p:",
+	while ((c = getopt_long(argc, argv, "ht:o:i:w:y:e:p:",
 	                        longopts, NULL)) != -1) {
 
 		switch (c) {
@@ -487,7 +484,7 @@ int main(int argc, char *argv[])
 			break;
 
 			case 'p' :
-			pdb = strdup(optarg);
+			cellfile = strdup(optarg);
 			break;
 
 			case 2 :
@@ -553,15 +550,6 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if ( beamfile != NULL ) {
-		beam = get_beam_parameters(beamfile);
-		if ( beam == NULL ) {
-			ERROR("Failed to load beam parameters from '%s'\n",
-			      beamfile);
-			return 1;
-		}
-	}
-
 	if ( holo_str != NULL ) {
 		holo = get_pointgroup(holo_str);
 		free(holo_str);
@@ -608,6 +596,10 @@ int main(int argc, char *argv[])
 	{
 		ERROR("The input reflection list does not appear to"
 		      " have symmetry %s\n", symmetry_name(mero));
+		ERROR("If your unit cell is monoclinic, you may need to specify"
+		      " the unique axis for your point group.  The default is"
+		      " unique axis c.\n");
+		ERROR("See 'man crystfel' for more details.\n");
 		return 1;
 	}
 
@@ -715,18 +707,18 @@ int main(int argc, char *argv[])
 		RefListIterator *iter;
 		UnitCell *cell;
 
-		if ( pdb == NULL ) {
-			ERROR("You must provide a PDB file when using "
+		if ( cellfile == NULL ) {
+			ERROR("You must provide a unit cell when using "
 			      "--cutoff-angstroms.\n");
 			return 1;
 		}
 
-		cell = load_cell_from_pdb(pdb);
+		cell = load_cell_from_file(cellfile);
 		if ( cell == NULL ) {
-			ERROR("Failed to load cell from '%s'\n", pdb);
+			ERROR("Failed to load cell from '%s'\n", cellfile);
 			return 1;
 		}
-		free(pdb);
+		free(cellfile);
 
 		n = reflist_new();
 
@@ -762,18 +754,18 @@ int main(int argc, char *argv[])
 		double csx, csy, csz;
 		double as, bs, cs;
 
-		if ( pdb == NULL ) {
-			ERROR("You must provide a PDB file when using "
+		if ( cellfile == NULL ) {
+			ERROR("You must provide a unit cell when using "
 			      "--cutoff-angstroms.\n");
 			return 1;
 		}
 
-		cell = load_cell_from_pdb(pdb);
+		cell = load_cell_from_file(cellfile);
 		if ( cell == NULL ) {
-			ERROR("Failed to load cell from '%s'\n", pdb);
+			ERROR("Failed to load cell from '%s'\n", cellfile);
 			return 1;
 		}
-		free(pdb);
+		free(cellfile);
 
 		cell_get_reciprocal(cell, &asx, &asy, &asz,
 				          &bsx, &bsy, &bsz,
