@@ -3,13 +3,13 @@
  *
  * Index patterns, output hkl+intensity etc.
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Richard Kirian
  * Copyright © 2012 Lorenzo Galli
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *   2011      Richard Kirian
  *   2012      Lorenzo Galli
  *   2012      Chunhong Yoon
@@ -48,12 +48,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#ifdef HAVE_CLOCK_GETTIME
-#include <time.h>
-#else
-#include <sys/time.h>
-#endif
-
 #include "version.h"
 #include "utils.h"
 #include "hdf5-file.h"
@@ -62,7 +56,6 @@
 #include "detector.h"
 #include "filters.h"
 #include "thread-pool.h"
-#include "beam-parameters.h"
 #include "geometry.h"
 #include "stream.h"
 #include "reflist-utils.h"
@@ -90,17 +83,15 @@ static void show_help(const char *s)
 "                           methods separated by commas.\n"
 "                           See 'man indexamajig' for details.\n"
 " -g. --geometry=<file>    Get detector geometry from file.\n"
-" -b, --beam=<file>        Get beam parameters from file (provides nominal\n"
-"                           wavelength value if no per-shot value is found in\n"
-"                           the HDF5 files.\n"
-" -p, --pdb=<file>         PDB file from which to get the unit cell to match.\n"
-"                           Default: 'molecule.pdb'.\n"
+" -p, --pdb=<file>         File (PDB or CrystFEL unit cell format) from which\n"
+"                           to get the unit cell.  Default: 'molecule.pdb'.\n"
 "     --basename           Remove the directory parts of the filenames.\n"
 " -x, --prefix=<p>         Prefix filenames from input file with <p>.\n"
 "     --peaks=<method>     Use 'method' for finding peaks.  Choose from:\n"
 "                           zaef  : Use Zaefferer (2000) gradient detection.\n"
 "                                    This is the default method.\n"
 "                           hdf5  : Get from a table in HDF5 file.\n"
+"                           cxi   : Get from CXI format HDF5 file.\n"
 "     --hdf5-peaks=<p>     Find peaks table in HDF5 file here.\n"
 "                           Default: /processing/hitfinder/peakinfo\n"
 "     --integration=<meth> Perform final pattern integration using <meth>.\n"
@@ -126,11 +117,12 @@ static void show_help(const char *s)
 "    --check-hdf5-snr    Check SNR for peaks from --peaks=hdf5.\n"
 "    --peak-radius=<r>   Integration radii for peak search.\n"
 "    --int-radius=<r>    Set the integration radii.  Default: 4,5,7.\n"
-"-e, --image=<element>   Use this image from the HDF5 file.\n"
-"                          Example: /data/data0.\n"
-"                          Default: The first one found.\n"
 "    --push-res=<n>      Integrate higher than apparent resolution cutoff.\n"
 "    --highres=<n>       Absolute resolution cutoff in Angstroms.\n"
+"    --fix-profile-radius Fix the reciprocal space profile radius for spot\n"
+"                         prediction (default: automatically determine.\n"
+"    --fix-bandwidth     Set the bandwidth for spot prediction.\n"
+"    --fix-divergence    Set the divergence (full angle) for spot prediction.\n"
 "\n"
 "\nFor time-resolved stuff, you might want to use:\n\n"
 "     --copy-hdf5-field <f>  Copy the value of field <f> into the stream. You\n"
@@ -149,6 +141,7 @@ static void show_help(const char *s)
 "     --no-peaks-in-stream Do not record peak search results in the stream.\n"
 "     --no-refls-in-stream Do not record integrated reflections in the stream.\n"
 "     --int-diag=<cond>    Show debugging information about reflections.\n"
+"     --no-refine          Skip the prediction refinement step.\n"
 );
 }
 
@@ -187,7 +180,7 @@ int main(int argc, char *argv[])
 	IndexingMethod *indm;
 	IndexingPrivate **ipriv;
 	char *indm_str = NULL;
-	char *pdb = NULL;
+	char *cellfile = NULL;
 	char *prefix = NULL;
 	char *speaks = NULL;
 	char *toler = NULL;
@@ -199,6 +192,8 @@ int main(int argc, char *argv[])
 	char *tempdir = NULL;
 	char *int_diag = NULL;
 	char *geom_filename = NULL;
+	struct beam_params beam;
+	int have_push_res = 0;
 
 	/* Defaults */
 	iargs.cell = NULL;
@@ -215,9 +210,8 @@ int main(int argc, char *argv[])
 	iargs.check_hdf5_snr = 0;
 	iargs.det = NULL;
 	iargs.peaks = PEAK_ZAEF;
-	iargs.beam = NULL;
-	iargs.element = NULL;
-	iargs.hdf5_peak_path = strdup("/processing/hitfinder/peakinfo");
+	iargs.beam = &beam;
+	iargs.hdf5_peak_path = NULL;
 	iargs.copyme = NULL;
 	iargs.pk_inn = -1.0;
 	iargs.pk_mid = -1.0;
@@ -240,24 +234,27 @@ int main(int argc, char *argv[])
 	iargs.int_meth = integration_method("rings-nocen", NULL);
 	iargs.push_res = 0.0;
 	iargs.highres = +INFINITY;
+	iargs.fix_profile_r = -1.0;
+	iargs.fix_bandwidth = -1.0;
+	iargs.fix_divergence = -1.0;
+	iargs.predict_refine = 1;
 
 	/* Long options */
 	const struct option longopts[] = {
 
 		/* Options with long and short versions */
 		{"help",               0, NULL,               'h'},
+		{"version",            0, NULL,               'v'},
 		{"input",              1, NULL,               'i'},
 		{"output",             1, NULL,               'o'},
 		{"indexing",           1, NULL,               'z'},
 		{"geometry",           1, NULL,               'g'},
-		{"beam",               1, NULL,               'b'},
 		{"pdb",                1, NULL,               'p'},
 		{"prefix",             1, NULL,               'x'},
 		{"threshold",          1, NULL,               't'},
-		{"image",              1, NULL,               'e'},
+		{"beam",               1, NULL,               'b'},
 
 		/* Long-only options with no arguments */
-		{"version",            0, NULL,                     99},
 		{"filter-noise",       0, &iargs.noisefilter,        1},
 		{"no-check-prefix",    0, &config_checkprefix,       0},
 		{"basename",           0, &config_basename,          1},
@@ -267,6 +264,7 @@ int main(int argc, char *argv[])
 		{"no-use-saturated",   0, &iargs.use_saturated,      0},
 		{"no-revalidate",      0, &iargs.no_revalidate,      1},
 		{"check-hdf5-snr",     0, &iargs.check_hdf5_snr,     1},
+		{"no-refine",          0, &iargs.predict_refine,     0},
 
 		/* Long-only options which don't actually do anything */
 		{"no-sat-corr",        0, &iargs.satcorr,            0},
@@ -295,12 +293,15 @@ int main(int argc, char *argv[])
 		{"res-push",           1, NULL,               19}, /* compat */
 		{"peak-radius",        1, NULL,               20},
 		{"highres",            1, NULL,               21},
+		{"fix-profile-radius", 1, NULL,               22},
+		{"fix-bandwidth",      1, NULL,               23},
+		{"fix-divergence",     1, NULL,               24},
 
 		{0, 0, NULL, 0}
 	};
 
 	/* Short options */
-	while ((c = getopt_long(argc, argv, "hi:o:z:p:x:j:g:t:b:e:",
+	while ((c = getopt_long(argc, argv, "hi:o:z:p:x:j:g:t:vb:",
 	                        longopts, NULL)) != -1)
 	{
 		switch (c) {
@@ -309,11 +310,17 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
-			case 99 :
+			case 'v' :
 			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
 			printf(CRYSTFEL_BOILERPLATE"\n");
 			return 0;
 
+			case 'b' :
+			ERROR("WARNING: This version of CrystFEL no longer "
+			      "uses beam files.  Please remove the beam file "
+			      "from your indexamajig command line.\n");
+			return 1;
+
 			case 'i' :
 			filename = strdup(optarg);
 			break;
@@ -327,7 +334,7 @@ int main(int argc, char *argv[])
 			break;
 
 			case 'p' :
-			pdb = strdup(optarg);
+			cellfile = strdup(optarg);
 			break;
 
 			case 'x' :
@@ -340,31 +347,12 @@ int main(int argc, char *argv[])
 
 			case 'g' :
 			geom_filename = optarg;
-			iargs.det = get_detector_geometry(optarg);
-			if ( iargs.det == NULL ) {
-				ERROR("Failed to read detector geometry from "
-				      "'%s'\n", optarg);
-				return 1;
-			}
 			break;
 
 			case 't' :
 			iargs.threshold = strtof(optarg, NULL);
 			break;
 
-			case 'b' :
-			iargs.beam = get_beam_parameters(optarg);
-			if ( iargs.beam == NULL ) {
-				ERROR("Failed to load beam parameters"
-				      " from '%s'\n", optarg);
-				return 1;
-			}
-			break;
-
-			case 'e' :
-			iargs.element = strdup(optarg);
-			break;
-
 			case 2 :
 			speaks = strdup(optarg);
 			break;
@@ -439,6 +427,7 @@ int main(int argc, char *argv[])
 				return 1;
 			}
 			iargs.push_res *= 1e9;  /* nm^-1 -> m^-1 */
+			have_push_res = 1;
 			break;
 
 			case 20 :
@@ -454,6 +443,28 @@ int main(int argc, char *argv[])
 			iargs.highres = 1.0 / (iargs.highres/1e10);
 			break;
 
+			case 22 :
+			if ( sscanf(optarg, "%f", &iargs.fix_profile_r) != 1 ) {
+				ERROR("Invalid value for "
+				      "--fix-profile-radius\n");
+				return 1;
+			}
+			break;
+
+			case 23 :
+			if ( sscanf(optarg, "%f", &iargs.fix_bandwidth) != 1 ) {
+				ERROR("Invalid value for --fix-bandwidth\n");
+				return 1;
+			}
+			break;
+
+			case 24 :
+			if ( sscanf(optarg, "%f", &iargs.fix_divergence) != 1 ) {
+				ERROR("Invalid value for --fix-divergence\n");
+				return 1;
+			}
+			break;
+
 			case 0 :
 			break;
 
@@ -468,13 +479,25 @@ int main(int argc, char *argv[])
 
 	}
 
+	/* Check for minimal information */
+	if ( filename == NULL ) {
+		ERROR("You need to provide the input filename (use -i)\n");
+		return 1;
+	}
+	if ( geom_filename == NULL ) {
+		ERROR("You need to specify the geometry filename (use -g)\n");
+		return 1;
+	}
+	if ( outfile == NULL ) {
+		ERROR("You need to specify the output filename (use -o)\n");
+		return 1;
+	}
+
 	if ( tempdir == NULL ) {
 		tempdir = strdup(".");
 	}
 
-	if ( filename == NULL ) {
-		filename = strdup("-");
-	}
+	/* Open input */
 	if ( strcmp(filename, "-") == 0 ) {
 		fh = stdin;
 	} else {
@@ -486,6 +509,7 @@ int main(int argc, char *argv[])
 	}
 	free(filename);
 
+	/* Parse peak detection method */
 	if ( speaks == NULL ) {
 		speaks = strdup("zaef");
 		STATUS("You didn't specify a peak detection method.\n");
@@ -495,12 +519,22 @@ int main(int argc, char *argv[])
 		iargs.peaks = PEAK_ZAEF;
 	} else if ( strcmp(speaks, "hdf5") == 0 ) {
 		iargs.peaks = PEAK_HDF5;
+	} else if ( strcmp(speaks, "cxi") == 0 ) {
+		iargs.peaks = PEAK_CXI;
 	} else {
 		ERROR("Unrecognised peak detection method '%s'\n", speaks);
 		return 1;
 	}
 	free(speaks);
+	if ( iargs.hdf5_peak_path == NULL ) {
+		if ( iargs.peaks == PEAK_HDF5 ) {
+			iargs.hdf5_peak_path = strdup("/processing/hitfinder/peakinfo");
+		} else if ( iargs.peaks == PEAK_CXI ) {
+			iargs.hdf5_peak_path = strdup("/entry_1/result_1");
+		}
+	}
 
+	/* Check prefix (if given) */
 	if ( prefix == NULL ) {
 		prefix = strdup("");
 	} else {
@@ -509,11 +543,22 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	/* Check number of processes */
 	if ( n_proc == 0 ) {
 		ERROR("Invalid number of processes.\n");
 		return 1;
 	}
 
+	/* Load detector geometry */
+	iargs.det = get_detector_geometry(geom_filename, iargs.beam);
+	if ( iargs.det == NULL ) {
+		ERROR("Failed to read detector geometry from  '%s'\n",
+		      geom_filename);
+		return 1;
+	}
+	add_geom_beam_stuff_to_copy_hdf5(iargs.copyme, iargs.det, iargs.beam);
+
+	/* Parse indexing methods */
 	if ( indm_str == NULL ) {
 
 		STATUS("You didn't specify an indexing method, so I  won't try "
@@ -532,6 +577,7 @@ int main(int argc, char *argv[])
 		free(indm_str);
 	}
 
+	/* Parse integration method */
 	if ( int_str != NULL ) {
 
 		int err;
@@ -547,7 +593,12 @@ int main(int argc, char *argv[])
 		/* Option provided for backwards compatibility */
 		iargs.int_meth |= INTEGRATION_SATURATED;
 	}
+	if ( have_push_res && !(iargs.int_meth & INTEGRATION_RESCUT) ) {
+		ERROR("WARNING: You used --push-res, but not -rescut, "
+		      "therefore --push-res will have no effect.\n");
+	}
 
+	/* Parse unit cell tolerance */
 	if ( toler != NULL ) {
 		int ttt;
 		ttt = sscanf(toler, "%f,%f,%f,%f",
@@ -560,6 +611,7 @@ int main(int argc, char *argv[])
 		free(toler);
 	}
 
+	/* Parse integration radii */
 	if ( intrad != NULL ) {
 		int r;
 		r = sscanf(intrad, "%f,%f,%f",
@@ -575,6 +627,7 @@ int main(int argc, char *argv[])
 		       " probably not appropriate for your patterns.\n");
 	}
 
+	/* Parse peak radii (used for peak detection) */
 	if ( pkrad != NULL ) {
 		int r;
 		r = sscanf(pkrad, "%f,%f,%f",
@@ -585,34 +638,20 @@ int main(int argc, char *argv[])
 		}
 		free(pkrad);
 	}
-
 	if ( iargs.pk_inn < 0.0 ) {
 		iargs.pk_inn = iargs.ir_inn;
 		iargs.pk_mid = iargs.ir_mid;
 		iargs.pk_out = iargs.ir_out;
 	}
 
-	if ( iargs.det == NULL ) {
-		ERROR("You need to provide a geometry file (please read the"
-		      " manual for more details).\n");
-		return 1;
-	}
-
-	if ( iargs.beam == NULL ) {
-		ERROR("You need to provide a beam parameters file (please read"
-		      " the manual for more details).\n");
-		return 1;
-	}
-
-	add_geom_beam_stuff_to_copy_hdf5(iargs.copyme, iargs.det, iargs.beam);
-
-	if ( pdb != NULL ) {
-		iargs.cell = load_cell_from_pdb(pdb);
+	/* Load unit cell (if given) */
+	if ( cellfile != NULL ) {
+		iargs.cell = load_cell_from_file(cellfile);
 		if ( iargs.cell == NULL ) {
-			ERROR("Couldn't read unit cell (from %s)\n", pdb);
+			ERROR("Couldn't read unit cell (from %s)\n", cellfile);
 			return 1;
 		}
-		free(pdb);
+		free(cellfile);
 		STATUS("This is what I understood your unit cell to be:\n");
 		cell_print(iargs.cell);
 	} else {
@@ -620,6 +659,7 @@ int main(int argc, char *argv[])
 		iargs.cell = NULL;
 	}
 
+	/* Parse integration diagnostic */
 	if ( int_diag != NULL ) {
 
 		int r;
@@ -661,13 +701,9 @@ int main(int argc, char *argv[])
 
 		free(int_diag);
 
-		if ( (n_proc > 1) && (iargs.int_diag != INTDIAG_NONE) ) {
-			n_proc = 1;
-			STATUS("Ignored \"-j\" because you used --int-diag.\n");
-		}
-
 	}
 
+	/* Open output stream */
 	st = open_stream_for_write_2(outfile, geom_filename, argc, argv);
 	if ( st == NULL ) {
 		ERROR("Failed to open stream '%s'\n", outfile);
@@ -678,7 +714,7 @@ int main(int argc, char *argv[])
 	/* Prepare the indexer */
 	if ( indm != NULL ) {
 		ipriv = prepare_indexing(indm, iargs.cell, iargs.det,
-		                         iargs.beam, iargs.tols);
+		                         iargs.tols);
 		if ( ipriv == NULL ) {
 			ERROR("Failed to prepare indexing.\n");
 			return 1;
@@ -695,9 +731,15 @@ int main(int argc, char *argv[])
 	create_sandbox(&iargs, n_proc, prefix, config_basename, fh,
 	               st, tempdir);
 
+	free_copy_hdf5_field_list(iargs.copyme);
+	cell_free(iargs.cell);
+	free(iargs.beam->photon_energy_from);
 	free(prefix);
 	free(tempdir);
 	free_detector_geometry(iargs.det);
+	free(iargs.hdf5_peak_path);
+	close_stream(st);
+	cleanup_indexing(indm, ipriv);
 
 	return 0;
 }
