@@ -3,11 +3,12 @@
  *
  * Generate partials for testing scaling
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2011-2014 Thomas White <taw@physics.org>
+ *   2011-2015 Thomas White <taw@physics.org>
+ *   2014      Valerio Mariani
  *
  * This file is part of CrystFEL.
  *
@@ -46,7 +47,6 @@
 #include "utils.h"
 #include "reflist-utils.h"
 #include "symmetry.h"
-#include "beam-parameters.h"
 #include "geometry.h"
 #include "stream.h"
 #include "thread-pool.h"
@@ -204,7 +204,7 @@ static void draw_and_write_image(struct image *image, RefList *reflections,
 		image->data[i] += poisson_noise(rng, background);
 	}
 
-	hdf5_write_image(image->filename, image);
+	hdf5_write_image(image->filename, image, NULL);
 	free(image->data);
 }
 
@@ -224,7 +224,6 @@ static void show_help(const char *s)
 " -o, --output=<file>      Write partials in stream format to <file>.\n"
 "     --images=<prefix>    Write images to <prefix>NNN.h5.\n"
 " -g. --geometry=<file>    Get detector geometry from file.\n"
-" -b, --beam=<file>        Get beam parameters from file\n"
 " -p, --pdb=<file>         PDB file from which to get the unit cell.\n"
 "\n"
 " -y, --symmetry=<sym>     Symmetry of the input reflection list.\n"
@@ -237,6 +236,12 @@ static void show_help(const char *s)
 "                           generated full intensities, if not using -i.\n"
 "     --noise-stddev=<val> Set the standard deviation of the noise.\n"
 "     --background=<val>   Background level in photons.  Default 3000.\n"
+"     --beam-divergence    Beam divergence in radians. Default 1 mrad.\n"
+"     --beam-bandwidth     Beam bandwidth as a fraction. Default 1%%.\n"
+"     --profile-radius     Reciprocal space reflection profile radius in m^-1.\n"
+"                           Default 0.001e9 m^-1\n"
+"     --photon-energy      Photon energy in eV.  Default 9000.\n"
+"     --really-random      Be non-deterministic.\n"
 "\n"
 );
 }
@@ -259,6 +264,7 @@ struct queue_args
 	double full_stddev;
 	double noise_stddev;
 	double background;
+	double profile_radius;
 
 	struct image *template_image;
 	double max_q;
@@ -334,7 +340,7 @@ static void run_job(void *vwargs, int cookie)
 	} while ( osf <= 0.0 );
 	crystal_set_osf(cr, osf);
 	crystal_set_mosaicity(cr, 0.0);
-	crystal_set_profile_radius(cr, wargs->image.beam->profile_radius);
+	crystal_set_profile_radius(cr, qargs->profile_radius);
 
 	/* Set up a random orientation */
 	orientation = random_quaternion(qargs->rngs[cookie]);
@@ -352,7 +358,7 @@ static void run_job(void *vwargs, int cookie)
 		snprintf(wargs->image.filename, 255, "dummy.h5");
 	}
 
-	reflections = find_intersections(&wargs->image, cr, PMODEL_SPHERE);
+	reflections = find_intersections(&wargs->image, cr, PMODEL_SCSPHERE);
 	crystal_set_reflections(cr, reflections);
 
 	for ( i=0; i<NBINS; i++ ) {
@@ -385,8 +391,12 @@ static void finalise_job(void *vqargs, void *vwargs)
 	struct worker_args *wargs = vwargs;
 	struct queue_args *qargs = vqargs;
 	int i;
+	int ret;
 
-	write_chunk(qargs->stream, &wargs->image, NULL, 0, 1);
+	ret = write_chunk(qargs->stream, &wargs->image, NULL, 0, 1, NULL);
+	if ( ret != 0) {
+		ERROR("WARNING: error writing stream file.\n");
+	}
 
 	for ( i=0; i<NBINS; i++ ) {
 		qargs->n_ref[i] += wargs->n_ref[i];
@@ -405,16 +415,36 @@ static void finalise_job(void *vqargs, void *vwargs)
 }
 
 
+static void fixup_geom(struct detector *det)
+{
+	int i;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+		det->panels[i].clen += det->panels[i].coffset;
+	}
+}
+
+
+static int geom_contains_references(struct detector *det)
+{
+	int i;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+		if ( det->panels[i].clen_from != NULL ) return 1;
+	}
+	return 0;
+}
+
+
 int main(int argc, char *argv[])
 {
 	int c;
 	char *input_file = NULL;
 	char *output_file = NULL;
-	char *beamfile = NULL;
 	char *geomfile = NULL;
 	char *cellfile = NULL;
 	struct detector *det = NULL;
-	struct beam_params *beam = NULL;
+	struct beam_params beam;
 	RefList *full = NULL;
 	char *sym_str = NULL;
 	SymOpList *sym;
@@ -426,26 +456,32 @@ int main(int argc, char *argv[])
 	struct queue_args qargs;
 	struct image image;
 	int n_threads = 1;
-	double cnoise = 0.0;
 	char *rval;
 	int i;
 	FILE *fh;
 	char *phist_file = NULL;
-	double osf_stddev = 2.0;
-	double full_stddev = 1000.0;
-	double noise_stddev = 20.0;
-	double background = 3000.0;
 	gsl_rng *rng_for_seeds;
 	int config_random = 0;
 	char *image_prefix = NULL;
 
+	/* Default simulation parameters */
+	double divergence = 0.001;
+	double bandwidth = 0.01;
+	double profile_radius = 0.001e9;
+	double photon_energy = 9000.0;
+	double osf_stddev = 2.0;
+	double full_stddev = 1000.0;
+	double noise_stddev = 20.0;
+	double background = 3000.0;
+	double cnoise = 0.0;
+
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
-		{"version",            0, NULL,                8 },
+		{"version",            0, NULL,               'v'},
+		{"beam",               1, NULL,               'b'},
 		{"output",             1, NULL,               'o'},
 		{"input",              1, NULL,               'i'},
-		{"beam",               1, NULL,               'b'},
 		{"pdb",                1, NULL,               'p'},
 		{"geometry",           1, NULL,               'g'},
 		{"symmetry",           1, NULL,               'y'},
@@ -458,6 +494,10 @@ int main(int argc, char *argv[])
 		{"noise-stddev",       1, NULL,                5},
 		{"images",             1, NULL,                6},
 		{"background",         1, NULL,                7},
+		{"beam-divergence",    1, NULL,                8},
+		{"beam-bandwidth",     1, NULL,                9},
+		{"profile-radius",     1, NULL,               10},
+		{"photon-energy",      1, NULL,               11},
 
 		{"really-random",      0, &config_random,      1},
 
@@ -465,7 +505,7 @@ int main(int argc, char *argv[])
 	};
 
 	/* Short options */
-	while ((c = getopt_long(argc, argv, "hi:o:b:p:g:y:n:r:j:c:",
+	while ((c = getopt_long(argc, argv, "hi:o:p:g:y:n:r:j:c:vb:",
 	                        longopts, NULL)) != -1)
 	{
 		switch (c) {
@@ -474,11 +514,17 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
-			case 8 :
+			case 'v' :
 			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
 			printf(CRYSTFEL_BOILERPLATE"\n");
 			return 0;
 
+			case 'b' :
+			ERROR("WARNING: This version of CrystFEL no longer "
+			      "uses beam files.  Please remove the beam file "
+			      "from your partial_sim command line.\n");
+			return 1;
+
 			case 'i' :
 			input_file = strdup(optarg);
 			break;
@@ -487,10 +533,6 @@ int main(int argc, char *argv[])
 			output_file = strdup(optarg);
 			break;
 
-			case 'b' :
-			beamfile = strdup(optarg);
-			break;
-
 			case 'p' :
 			cellfile = strdup(optarg);
 			break;
@@ -577,8 +619,55 @@ int main(int argc, char *argv[])
 				return 1;
 			}
 			if ( background < 0.0 ) {
-				ERROR("Invalid background level.");
-				ERROR(" (must be positive).\n");
+				ERROR("Background level must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 8 :
+			divergence = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid beam divergence.\n");
+				return 1;
+			}
+			if ( divergence < 0.0 ) {
+				ERROR("Beam divergence must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 9 :
+			bandwidth = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid beam bandwidth.\n");
+				return 1;
+			}
+			if ( bandwidth < 0.0 ) {
+				ERROR("Beam bandwidth must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 10 :
+			profile_radius = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid profile radius.\n");
+				return 1;
+			}
+			if ( divergence < 0.0 ) {
+				ERROR("Profile radius must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 11 :
+			photon_energy = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid photon energy.\n");
+				return 1;
+			}
+			if ( photon_energy < 0.0 ) {
+				ERROR("Photon energy must be positive.\n");
 				return 1;
 			}
 			break;
@@ -606,24 +695,12 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	/* Load beam */
-	if ( beamfile == NULL ) {
-		ERROR("You need to provide a beam parameters file.\n");
-		return 1;
-	}
-	beam = get_beam_parameters(beamfile);
-	if ( beam == NULL ) {
-		ERROR("Failed to load beam parameters from '%s'\n", beamfile);
-		return 1;
-	}
-	free(beamfile);
-
 	/* Load cell */
 	if ( cellfile == NULL ) {
 		ERROR("You need to give a PDB file with the unit cell.\n");
 		return 1;
 	}
-	cell = load_cell_from_pdb(cellfile);
+	cell = load_cell_from_file(cellfile);
 	if ( cell == NULL ) {
 		ERROR("Failed to get cell from '%s'\n", cellfile);
 		return 1;
@@ -641,11 +718,24 @@ int main(int argc, char *argv[])
 		ERROR("You need to give a geometry file.\n");
 		return 1;
 	}
-	det = get_detector_geometry(geomfile);
+	det = get_detector_geometry(geomfile, &beam);
 	if ( det == NULL ) {
 		ERROR("Failed to read geometry from '%s'\n", geomfile);
 		return 1;
 	}
+	if ( (beam.photon_energy > 0.0) && (beam.photon_energy_from == NULL) ) {
+		ERROR("WARNING: An explicit photon energy was found in the "
+		      "geometry file.  It will be ignored!\n");
+		ERROR("The value given on the command line "
+		      "(with --photon-energy) will be used instead.\n");
+	}
+	if ( geom_contains_references(det) ) {
+		ERROR("Geometry file contains a reference to an HDF5 location"
+		      " for the camera length.  Change it to a numerical value "
+		      " and try again.\n");
+		return 1;
+	}
+	fixup_geom(det);
 
 	if ( sym_str == NULL ) sym_str = strdup("1");
 	sym = get_pointgroup(sym_str);
@@ -664,10 +754,15 @@ int main(int argc, char *argv[])
 			      input_file);
 			return 1;
 		}
-		free(input_file);
 		if ( check_list_symmetry(full, sym) ) {
 			ERROR("The input reflection list does not appear to"
 			      " have symmetry %s\n", symmetry_name(sym));
+			if ( cell_get_lattice_type(cell) == L_MONOCLINIC ) {
+				ERROR("You may need to specify the unique axis "
+				      "in your point group.  The default is "
+				      "unique axis c.\n");
+				ERROR("See 'man crystfel' for more details.\n");
+			}
 			return 1;
 		}
 
@@ -696,13 +791,13 @@ int main(int argc, char *argv[])
 	free(output_file);
 
 	image.det = det;
+	image.beam = &beam;
 	image.width = det->max_fs + 1;
 	image.height = det->max_ss + 1;
 
-	image.lambda = ph_en_to_lambda(eV_to_J(beam->photon_energy));
-	image.div = beam->divergence;
-	image.bw = beam->bandwidth;
-	image.beam = beam;
+	image.lambda = ph_en_to_lambda(eV_to_J(photon_energy));
+	image.div = divergence;
+	image.bw = bandwidth;
 	image.filename = "dummy.h5";
 	image.copyme = NULL;
 	image.crystals = NULL;
@@ -711,6 +806,34 @@ int main(int argc, char *argv[])
 	image.num_peaks = 0;
 	image.num_saturated_peaks = 0;
 	image.spectrum_size = 0;
+	image.spectrum = NULL;
+	image.serial = 0;
+	image.event = NULL;
+
+	STATUS("Simulation parameters:\n");
+	STATUS("                  Photon energy: %.2f eV (wavelength %.5f A)\n",
+	       photon_energy, image.lambda*1e10);
+	STATUS("                Beam divergence: %.5f mrad\n", image.div*1e3);
+	STATUS("                 Beam bandwidth: %.5f %%\n", image.bw*100.0);
+	STATUS("Reciprocal space profile radius: %e m^-1\n", profile_radius);
+	if ( image_prefix != NULL ) {
+		STATUS("                     Background: %.2f detector units\n",
+		       background);
+	} else {
+		STATUS("                     Background: none (no image "
+		       "output\n");
+	}
+	STATUS("               Partiality model: scsphere (hardcoded)\n");
+	STATUS("       Noise standard deviation: %.2f detector units\n",
+	       noise_stddev);
+	if ( random_intensities ) {
+		STATUS("               Full intensities: randomly generated: "
+		       "abs(Gaussian(sigma=%.2f))\n", full_stddev);
+	} else {
+		STATUS("               Full intensities: from %s\n",
+		       input_file);
+	}
+	STATUS("   Max error in cell components: %.2f %%\n", cnoise);
 
 	if ( random_intensities ) {
 		full = reflist_new();
@@ -733,6 +856,7 @@ int main(int argc, char *argv[])
 	qargs.background = background;
 	qargs.max_q = largest_q(&image);
 	qargs.image_prefix = image_prefix;
+	qargs.profile_radius = profile_radius;
 
 	qargs.rngs = malloc(n_threads * sizeof(gsl_rng *));
 	if ( qargs.rngs == NULL ) {
@@ -840,11 +964,11 @@ int main(int argc, char *argv[])
 	close_stream(stream);
 	cell_free(cell);
 	free_detector_geometry(det);
-	free(beam);
 	free_symoplist(sym);
 	reflist_free(full);
 	free(save_file);
 	free(geomfile);
+	free(input_file);
 
 	return 0;
 }
