@@ -3,11 +3,11 @@
  *
  * Scaling and post refinement for coherent nanocrystallography
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2013 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -50,11 +50,13 @@
 #include <thread-pool.h>
 #include <reflist.h>
 #include <reflist-utils.h>
+#include <cell.h>
+#include <cell-utils.h>
 
 #include "version.h"
 #include "post-refinement.h"
-#include "hrs-scaling.h"
-#include "scaling-report.h"
+#include "merge.h"
+#include "rejection.h"
 
 
 static void show_help(const char *s)
@@ -69,109 +71,18 @@ static void show_help(const char *s)
 "  -i, --input=<filename>     Specify the name of the input 'stream'.\n"
 "  -o, --output=<filename>    Output filename.  Default: partialator.hkl.\n"
 "  -y, --symmetry=<sym>       Merge according to symmetry <sym>.\n"
+"      --start-after=<n>      Skip <n> crystals at the start of the stream.\n"
+"      --stop-after=<n>       Stop after merging <n> crystals.\n"
 "  -n, --iterations=<n>       Run <n> cycles of scaling and post-refinement.\n"
-"      --no-scale             Fix all the scaling factors at unity.\n"
+"      --no-scale             Disable scale factor (G, B) refinement.\n"
+"      --no-pr                Disable orientation/physics refinement.\n"
 "  -m, --model=<model>        Specify partiality model.\n"
 "      --min-measurements=<n> Minimum number of measurements to require.\n"
 "      --no-polarisation      Disable polarisation correction.\n"
-"  -j <n>                     Run <n> analyses in parallel.\n");
-}
-
-
-struct refine_args
-{
-	RefList *full;
-	Crystal *crystal;
-	PartialityModel pmodel;
-	struct prdata prdata;
-};
-
-
-struct queue_args
-{
-	int n_started;
-	int n_done;
-	Crystal **crystals;
-	int n_crystals;
-	struct srdata *srdata;
-	struct refine_args task_defaults;
-};
-
-
-static void refine_image(void *task, int id)
-{
-	struct refine_args *pargs = task;
-	Crystal *cr = pargs->crystal;
-
-	pargs->prdata = pr_refine(cr, pargs->full, pargs->pmodel);
-}
-
-
-static void *get_image(void *vqargs)
-{
-	struct refine_args *task;
-	struct queue_args *qargs = vqargs;
-
-	task = malloc(sizeof(struct refine_args));
-	memcpy(task, &qargs->task_defaults, sizeof(struct refine_args));
-
-	task->crystal = qargs->crystals[qargs->n_started];
-
-	qargs->n_started++;
-
-	return task;
-}
-
-
-static void done_image(void *vqargs, void *task)
-{
-	struct queue_args *qargs = vqargs;
-	struct refine_args *pargs = task;
-
-	qargs->n_done++;
-	if ( pargs->prdata.refined ) {
-		qargs->srdata->n_refined += pargs->prdata.refined;
-		qargs->srdata->n_filtered += pargs->prdata.n_filtered;
-	}
-
-	progress_bar(qargs->n_done, qargs->n_crystals, "Refining");
-	free(task);
-}
-
-
-static void refine_all(Crystal **crystals, int n_crystals,
-                       RefList *full, int nthreads, PartialityModel pmodel,
-                       struct srdata *srdata)
-{
-	struct refine_args task_defaults;
-	struct queue_args qargs;
-
-	/* If the partiality model is "p=1", this refinement is really, really
-	 * easy... */
-	if ( pmodel == PMODEL_UNITY ) return;
-
-	task_defaults.full = full;
-	task_defaults.crystal = NULL;
-	task_defaults.pmodel = pmodel;
-	task_defaults.prdata.refined = 0;
-	task_defaults.prdata.n_filtered = 0;
-
-	qargs.task_defaults = task_defaults;
-	qargs.n_started = 0;
-	qargs.n_done = 0;
-	qargs.n_crystals = n_crystals;
-	qargs.crystals = crystals;
-	qargs.srdata = srdata;
-
-	/* Don't have threads which are doing nothing */
-	if ( n_crystals < nthreads ) nthreads = n_crystals;
-
-	run_threads(nthreads, refine_image, get_image, done_image,
-	            &qargs, n_crystals, 0, 0, 0);
-
-	STATUS("%5.2f eigenvalues filtered on final iteration per successfully "
-	       "refined crystal\n",
-	       (double)srdata->n_filtered/srdata->n_refined);
+"      --max-adu=<n>          Saturation value of detector.\n"
+"      --push-res=<n>         Merge higher than apparent resolution cutoff.\n"
+"  -j <n>                     Run <n> analyses in parallel.\n"
+"      --no-free              Disable cross-validation (testing only).\n");
 }
 
 
@@ -189,13 +100,173 @@ static void display_progress(int n_images, int n_crystals)
 }
 
 
-static const char *str_flags(Crystal *cr)
+static RefList *apply_max_adu(RefList *list, double max_adu)
 {
-	if ( crystal_get_user_flag(cr) ) {
-		return "N";
+	RefList *nlist;
+	Reflection *refl;
+	RefListIterator *iter;
+
+	nlist = reflist_new();
+	if ( nlist == NULL ) return NULL;
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+		if ( get_peak(refl) < max_adu ) {
+			signed int h, k, l;
+			get_indices(refl, &h, &k, &l);
+			Reflection *nrefl = add_refl(nlist, h, k, l);
+			copy_data(nrefl, refl);
+		}
+	}
+	reflist_free(list);
+	return nlist;
+}
+
+
+static void skip_to_end(FILE *fh)
+{
+	int c;
+	do {
+		c = fgetc(fh);
+	} while ( (c != '\n') && (c != EOF) );
+}
+
+
+static int set_initial_params(Crystal *cr, FILE *fh)
+{
+	if ( fh != NULL ) {
+
+		int err;
+		int n;
+		float osf, B;
+
+		err = fscanf(fh, "%i %f %f", &n, &osf, &B);
+		if ( err != 3 ) {
+			ERROR("Failed to read parameters.\n");
+			return 1;
+		}
+
+		crystal_set_osf(cr, osf);
+		crystal_set_Bfac(cr, B*1e-20);
+
+		skip_to_end(fh);
+
+	} else {
+
+		crystal_set_osf(cr, 0.0);
+		crystal_set_Bfac(cr, 0.0);
+
 	}
 
-	return "-";
+	return 0;
+}
+
+
+/* Flag a random 5% of reflections */
+static void select_free_reflections(RefList *list, gsl_rng *rng)
+{
+	Reflection *refl;
+	RefListIterator *iter;
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+		set_flag(refl, random_flat(rng, 1.0) > 0.95);
+	}
+}
+
+
+static void write_to_pgraph(FILE *fh, RefList *list, RefList *full, Crystal *cr,
+                            int fr)
+{
+	Reflection *refl;
+	RefListIterator *iter;
+	double G = crystal_get_osf(cr);
+	double B = crystal_get_Bfac(cr);
+	UnitCell *cell = crystal_get_cell(cr);
+
+	for ( refl = first_refl(list, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+		signed int h, k, l;
+		double pobs, pcalc;
+		double res, corr, Ipart;
+		Reflection *match;
+
+		if ( !get_flag(refl) ) continue;  /* Not free-flagged */
+
+		get_indices(refl, &h, &k, &l);
+		res = resolution(cell, h, k, l);
+		if ( 2.0*res > crystal_get_resolution_limit(cr) ) continue;
+
+		match = find_refl(full, h, k, l);
+		if ( match == NULL ) continue;
+
+		/* Calculated partiality */
+		pcalc = get_partiality(refl);
+
+		/* Observed partiality */
+		corr = exp(-G) * exp(B*res*res) * get_lorentz(refl);
+		Ipart = get_intensity(refl) * corr;
+		pobs = Ipart / get_intensity(match);
+
+		fprintf(fh, "%5i %4i %4i %4i %8.4f %8.3f %8.3f\n",
+		        fr, h, k, l, 2*res/1e9, pcalc, pobs);
+
+	}
+}
+
+
+static void write_pgraph(RefList *full, Crystal **crystals, int n_crystals,
+                         int iter)
+{
+	FILE *fh;
+	char tmp[256];
+	int i;
+
+	snprintf(tmp, 256, "pgraph-iter%i.dat", iter);
+
+	fh = fopen(tmp, "w");
+	if ( fh == NULL ) {
+		ERROR("Failed to open '%s'\n", tmp);
+		return;
+	}
+
+	fprintf(fh, "   fr    h    k    l  1/d(nm)     pcalc   pobs\n");
+
+	for ( i=0; i<n_crystals; i++ ) {
+		if ( crystal_get_user_flag(crystals[i]) != 0 ) continue;
+		write_to_pgraph(fh, crystal_get_reflections(crystals[i]), full,
+		                crystals[i], i);
+	}
+
+	fclose(fh);
+
+}
+
+
+static void normalise_scales(Crystal **crystals, int n_crystals)
+{
+	int i;
+	int n = 0;
+	double sum_B = 0.0;
+	double mean_B;
+
+	for ( i=0; i<n_crystals; i++ ) {
+		if ( crystal_get_user_flag(crystals[i]) != 0 ) continue;
+		sum_B += crystal_get_Bfac(crystals[i]);
+		n++;
+	}
+	mean_B = sum_B / n;
+	STATUS("Mean B = %e A^2\n", mean_B*1e20);
+	for ( i=0; i<n_crystals; i++ ) {
+		double B = crystal_get_Bfac(crystals[i]);
+		crystal_set_Bfac(crystals[i], B-mean_B);
+	}
 }
 
 
@@ -212,38 +283,54 @@ int main(int argc, char *argv[])
 	int n_iter = 10;
 	RefList *full;
 	int n_images = 0;
+	int n_images_seen = 0;
 	int n_crystals = 0;
+	int n_crystals_seen = 0;
 	char cmdline[1024];
-	SRContext *sr;
-	int noscale = 0;
+	int no_scale = 0;
+	int no_pr = 0;
 	Stream *st;
 	Crystal **crystals;
 	char *pmodel_str = NULL;
-	PartialityModel pmodel = PMODEL_SPHERE;
+	PartialityModel pmodel = PMODEL_SCSPHERE;
 	int min_measurements = 2;
 	char *rval;
-	struct srdata srdata;
 	int polarisation = 1;
+	int start_after = 0;
+	int stop_after = 0;
+	double max_adu = +INFINITY;
+	char *sparams_fn = NULL;
+	FILE *sparams_fh;
+	double push_res = 0.0;
+	gsl_rng *rng;
+	int no_free = 0;
 
 	/* Long options */
 	const struct option longopts[] = {
 
 		{"help",               0, NULL,               'h'},
-		{"version",            0, NULL,                3 },
+		{"version",            0, NULL,               'v'},
 		{"input",              1, NULL,               'i'},
 		{"output",             1, NULL,               'o'},
+		{"start-after",        1, NULL,               's'},
+		{"stop-after",         1, NULL,               'f'},
 		{"symmetry",           1, NULL,               'y'},
 		{"iterations",         1, NULL,               'n'},
-		{"reference",          1, NULL,               'r'},
 		{"model",              1, NULL,               'm'},
 
 		{"min-measurements",   1, NULL,                2},
+		{"max-adu",            1, NULL,                3},
+		{"start-params",       1, NULL,                4},
+		{"push-res",           1, NULL,                5},
+		{"res-push",           1, NULL,                5}, /* compat */
 
-		{"no-scale",           0, &noscale,            1},
+		{"no-scale",           0, &no_scale,           1},
+		{"no-pr",              0, &no_pr,              1},
 		{"no-polarisation",    0, &polarisation,       0},
 		{"no-polarization",    0, &polarisation,       0},
 		{"polarisation",       0, &polarisation,       1}, /* compat */
 		{"polarization",       0, &polarisation,       1}, /* compat */
+		{"no-free",            0, &no_free,            1},
 
 		{0, 0, NULL, 0}
 	};
@@ -265,7 +352,7 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
-			case 3 :
+			case 'v' :
 			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
 			printf(CRYSTFEL_BOILERPLATE"\n");
 			return 0;
@@ -278,6 +365,26 @@ int main(int argc, char *argv[])
 			nthreads = atoi(optarg);
 			break;
 
+			case 's' :
+			errno = 0;
+			start_after = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid value for --start-after (%s)\n",
+				      optarg);
+				return 1;
+			}
+			break;
+
+			case 'f' :
+			errno = 0;
+			stop_after = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid value for --stop-after (%s)\n",
+				      optarg);
+				return 1;
+			}
+			break;
+
 			case 'y' :
 			sym_str = strdup(optarg);
 			break;
@@ -303,6 +410,29 @@ int main(int argc, char *argv[])
 			}
 			break;
 
+			case 3 :
+			errno = 0;
+			max_adu = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid value for --max-adu.\n");
+				return 1;
+			}
+			break;
+
+			case 4 :
+			sparams_fn = strdup(optarg);
+			break;
+
+			case 5 :
+			errno = 0;
+			push_res = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid value for --push-res.\n");
+				return 1;
+			}
+			push_res = push_res*1e9;
+			break;
+
 			case 0 :
 			break;
 
@@ -342,29 +472,49 @@ int main(int argc, char *argv[])
 	free(sym_str);
 
 	if ( pmodel_str != NULL ) {
-		if ( strcmp(pmodel_str, "sphere") == 0 ) {
-			pmodel = PMODEL_SPHERE;
-		} else if ( strcmp(pmodel_str, "unity") == 0 ) {
+		if ( strcmp(pmodel_str, "unity") == 0 ) {
 			pmodel = PMODEL_UNITY;
-		} else if ( strcmp(pmodel_str, "gaussian") == 0 ) {
-			pmodel = PMODEL_GAUSSIAN;
-		} else if ( strcmp(pmodel_str, "thin") == 0 ) {
-			pmodel = PMODEL_THIN;
+		} else if ( strcmp(pmodel_str, "scgaussian") == 0 ) {
+			pmodel = PMODEL_SCGAUSSIAN;
 		} else if ( strcmp(pmodel_str, "scsphere") == 0 ) {
 			pmodel = PMODEL_SCSPHERE;
+		} else if ( strcmp(pmodel_str, "random") == 0 ) {
+			pmodel = PMODEL_RANDOM;
 		} else {
 			ERROR("Unknown partiality model '%s'.\n", pmodel_str);
 			return 1;
 		}
 	}
 
+	if ( (pmodel == PMODEL_UNITY) && !no_pr ) {
+		no_pr = 1;
+		STATUS("Not going to refine the unity model.\n");
+	}
+
 	gsl_set_error_handler_off();
+	rng = gsl_rng_alloc(gsl_rng_mt19937);
 
 	/* Fill in what we know about the images so far */
 	n_images = 0;
+	n_images_seen = 0;
 	n_crystals = 0;
+	n_crystals_seen = 0;
 	images = NULL;
 	crystals = NULL;
+	if ( sparams_fn != NULL ) {
+		char line[1024];
+		sparams_fh = fopen(sparams_fn, "r");
+		if ( sparams_fh == NULL ) {
+			ERROR("Failed to open '%s'\n", sparams_fn);
+			return 1;
+		}
+		fgets(line, 1024, sparams_fh);
+		STATUS("Reading initial scaling factors (G,B) from '%s'\n",
+		       sparams_fn);
+		free(sparams_fn);
+	} else {
+		sparams_fh = NULL;
+	}
 
 	do {
 
@@ -394,7 +544,7 @@ int main(int argc, char *argv[])
 			return 1;
 		}
 
-		n_images++;
+		n_images_seen++;
 
 		for ( i=0; i<cur->n_crystals; i++ ) {
 
@@ -402,6 +552,9 @@ int main(int argc, char *argv[])
 			Crystal **crystals_new;
 			RefList *cr_refl;
 
+			n_crystals_seen++;
+			if ( n_crystals_seen <= start_after ) continue;
+
 			crystals_new = realloc(crystals,
 			                      (n_crystals+1)*sizeof(Crystal *));
 			if ( crystals_new == NULL ) {
@@ -419,27 +572,46 @@ int main(int argc, char *argv[])
 			/* This is the raw list of reflections */
 			cr_refl = crystal_get_reflections(cr);
 
+			cr_refl = apply_max_adu(cr_refl, max_adu);
+
 			if ( polarisation ) {
 				polarisation_correction(cr_refl,
 						        crystal_get_cell(cr),
 						        cur);
 			}
 
+			if ( !no_free ) select_free_reflections(cr_refl, rng);
+
 			as = asymmetric_indices(cr_refl, sym);
 			crystal_set_reflections(cr, as);
+			crystal_set_user_flag(cr, PRFLAG_OK);
 			reflist_free(cr_refl);
 
+			if ( set_initial_params(cr, sparams_fh) ) {
+				ERROR("Failed to set initial parameters\n");
+				return 1;
+			}
+
 			n_crystals++;
 
+			if ( n_crystals == stop_after ) break;
+
+		}
+
+		if ( n_crystals > 0 ) {
+			n_images++;
 		}
 
 		if ( n_images % 100 == 0 ) {
 			display_progress(n_images, n_crystals);
 		}
 
+		if ( (stop_after>0) && (n_crystals == stop_after) ) break;
+
 	} while ( 1 );
 	display_progress(n_images, n_crystals);
 	fprintf(stderr, "\n");
+	if ( sparams_fh != NULL ) fclose(sparams_fh);
 
 	close_stream(st);
 
@@ -449,92 +621,87 @@ int main(int argc, char *argv[])
 		for ( j=0; j<images[i].n_crystals; j++ ) {
 
 			Crystal *cryst;
-			int n_gained = 0;
-			int n_lost = 0;
-			double mean_p_change = 0.0;
 
 			cryst = images[i].crystals[j];
 			crystal_set_image(cryst, &images[i]);
 
 			/* Now it's safe to do the following */
-			update_partialities_2(cryst, pmodel, &n_gained, &n_lost,
-			                      &mean_p_change);
-			assert(n_gained == 0);  /* That'd just be silly */
+			update_partialities(cryst, pmodel);
 
 		}
 	}
 
+	/* Make a first pass at cutting out crap */
+	STATUS("Checking patterns.\n");
+	//early_rejection(crystals, n_crystals);
+
 	/* Make initial estimates */
-	STATUS("Performing initial scaling.\n");
-	if ( noscale ) STATUS("Scale factors fixed at 1.\n");
-	full = scale_intensities(crystals, n_crystals,
-	                         nthreads, noscale, pmodel, min_measurements);
+	full = merge_intensities(crystals, n_crystals, nthreads, pmodel,
+	                         min_measurements, push_res);
 
-	srdata.crystals = crystals;
-	srdata.n = n_crystals;
-	srdata.full = full;
-	srdata.n_filtered = 0;
-	srdata.n_refined = 0;
+	check_rejection(crystals, n_crystals, full);
 
-	sr = sr_titlepage(crystals, n_crystals, "scaling-report.pdf",
-	                  infile, cmdline);
-	sr_iteration(sr, 0, &srdata);
+	write_pgraph(full, crystals, n_crystals, 0);
 
 	/* Iterate */
 	for ( i=0; i<n_iter; i++ ) {
-		int n_noscale = 0;
-		int n_noref = 0;
-		int n_solve = 0;
-		int n_lost = 0;
-		int n_dud = 0;
-		int j;
 
-		STATUS("Post refinement cycle %i of %i\n", i+1, n_iter);
+		double init_dev, init_free_dev;
+		double final_dev, final_free_dev;
 
-		srdata.n_filtered = 0;
+		STATUS("Refinement cycle %i of %i\n", i+1, n_iter);
 
-		/* Refine the geometry of all patterns to get the best fit */
+		/* Refine all crystals to get the best fit */
 		refine_all(crystals, n_crystals, full, nthreads, pmodel,
-		           &srdata);
+		           no_scale, no_pr, &init_dev, &init_free_dev,
+		           &final_dev, &final_free_dev);
 
-		for ( j=0; j<n_crystals; j++ ) {
-			int flag;
-			flag = crystal_get_user_flag(crystals[j]);
-			if ( flag != 0 ) n_dud++;
-			if ( flag == 1 ) {
-				n_noscale++;
-			} else if ( flag == 2 ) {
-				n_noref++;
-			} else if ( flag == 3 ) {
-				n_solve++;
-			} else if ( flag == 4 ) {
-				n_lost++;
-			}
-		}
+		STATUS("Overall residual: initial = %e, final = %e\n",
+		       init_dev, final_dev);
+		STATUS("Overall free residual: initial = %e, final = %e\n",
+		       init_free_dev, final_free_dev);
+
+		check_rejection(crystals, n_crystals, full);
+		normalise_scales(crystals, n_crystals);
 
-		if ( n_dud ) {
-			STATUS("%i crystals could not be refined this cycle.\n",
-			       n_dud);
-			STATUS("%i scaling failed.\n", n_noscale);
-			STATUS("%i not enough reflections.\n", n_noref);
-			STATUS("%i solve failed.\n", n_solve);
-			STATUS("%i lost too many reflections.\n", n_lost);
-		}
 		/* Re-estimate all the full intensities */
 		reflist_free(full);
-		full = scale_intensities(crystals, n_crystals, nthreads,
-		                         noscale, pmodel, min_measurements);
+		full = merge_intensities(crystals, n_crystals, nthreads,
+		                         pmodel, min_measurements, push_res);
 
-		srdata.full = full;
-
-		sr_iteration(sr, i+1, &srdata);
+		write_pgraph(full, crystals, n_crystals, i+1);
 
 	}
 
-	sr_finish(sr);
-
 	/* Output results */
-	write_reflist(outfile, full);
+	write_reflist_2(outfile, full, sym);
+
+	/* Output split results */
+	char tmp[1024];
+	RefList *split;
+	Crystal *crystals1[n_crystals];
+	Crystal *crystals2[n_crystals];
+	int n_crystals1 = 0;
+	int n_crystals2 = 0;
+	for ( i=0; i<n_crystals; i++ ) {
+		if ( i % 2 ) {
+			crystals1[n_crystals1] = crystals[i];
+			n_crystals1++;
+		} else {
+			crystals2[n_crystals2] = crystals[i];
+			n_crystals2++;
+		}
+	}
+	snprintf(tmp, 1024, "%s1", outfile);
+	split = merge_intensities(crystals1, n_crystals1, nthreads,
+		                  pmodel, min_measurements, push_res);
+	write_reflist_2(tmp, split, sym);
+	reflist_free(split);
+	snprintf(tmp, 1024, "%s2", outfile);
+	split = merge_intensities(crystals2, n_crystals2, nthreads,
+		                  pmodel, min_measurements, push_res);
+	write_reflist_2(tmp, split, sym);
+	reflist_free(split);
 
 	/* Dump parameters */
 	FILE *fh;
@@ -542,16 +709,19 @@ int main(int argc, char *argv[])
 	if ( fh == NULL ) {
 		ERROR("Couldn't open partialator.params!\n");
 	} else {
+		fprintf(fh, "  cr        OSF       relB         div flag\n");
 		for ( i=0; i<n_crystals; i++ ) {
-			fprintf(fh, "%4i %5.2f %8.5e %s\n", i,
+			fprintf(fh, "%4i %10.5f %10.2f %8.5e %s\n", i,
 			        crystal_get_osf(crystals[i]),
+				crystal_get_Bfac(crystals[i])*1e20,
 			        crystal_get_image(crystals[i])->div,
-			        str_flags(crystals[i]));
+			        str_prflag(crystal_get_user_flag(crystals[i])));
 		}
 		fclose(fh);
 	}
 
 	/* Clean up */
+	gsl_rng_free(rng);
 	for ( i=0; i<n_crystals; i++ ) {
 		reflist_free(crystal_get_reflections(crystals[i]));
 		crystal_free(crystals[i]);
