@@ -3,12 +3,13 @@
  *
  * Simulate diffraction patterns from small crystals
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2009-2014 Thomas White <taw@physics.org>
+ *   2009-2015 Thomas White <taw@physics.org>
  *   2013-2014 Chun Hong Yoon <chun.hong.yoon@desy.de>
+ *   2014      Valerio Mariani
  *   2013      Alexandra Tolstikova
  *
  * This file is part of CrystFEL.
@@ -50,7 +51,6 @@
 #include "hdf5-file.h"
 #include "detector.h"
 #include "peaks.h"
-#include "beam-parameters.h"
 #include "symmetry.h"
 #include "reflist.h"
 #include "reflist-utils.h"
@@ -68,14 +68,13 @@ static void show_help(const char *s)
 " -h, --help                Display this help message.\n"
 "     --version             Print CrystFEL version number and exit.\n"
 "\n"
-" -p, --pdb=<file>          PDB file from which to get the unit cell.\n"
+" -p, --pdb=<file>          File from which to get the unit cell.\n"
 "                            (The actual Bragg intensities come from the\n"
 "                            intensities file)\n"
 "     --gpu                 Use the GPU to speed up the calculation.\n"
 "     --gpu-dev=<n>         Use GPU device <n>.  Omit this option to see the\n"
 "                            available devices.\n"
 " -g, --geometry=<file>     Get detector geometry from file.\n"
-" -b, --beam=<file>         Get beam parameters from file.\n"
 " -n, --number=<N>          Generate N images.  Default 1.\n"
 "     --no-images           Do not output any HDF5 files.\n"
 " -o, --output=<filename>   Output HDF5 filename.  Default: sim.h5.\n"
@@ -95,6 +94,10 @@ static void show_help(const char *s)
 "     --background=<N>      Add N photons of Poisson background (default 0).\n"
 "     --template=<file>     Take orientations from stream <file>.\n"
 "     --no-fringes          Exclude the side maxima of Bragg peaks.\n"
+"     --beam-bandwidth      Beam bandwidth as a fraction. Default 1%%.\n"
+"     --photon-energy       Photon energy in eV.  Default 9000.\n"
+"     --nphotons            Number of photons per X-ray pulse.  Default 1e12.\n"
+"     --beam-radius         Radius of beam in metres (default 1e-6).\n"
 );
 }
 
@@ -208,24 +211,46 @@ static struct quaternion read_quaternion()
 }
 
 
-static int random_ncells(double len, double min_nm, double max_nm)
+static int random_ncells(double len, double min_m, double max_m)
 {
 	int min_cells, max_cells, wid;
 
-	min_cells = min_nm*1e-9 / len;
-	max_cells = max_nm*1e-9 / len;
+	min_cells = min_m / len;
+	max_cells = max_m / len;
 	wid = max_cells - min_cells;
 
 	return wid*random()/RAND_MAX + min_cells;
 }
 
 
+static void fixup_geom(struct detector *det)
+{
+	int i;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+		det->panels[i].clen += det->panels[i].coffset;
+	}
+}
+
+
+static int geom_contains_references(struct detector *det)
+{
+	int i;
+
+	for ( i=0; i<det->n_panels; i++ ) {
+		if ( det->panels[i].clen_from != NULL ) return 1;
+	}
+	return 0;
+}
+
+
 int main(int argc, char *argv[])
 {
 	int c;
 	struct image image;
 	struct gpu_context *gctx = NULL;
-	double *powder;
+	struct image *powder;
+	float *powder_data;
 	char *intfile = NULL;
 	double *intensities;
 	char *rval;
@@ -242,7 +267,6 @@ int main(int argc, char *argv[])
 	char *grad_str = NULL;
 	char *outfile = NULL;
 	char *geometry = NULL;
-	char *beamfile = NULL;
 	char *spectrum_str = NULL;
 	GradientMethod grad;
 	SpectrumType spectrum_type;
@@ -260,16 +284,22 @@ int main(int argc, char *argv[])
 	SymOpList *sym;
 	int nsamples = 3;
 	gsl_rng *rng;
-	int background = 0;
+	double background = 0.0;
 	char *template_file = NULL;
 	Stream *st = NULL;
 	int no_fringes = 0;
+	double nphotons = 1e12;
+	double beam_radius = 1e-6;  /* metres */
+	double bandwidth = 0.01;
+	double photon_energy = 9000.0;
+	struct beam_params beam;
 
 	/* Long options */
 	const struct option longopts[] = {
 		{"help",               0, NULL,               'h'},
-		{"version",            0, NULL,                7 },
+		{"version",            0, NULL,               'v'},
 		{"gpu",                0, &config_gpu,         1},
+		{"beam",               1, NULL,               'b'},
 		{"random-orientation", 0, NULL,               'r'},
 		{"number",             1, NULL,               'n'},
 		{"no-images",          0, &config_noimages,    1},
@@ -281,22 +311,27 @@ int main(int argc, char *argv[])
 		{"pdb",                1, NULL,               'p'},
 		{"output",             1, NULL,               'o'},
 		{"geometry",           1, NULL,               'g'},
-		{"beam",               1, NULL,               'b'},
 		{"sample-spectrum",    1, NULL,               's'},
 		{"type-spectrum",      1, NULL,               'x'},
 		{"spectrum",           1, NULL,               'x'},
 		{"really-random",      0, &config_random,      1},
 		{"no-fringes",         0, &no_fringes,         1},
+
 		{"gpu-dev",            1, NULL,                2},
 		{"min-size",           1, NULL,                3},
 		{"max-size",           1, NULL,                4},
 		{"background",         1, NULL,                5},
 		{"template",           1, NULL,                6},
+		{"beam-bandwidth",     1, NULL,                7},
+		{"photon-energy",      1, NULL,                9},
+		{"nphotons",           1, NULL,               10},
+		{"beam-radius",        1, NULL,               11},
+
 		{0, 0, NULL, 0}
 	};
 
 	/* Short options */
-	while ((c = getopt_long(argc, argv, "hrn:i:t:p:o:g:b:y:s:x:",
+	while ((c = getopt_long(argc, argv, "hrn:i:t:p:o:g:y:s:x:vb:",
 	                        longopts, NULL)) != -1) {
 
 		switch (c) {
@@ -305,11 +340,17 @@ int main(int argc, char *argv[])
 			show_help(argv[0]);
 			return 0;
 
-			case 7 :
+			case 'v' :
 			printf("CrystFEL: " CRYSTFEL_VERSIONSTRING "\n");
 			printf(CRYSTFEL_BOILERPLATE"\n");
 			return 0;
 
+			case 'b' :
+			ERROR("WARNING: This version of CrystFEL no longer "
+			      "uses beam files.  Please remove the beam file "
+			      "from your pattern_sim command line.\n");
+			return 1;
+
 			case 'r' :
 			config_randomquat = 1;
 			break;
@@ -346,10 +387,6 @@ int main(int argc, char *argv[])
 			geometry = strdup(optarg);
 			break;
 
-			case 'b' :
-			beamfile = strdup(optarg);
-			break;
-
 			case 'y' :
 			sym_str = strdup(optarg);
 			break;
@@ -376,6 +413,7 @@ int main(int argc, char *argv[])
 				ERROR("Invalid minimum size.\n");
 				return 1;
 			}
+			min_size /= 1e9;
 			random_size++;
 			break;
 
@@ -385,11 +423,12 @@ int main(int argc, char *argv[])
 				ERROR("Invalid maximum size.\n");
 				return 1;
 			}
+			max_size /= 1e9;
 			random_size++;
 			break;
 
 			case 5 :
-			background = strtol(optarg, &rval, 10);
+			background = strtod(optarg, &rval);
 			if ( *rval != '\0' ) {
 				ERROR("Invalid background level.\n");
 				return 1;
@@ -400,6 +439,55 @@ int main(int argc, char *argv[])
 			template_file = strdup(optarg);
 			break;
 
+			case 7 :
+			bandwidth = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid beam bandwidth.\n");
+				return 1;
+			}
+			if ( bandwidth < 0.0 ) {
+				ERROR("Beam bandwidth must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 9 :
+			photon_energy = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid photon energy.\n");
+				return 1;
+			}
+			if ( photon_energy < 0.0 ) {
+				ERROR("Photon energy must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 10 :
+			nphotons = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid number of photons.\n");
+				return 1;
+			}
+			if ( nphotons < 0.0 ) {
+				ERROR("Number of photons must be positive.\n");
+				return 1;
+			}
+			break;
+
+			case 11 :
+			beam_radius = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid beam radius.\n");
+				return 1;
+			}
+			if ( beam_radius < 0.0 ) {
+				ERROR("Beam radius must be positive.\n");
+				return 1;
+			}
+			break;
+
+
 			case 0 :
 			break;
 
@@ -474,12 +562,26 @@ int main(int argc, char *argv[])
 		ERROR("You need to specify a geometry file with --geometry\n");
 		return 1;
 	}
-
-	if ( beamfile == NULL ) {
-		ERROR("You need to specify a beam parameter file"
-		      " with --beam\n");
+	image.beam = &beam;
+	image.det = get_detector_geometry(geometry, image.beam);
+	if ( image.det == NULL ) {
+		ERROR("Failed to read detector geometry from '%s'\n", geometry);
 		return 1;
 	}
+	free(geometry);
+	if ( (beam.photon_energy > 0.0) && (beam.photon_energy_from == NULL) ) {
+		ERROR("WARNING: An explicit photon energy was found in the "
+		      "geometry file.  It will be ignored!\n");
+		ERROR("The value given on the command line "
+		      "(with --photon-energy) will be used instead.\n");
+	}
+	if ( geom_contains_references(image.det) ) {
+		ERROR("Geometry file contains a reference to an HDF5 location"
+		      " for the camera length.  Change it to a numerical value "
+		      " and try again.\n");
+		return 1;
+	}
+	fixup_geom(image.det);
 
 	if ( spectrum_str == NULL ) {
 		STATUS("You didn't specify a spectrum type, so"
@@ -500,6 +602,12 @@ int main(int argc, char *argv[])
 	}
 	free(spectrum_str);
 
+	/* Load unit cell */
+	input_cell = load_cell_from_file(filename);
+	if ( input_cell == NULL ) {
+		exit(1);
+	}
+
 	if ( intfile == NULL ) {
 
 		/* Gentle reminder */
@@ -519,7 +627,6 @@ int main(int argc, char *argv[])
 			ERROR("Problem reading input file %s\n", intfile);
 			return 1;
 		}
-		free(intfile);
 
 		if ( grad == GRADIENT_PHASED ) {
 			phases = phases_from_list(reflections);
@@ -534,6 +641,13 @@ int main(int argc, char *argv[])
 		if ( check_list_symmetry(reflections, sym) ) {
 			ERROR("The input reflection list does not appear to"
 			      " have symmetry %s\n", symmetry_name(sym));
+			if ( cell_get_lattice_type(input_cell) == L_MONOCLINIC )
+			{
+				ERROR("You may need to specify the unique axis "
+				      "in your point group.  The default is "
+				      "unique axis c.\n");
+				ERROR("See 'man crystfel' for more details.\n");
+			}
 			return 1;
 		}
 
@@ -541,40 +655,14 @@ int main(int argc, char *argv[])
 
 	}
 
-	image.det = get_detector_geometry(geometry);
-	if ( image.det == NULL ) {
-		ERROR("Failed to read detector geometry from '%s'\n", geometry);
-		return 1;
-	}
-	free(geometry);
-
-	image.beam = get_beam_parameters(beamfile);
-	if ( image.beam == NULL ) {
-		ERROR("Failed to read beam parameters from '%s'\n", beamfile);
-		return 1;
-	}
-
 	/* Define image parameters */
 	image.width = image.det->max_fs + 1;
 	image.height = image.det->max_ss + 1;
-	if ( image.beam->photon_energy_from != NULL ) {
-		ERROR("Photon energy must be specified, not taken from the"
-		      " HDF5 file.  Please alter %s accordingly.\n", beamfile)
-		return 1;
-	}
 
-	double wl = ph_en_to_lambda(eV_to_J(image.beam->photon_energy));
+	double wl = ph_en_to_lambda(eV_to_J(photon_energy));
 	image.lambda = wl;
-	image.bw = image.beam->bandwidth;
-	image.div = image.beam->divergence;
+	image.bw = bandwidth;
 	image.nsamples = nsamples;
-	free(beamfile);
-
-	/* Load unit cell */
-	input_cell = load_cell_from_pdb(filename);
-	if ( input_cell == NULL ) {
-		exit(1);
-	}
 
 	/* Initialise stuff */
 	image.filename = NULL;
@@ -591,11 +679,54 @@ int main(int argc, char *argv[])
 		gsl_rng_set(rng, seed);
 	}
 
-	powder = calloc(image.width*image.height, sizeof(*powder));
+	powder = calloc(1, sizeof(struct image));
+	powder->width = image.width;
+	powder->height = image.height;
+	powder->det = image.det;
+	powder_data = calloc(image.width*image.height, sizeof(float));
+	powder->data = powder_data;
 
 	/* Splurge a few useful numbers */
-	STATUS("Wavelength is %f nm\n", image.lambda/1.0e-9);
+	STATUS("Simulation parameters:\n");
+	STATUS("                  Photon energy: %.2f eV (wavelength %.5f A)\n",
+	       photon_energy, image.lambda*1e10);
+	STATUS("              Number of photons: %.0f (%.2f mJ)\n", nphotons,
+	       eV_to_J(photon_energy)*nphotons*1e3);
+	STATUS("                Beam divergence: not simulated\n");
+	STATUS("                    Beam radius: %.2f microns\n",
+	      beam_radius*1e6);
+	STATUS("                     Background: %.2f photons\n", background);
 
+	switch ( spectrum_type ) {
+
+		case SPECTRUM_TOPHAT:
+		STATUS("                 X-ray spectrum: top hat, "
+		       "width %.5f %%\n", image.bw*100.0);
+		break;
+
+		case SPECTRUM_SASE:
+		STATUS("                 X-ray spectrum: SASE, "
+		       "bandwidth %.5f %%\n", image.bw*100.0);
+		break;
+
+		case SPECTRUM_TWOCOLOUR:
+		STATUS("                 X-ray spectrum: two colour, "
+		       "separation %.5f %%\n", image.bw*100.0);
+		break;
+	}
+	if ( random_size ) {
+		STATUS("                   Crystal size: random, between "
+		       "%.2f and %.2f nm along each of a, b and c\n",
+		       min_size*1e9, max_size*1e9);
+	} else {
+		STATUS("                   Crystal size: 8 unit cells along "
+		       "each of a, b and c\n");
+	}
+	if ( intfile == NULL ) {
+		STATUS("               Full intensities: all equal");
+	} else {
+		STATUS("               Full intensities: from %s\n", intfile);
+	}
 	do {
 
 		int na, nb, nc;
@@ -704,23 +835,30 @@ int main(int argc, char *argv[])
 	               na, nb, nc, na*a/1.0e-9, nb*b/1.0e-9, nc*c/1.0e-9);
 
 		if ( config_gpu ) {
+
+			int err;
+
 			if ( gctx == NULL ) {
 				gctx = setup_gpu(config_nosfac,
 				                 intensities, flags, sym_str,
 				                 gpu_dev);
 			}
-			get_diffraction_gpu(gctx, &image, na, nb, nc, cell,
-			                    no_fringes);
+			err = get_diffraction_gpu(gctx, &image, na, nb, nc,
+			                          cell, no_fringes);
+			if ( err ) image.data = NULL;
+
 		} else {
 			get_diffraction(&image, na, nb, nc, intensities, phases,
 			                flags, cell, grad, sym, no_fringes);
 		}
 		if ( image.data == NULL ) {
 			ERROR("Diffraction calculation failed.\n");
+			done = 1;
 			goto skip;
 		}
 
-		record_image(&image, !config_nonoise, background, rng);
+		record_image(&image, !config_nonoise, background, rng,
+		             beam_radius, nphotons);
 
 		if ( powder_fn != NULL ) {
 
@@ -730,14 +868,12 @@ int main(int argc, char *argv[])
 
 			for ( x=0; x<image.width; x++ ) {
 			for ( y=0; y<image.height; y++ ) {
-				powder[x+w*y] += (double)image.data[x+w*y];
+				powder->data[x+w*y] += (double)image.data[x+w*y];
 			}
 			}
 
 			if ( !(ndone % 10) ) {
-				hdf5_write(powder_fn, powder,
-				           image.width, image.height,
-				           H5T_NATIVE_DOUBLE);
+				hdf5_write_image(powder_fn, powder, NULL);
 			}
 		}
 
@@ -755,7 +891,7 @@ int main(int argc, char *argv[])
 			number++;
 
 			/* Write the output file */
-			hdf5_write_image(filename, &image);
+			hdf5_write_image(filename, &image, NULL);
 
 		}
 
@@ -773,9 +909,7 @@ skip:
 	} while ( !done );
 
 	if ( powder_fn != NULL ) {
-		hdf5_write(powder_fn, powder,
-		           image.width, image.height,
-		           H5T_NATIVE_DOUBLE);
+		hdf5_write_image(powder_fn, powder, NULL);
 	}
 
 	if ( gctx != NULL ) {
@@ -783,8 +917,9 @@ skip:
 	}
 
 	free(image.det->panels);
+	free(intfile);
 	free(image.det);
-	free(image.beam);
+	free(powder->data);
 	free(powder);
 	cell_free(input_cell);
 	free(intensities);
