@@ -3,11 +3,11 @@
  *
  * Post refinement
  *
- * Copyright © 2012 Deutsches Elektronen-Synchrotron DESY,
- *                  a research centre of the Helmholtz Association.
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
+ *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2012 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -43,40 +43,37 @@
 #include "geometry.h"
 
 
-/* Refineable parameters.
- * Don't forget to add new things to backup_crystal() et al.! */
-enum {
-	REF_ASX,
-	REF_ASY,
-	REF_ASZ,
-	REF_BSX,
-	REF_BSY,
-	REF_BSZ,
-	REF_CSX,
-	REF_CSY,
-	REF_CSZ,
-	REF_R,
-	NUM_PARAMS,
-	REF_DIV,
-};
-
-
 struct prdata
 {
 	int refined;
 	int n_filtered;
+	double initial_residual;
+	double initial_free_residual;
+	double final_residual;
+	double final_free_residual;
+};
+
+enum prflag
+{
+	PRFLAG_OK = 0,
+	PRFLAG_FEWREFL = 16,
+	PRFLAG_SOLVEFAIL = 17,
+	PRFLAG_EARLY = 18,
+	PRFLAG_CC = 19,
+	PRFLAG_BIGB = 20,
 };
 
 
-extern struct prdata pr_refine(Crystal *cr, const RefList *full,
-                               PartialityModel pmodel);
+extern const char *str_prflag(enum prflag flag);
 
-/* Exported so it can be poked by tests/pr_gradient_check */
-extern double p_gradient(Crystal *cr, int k, Reflection *refl,
-                         PartialityModel pmodel);
+extern void refine_all(Crystal **crystals, int n_crystals,
+                       RefList *full, int nthreads, PartialityModel pmodel,
+                       int no_scale, int no_pr,
+                       double *initial_residual, double *initial_free_residual,
+                       double *final_residual, double *final_free_residual);
 
-
-extern double l_gradient(Crystal *cr, int k, Reflection *refl,
-                         PartialityModel pmodel);
+/* Exported so it can be poked by tests/pr_p_gradient_check */
+extern double gradient(Crystal *cr, int k, Reflection *refl,
+                       PartialityModel pmodel);
 
 #endif	/* POST_REFINEMENT_H */
