@@ -3,12 +3,13 @@
  *
  * Assemble and process FEL Bragg intensities
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  * Copyright © 2012 Lorenzo Galli
  *
  * Authors:
- *   2009-2014 Thomas White <taw@physics.org>
+ *   2015      Keitaro Yamashita <k.yamashita@spring8.or.jp>
+ *   2009-2015 Thomas White <taw@physics.org>
  *   2011      Andrew Martin <andrew.martin@desy.de>
  *   2012      Lorenzo Galli <lorenzo.galli@desy.de>
  *   2014      Chunhong Yoon <chun.hong.yoon@desy.de>
@@ -67,6 +68,7 @@ static void show_help(const char *s)
 "  -i, --input=<filename>    Specify input filename (\"-\" for stdin).\n"
 "  -o, --output=<filename>   Specify output filename for merged intensities\n"
 "                             Default: processed.hkl).\n"
+"      --stat=<filename>     Specify output filename for merging statistics.\n"
 "  -y, --symmetry=<sym>      Merge according to point group <sym>.\n"
 "\n"
 "      --start-after=<n>     Skip <n> crystals at the start of the stream.\n"
@@ -78,11 +80,14 @@ static void show_help(const char *s)
 "\n"
 "      --scale               Scale each pattern for best fit with the current\n"
 "                             model.\n"
+"      --even-only           Merge even numbered crystals only\n"
+"      --odd-only            Merge odd numbered crystals only\n"
 "      --no-polarisation     Disable polarisation correction.\n"
 "      --min-measurements=<n> Require at least <n> measurements before a\n"
 "                             reflection appears in the output.  Default: 2\n"
 "      --min-snr=<n>         Require individual intensity measurements to\n"
 "                             have I > n * sigma(I).  Default: -infinity.\n"
+"      --min-cc=<n>          Reject frames with CC less than n. Default: infinity.\n"
 "      --max-adu=<n>         Maximum peak value.  Default: infinity.\n"
 "      --min-res=<n>         Merge only crystals which diffract above <n> A.\n"
 "      --push-res=<n>        Integrate higher than apparent resolution cutoff.\n"
@@ -180,6 +185,59 @@ static double scale_intensities(RefList *reference, RefList *new,
 }
 
 
+static double cc_intensities(RefList *reference, RefList *new,
+                             const SymOpList *sym)
+{
+	/* "x" is "reference" */
+	float s_xy = 0.0;
+	float s_x = 0.0;
+	float s_y = 0.0;
+	float s_x2 = 0.0;
+	float s_y2 = 0.0;
+	int n = 0;
+	float t1, t2;
+
+	Reflection *refl;
+	RefListIterator *iter;
+
+	for ( refl = first_refl(new, &iter);
+	      refl != NULL;
+	      refl = next_refl(refl, iter) )
+	{
+
+		double i1, i2;
+		signed int hu, ku, lu;
+		signed int h, k, l;
+		Reflection *reference_version;
+
+		get_indices(refl, &h, &k, &l);
+		get_asymm(sym, h, k, l, &hu, &ku, &lu);
+
+		reference_version = find_refl(reference, hu, ku, lu);
+		if ( reference_version == NULL ) continue;
+
+		i1 = get_intensity(reference_version);
+		i2 = get_intensity(refl);
+
+
+		s_xy += i1 * i2;
+		s_x += i1;
+		s_y += i2;
+		s_x2 += i1 * i1;
+		s_y2 += i2 * i2;
+		n++;
+
+	}
+
+	t1 = s_x2 - s_x*s_x / n;
+	t2 = s_y2 - s_y*s_y / n;
+
+	if ( (t1 <= 0.0) || (t2 <= 0.0) ) return 0.0;
+
+	return (s_xy - s_x*s_y/n) / sqrt(t1*t2);
+}
+
+
 static double *check_hist_size(int n, double *hist_vals)
 {
 	int ns;
@@ -203,25 +261,38 @@ static int merge_crystal(RefList *model, struct image *image, Crystal *cr,
                          double **hist_vals, signed int hist_h,
                          signed int hist_k, signed int hist_l, int *hist_n,
                          int config_nopolar, double min_snr, double max_adu,
-                         double push_res)
+                         double push_res, double min_cc, int do_scale,
+                         FILE *stat)
 {
 	Reflection *refl;
 	RefListIterator *iter;
-	double scale;
+	RefList *new_refl;
+	double scale, cc;
+
+	new_refl = crystal_get_reflections(cr);
 
 	/* First, correct for polarisation */
 	if ( !config_nopolar ) {
-		polarisation_correction(crystal_get_reflections(cr),
-		                        crystal_get_cell(cr), image);
+		polarisation_correction(new_refl, crystal_get_cell(cr), image);
 	}
 
 	if ( reference != NULL ) {
-		scale = scale_intensities(reference,
-		                          crystal_get_reflections(cr), sym);
+		if ( do_scale ) {
+			scale = scale_intensities(reference, new_refl, sym);
+		} else {
+			scale = 1.0;
+		}
+		cc = cc_intensities(reference, new_refl, sym);
+		if ( cc < min_cc ) return 1;
+		if ( isnan(scale) ) return 1;
+		if ( scale <= 0.0 ) return 1;
+		if ( stat != NULL ) {
+			fprintf(stat, "%s %f %f\n", image->filename, scale, cc);
+		}
+
 	} else {
 		scale = 1.0;
 	}
-	if ( isnan(scale) ) return 1;
 
 	for ( refl = first_refl(crystal_get_reflections(cr), &iter);
 	      refl != NULL;
@@ -311,7 +382,6 @@ static void display_progress(int n_images, int n_crystals, int n_crystals_used)
 }
 
 
-
 static int merge_all(Stream *st, RefList *model, RefList *reference,
                      const SymOpList *sym,
                      double **hist_vals, signed int hist_h,
@@ -319,7 +389,7 @@ static int merge_all(Stream *st, RefList *model, RefList *reference,
                      int *hist_i, int config_nopolar, int min_measurements,
                      double min_snr, double max_adu,
                      int start_after, int stop_after, double min_res,
-                     double push_res)
+                     double push_res, double min_cc, int do_scale, int flag_even_odd, char *stat_output)
 {
 	int rval;
 	int n_images = 0;
@@ -328,6 +398,15 @@ static int merge_all(Stream *st, RefList *model, RefList *reference,
 	Reflection *refl;
 	RefListIterator *iter;
 	int n_crystals_seen = 0;
+	FILE *stat = NULL;
+
+	if ( stat_output != NULL ) {
+		stat = fopen(stat_output, "w");
+		if ( stat == NULL ) {
+			ERROR("Failed to open statistics output file %s\n",
+			      stat_output);
+		}
+	}
 
 	do {
 
@@ -337,7 +416,8 @@ static int merge_all(Stream *st, RefList *model, RefList *reference,
 		image.det = NULL;
 
 		/* Get data from next chunk */
-		rval = read_chunk(st, &image);
+		rval = read_chunk_2(st, &image, STREAM_READ_REFLECTIONS |
+		                    STREAM_READ_UNITCELL);
 		if ( rval ) break;
 
 		n_images++;
@@ -348,20 +428,20 @@ static int merge_all(Stream *st, RefList *model, RefList *reference,
 			Crystal *cr = image.crystals[i];
 
 			n_crystals_seen++;
-			if ( n_crystals_seen <= start_after ) continue;
-
-			if ( crystal_get_resolution_limit(cr) < min_res ) {
-				continue;
+			if ( (n_crystals_seen > start_after)
+			  && (crystal_get_resolution_limit(cr) >= min_res)
+			  && (flag_even_odd == 2 || n_crystals_seen%2 == flag_even_odd) )
+			{
+				n_crystals++;
+				r = merge_crystal(model, &image, cr, reference,
+				                  sym, hist_vals,
+						  hist_h, hist_k, hist_l,
+				                  hist_i, config_nopolar,
+						  min_snr, max_adu, push_res,
+						  min_cc, do_scale, stat);
+				if ( r == 0 ) n_crystals_used++;
 			}
 
-			n_crystals++;
-			r = merge_crystal(model, &image, cr, reference, sym,
-			                  hist_vals, hist_h, hist_k, hist_l,
-			                  hist_i, config_nopolar, min_snr,
-			                  max_adu, push_res);
-
-			if ( r == 0 ) n_crystals_used++;
-
 			reflist_free(crystal_get_reflections(cr));
 			cell_free(crystal_get_cell(cr));
 			crystal_free(cr);
@@ -397,6 +477,10 @@ static int merge_all(Stream *st, RefList *model, RefList *reference,
 		set_esd_intensity(refl, sqrt(var)/sqrt(red));
 	}
 
+	if ( stat != NULL ) {
+		fclose(stat);
+	}
+
 	return 0;
 }
 
@@ -406,9 +490,13 @@ int main(int argc, char *argv[])
 	int c;
 	char *filename = NULL;
 	char *output = NULL;
+	char *stat_output = NULL;
 	Stream *st;
 	RefList *model;
 	int config_scale = 0;
+	int config_evenonly = 0;
+	int config_oddonly = 0;
+	int flag_even_odd = 2;
 	char *sym_str = NULL;
 	SymOpList *sym;
 	char *histo = NULL;
@@ -429,6 +517,8 @@ int main(int argc, char *argv[])
 	double max_adu = +INFINITY;
 	double min_res = 0.0;
 	double push_res = +INFINITY;
+	double min_cc = -INFINITY;
+	int twopass = 0;
 
 	/* Long options */
 	const struct option longopts[] = {
@@ -438,6 +528,8 @@ int main(int argc, char *argv[])
 		{"start-after",        1, NULL,               's'},
 		{"stop-after",         1, NULL,               'f'},
 		{"scale",              0, &config_scale,       1},
+		{"even-only",          0, &config_evenonly,    1},
+		{"odd-only",           0, &config_oddonly,     1},
 		{"no-polarisation",    0, &config_nopolar,     1},
 		{"no-polarization",    0, &config_nopolar,     1},
 		{"symmetry",           1, NULL,               'y'},
@@ -450,6 +542,8 @@ int main(int argc, char *argv[])
 		{"push-res",           1, NULL,                6},
 		{"res-push",           1, NULL,                6}, /* compat */
 		{"version",            0, NULL,                7},
+		{"min-cc",             1, NULL,                8},
+		{"stat",               1, NULL,                9},
 		{0, 0, NULL, 0}
 	};
 
@@ -475,7 +569,8 @@ int main(int argc, char *argv[])
 			errno = 0;
 			start_after = strtod(optarg, &rval);
 			if ( *rval != '\0' ) {
-				ERROR("Invalid value for --start-after.\n");
+				ERROR("Invalid value for --start-after (%s)\n",
+				      optarg);
 				return 1;
 			}
 			break;
@@ -484,7 +579,8 @@ int main(int argc, char *argv[])
 			errno = 0;
 			stop_after = strtod(optarg, &rval);
 			if ( *rval != '\0' ) {
-				ERROR("Invalid value for --stop-after.\n");
+				ERROR("Invalid value for --stop-after (%s)\n",
+				      optarg);
 				return 1;
 			}
 			break;
@@ -560,6 +656,21 @@ int main(int argc, char *argv[])
 			case '?' :
 			break;
 
+			case 8 :
+			errno = 0;
+			min_cc = strtod(optarg, &rval);
+			if ( *rval != '\0' ) {
+				ERROR("Invalid value for --min-cc.\n");
+				return 1;
+			}
+			twopass = 1;
+			break;
+
+			case 9 :
+			stat_output = strdup(optarg);
+			twopass = 1;
+			break;
+
 			case 0 :
 			break;
 
@@ -635,17 +746,29 @@ int main(int argc, char *argv[])
 
 	}
 
+	if ( config_evenonly && config_oddonly ) {
+		ERROR("Don't specify both --even-only and --odd-only\n")
+		return 1;
+	}
+
+	/* 0: even-only, 1: odd-only, 2: use all */
+	flag_even_odd = config_evenonly ? 0 : config_oddonly ? 1 : 2;
+
+	/* Need to do a second pass if we are scaling */
+	if ( config_scale ) twopass = 1;
+
 	hist_i = 0;
 	r = merge_all(st, model, NULL, sym, &hist_vals, hist_h, hist_k, hist_l,
 	              &hist_i, config_nopolar, min_measurements, min_snr,
-	              max_adu, start_after, stop_after, min_res, push_res);
+	              max_adu, start_after, stop_after, min_res, push_res,
+	              min_cc, config_scale, flag_even_odd, stat_output);
 	fprintf(stderr, "\n");
 	if ( r ) {
 		ERROR("Error while reading stream.\n");
 		return 1;
 	}
 
-	if ( config_scale ) {
+	if ( twopass ) {
 
 		RefList *reference;
 
@@ -658,20 +781,23 @@ int main(int argc, char *argv[])
 
 			int r;
 
-			STATUS("Extra pass for scaling...\n");
+			STATUS("Second pass for scaling and/or CCs...\n");
 
 			reference = model;
 			model = reflist_new();
 
-			free(hist_vals);
-			hist_vals = malloc(1*sizeof(double));
-			hist_i = 0;
+			if ( hist_vals != NULL ) {
+				free(hist_vals);
+				hist_vals = malloc(1*sizeof(double));
+				hist_i = 0;
+			}
 
-			r = merge_all(st, model, reference, sym,
-				     &hist_vals, hist_h, hist_k, hist_l, &hist_i,
-				     config_nopolar, min_measurements, min_snr,
-				     max_adu, start_after, stop_after, min_res,
-				     push_res);
+			r = merge_all(st, model, reference, sym, &hist_vals,
+			              hist_h, hist_k, hist_l, &hist_i,
+				      config_nopolar, min_measurements, min_snr,
+				      max_adu, start_after, stop_after, min_res,
+				      push_res, min_cc, config_scale,
+				      flag_even_odd, stat_output);
 			fprintf(stderr, "\n");
 			if ( r ) {
 				ERROR("Error while reading stream.\n");
@@ -703,6 +829,7 @@ int main(int argc, char *argv[])
 	reflist_free(model);
 	free(output);
 	free(filename);
+	free(stat_output);
 
 	return 0;
 }
