@@ -3,11 +3,12 @@
  *
  * The processing pipeline for one image
  *
- * Copyright © 2012-2014 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2014 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
+ *   2014      Valerio Mariani
  *
  * This file is part of CrystFEL.
  *
@@ -33,13 +34,16 @@
 #include <config.h>
 #endif
 
+struct index_args;
 
 #include "integration.h"
+#include "im-sandbox.h"
 
 
 enum {
 	PEAK_ZAEF,
 	PEAK_HDF5,
+	PEAK_CXI,
 };
 
 
@@ -61,7 +65,6 @@ struct index_args
 	int peaks;                /* Peak detection method */
 	float tols[4];
 	struct beam_params *beam;
-	char *element;
 	char *hdf5_peak_path;
 	float pk_inn;
 	float pk_mid;
@@ -82,6 +85,10 @@ struct index_args
 	signed int int_diag_l;
 	float push_res;
 	float highres;
+	float fix_profile_r;
+	float fix_bandwidth;
+	float fix_divergence;
+	int predict_refine;
 };
 
 
@@ -89,16 +96,14 @@ struct index_args
 struct pattern_args
 {
 	/* "Input" */
-	char *filename;
-
-	/* "Output" */
-	int n_crystals;
+	struct filename_plus_event *filename_p_e;
 };
 
 
 extern void process_image(const struct index_args *iargs,
                           struct pattern_args *pargs, Stream *st,
-                          int cookie, const char *tmpdir, int results_pipe);
+                          int cookie, const char *tmpdir, int serial,
+                          struct sb_shm *sb_shared);
 
 
-#endif	/* PROCESS_IMAGEs_H */
+#endif	/* PROCESS_IMAGE_H */
