@@ -3,11 +3,11 @@
  *
  * Draw pretty renderings of reflection lists
  *
- * Copyright © 2012-2013 Deutsches Elektronen-Synchrotron DESY,
+ * Copyright © 2012-2015 Deutsches Elektronen-Synchrotron DESY,
  *                       a research centre of the Helmholtz Association.
  *
  * Authors:
- *   2010-2013 Thomas White <taw@physics.org>
+ *   2010-2015 Thomas White <taw@physics.org>
  *
  * This file is part of CrystFEL.
  *
@@ -71,7 +71,7 @@ static void show_help(const char *s)
 "      --zone=<z>          Show the <z>th Laue zone.\n"
 "  -o, --output=<filename> Output filename.  Default: za.pdf\n"
 "      --boost=<val>       Squash colour scale by <val>.\n"
-"  -p, --pdb=<file>        PDB file from which to get the unit cell.\n"
+"  -p, --pdb=<file>        File from which to get the unit cell.\n"
 "  -y, --symmetry=<sym>    Expand reflections according to point group <sym>.\n"
 "\n"
 "  -c, --colscale=<scale>  Use the given colour scale.  Choose from:\n"
@@ -91,6 +91,8 @@ static void show_help(const char *s)
 "                                     reflection (no 'epsilon' correction).\n"
 "\n"
 "      --res-ring=<r>      Draw a resolution ring at <r> Angstroms.\n"
+"      --highres=<r>       Render spots only up to <r> Angstroms.\n"
+"      --no-axes           Do not draw reciprocal space axes.\n"
 "\n"
 "      --colour-key        Draw (only) the key for the current colour scale.\n"
 "                           The key will be written to 'key.pdf' in the\n"
@@ -410,7 +412,7 @@ static void render_za(UnitCell *cell, RefList *list,
                       signed int xh, signed int xk, signed int xl,
                       signed int yh, signed int yk, signed int yl,
                       const char *outfile, double scale_top, signed int zone,
-                      struct resrings *rings)
+                      struct resrings *rings, int noaxes)
 {
 	cairo_surface_t *surface;
 	cairo_t *dctx;
@@ -529,12 +531,6 @@ static void render_za(UnitCell *cell, RefList *list,
 	             max_r, theta, as, bs, cx, cy, scale,
 	             max_val, zone);
 
-	/* Centre marker */
-	cairo_arc(dctx, (double)cx,
-			(double)cy, max_r, 0, 2*M_PI);
-	cairo_set_source_rgb(dctx, 1.0, 0.0, 0.0);
-	cairo_fill(dctx);
-
 	/* Resolution rings */
 	for ( i=0; i<rings->n_rings; i++ ) {
 
@@ -558,37 +554,48 @@ static void render_za(UnitCell *cell, RefList *list,
 		cairo_restore(dctx);
 	}
 
-	/* Draw indexing lines */
-	cairo_set_line_cap(dctx, CAIRO_LINE_CAP_ROUND);
-	cairo_set_line_width(dctx, 2.0);
-	cairo_move_to(dctx, (double)cx, (double)cy);
-	u = rmax*sin(theta);
-	v = rmax*cos(theta);
-	cairo_line_to(dctx, cx+u*scale, cy+v*scale);
-	cairo_set_source_rgb(dctx, 0.0, 1.0, 0.0);
-	cairo_stroke(dctx);
+	if ( !noaxes ) {
 
-	cairo_set_font_size(dctx, 40.0);
-	snprintf(tmp, 255, "%i%i%i", abs(xh), abs(xk), abs(xl));
-	cairo_text_extents(dctx, tmp, &size);
+		/* Centre marker */
+		cairo_arc(dctx, (double)cx,
+				(double)cy, max_r, 0, 2*M_PI);
+		cairo_set_source_rgb(dctx, 1.0, 0.0, 0.0);
+		cairo_fill(dctx);
 
-	cairo_move_to(dctx, cx+u*scale + 20.0, cy+v*scale + size.height/2.0);
-	render_overlined_indices(dctx, xh, xk, xl);
-	cairo_fill(dctx);
+		/* Draw indexing lines */
+		cairo_set_line_cap(dctx, CAIRO_LINE_CAP_ROUND);
+		cairo_set_line_width(dctx, 2.0);
+		cairo_move_to(dctx, (double)cx, (double)cy);
+		u = rmax*sin(theta);
+		v = rmax*cos(theta);
+		cairo_line_to(dctx, cx+u*scale, cy+v*scale);
+		cairo_set_source_rgb(dctx, 0.0, 1.0, 0.0);
+		cairo_stroke(dctx);
 
-	snprintf(tmp, 255, "%i%i%i", abs(yh), abs(yk), abs(yl));
-	cairo_text_extents(dctx, tmp, &size);
+		cairo_set_font_size(dctx, 40.0);
+		snprintf(tmp, 255, "%i%i%i", abs(xh), abs(xk), abs(xl));
+		cairo_text_extents(dctx, tmp, &size);
 
-	cairo_set_line_width(dctx, 2.0);
-	cairo_move_to(dctx, (double)cx, (double)cy);
-	cairo_line_to(dctx, cx, cy+rmax*scale);
-	cairo_set_source_rgb(dctx, 0.0, 1.0, 0.0);
-	cairo_stroke(dctx);
+		cairo_move_to(dctx, cx+u*scale + 20.0,
+		                    cy+v*scale + size.height/2.0);
+		render_overlined_indices(dctx, xh, xk, xl);
+		cairo_fill(dctx);
 
-	cairo_move_to(dctx, cx - size.width/2.0,
-	                    cy+rmax*scale + size.height + 20.0);
-	render_overlined_indices(dctx, yh, yk, yl);
-	cairo_fill(dctx);
+		snprintf(tmp, 255, "%i%i%i", abs(yh), abs(yk), abs(yl));
+		cairo_text_extents(dctx, tmp, &size);
+
+		cairo_set_line_width(dctx, 2.0);
+		cairo_move_to(dctx, (double)cx, (double)cy);
+		cairo_line_to(dctx, cx, cy+rmax*scale);
+		cairo_set_source_rgb(dctx, 0.0, 1.0, 0.0);
+		cairo_stroke(dctx);
+
+		cairo_move_to(dctx, cx - size.width/2.0,
+			            cy+rmax*scale + size.height + 20.0);
+		render_overlined_indices(dctx, yh, yk, yl);
+		cairo_fill(dctx);
+
+	}
 
 	if ( png ) {
 		int r = cairo_surface_write_to_png(surface, outfile);
@@ -741,7 +748,7 @@ int main(int argc, char *argv[])
 	int config_sqrt = 0;
 	int config_colkey = 0;
 	int config_zawhinge = 0;
-	char *pdb = NULL;
+	char *cellfile = NULL;
 	int r = 0;
 	double boost = 1.0;
 	char *sym_str = NULL;
@@ -760,6 +767,8 @@ int main(int argc, char *argv[])
 	long int zone = 0;
 	double res;
 	struct resrings rings;
+	float highres = -1.0;
+	int config_noaxes = 0;
 
 	rings.n_rings = 0;
 
@@ -778,9 +787,11 @@ int main(int argc, char *argv[])
 		{"right",              1, NULL,               'r'},
 		{"counts",             0, &config_sqrt,        1},
 		{"colour-key",         0, &config_colkey,      1},
+		{"no-axes",            0, &config_noaxes,      1},
 		{"scale-top",          1, NULL,                2},
 		{"zone",               1, NULL,                3},
 		{"res-ring",           1, NULL,                4},
+		{"highres",            1, NULL,                6},
 		{0, 0, NULL, 0}
 	};
 
@@ -800,7 +811,7 @@ int main(int argc, char *argv[])
 			return 0;
 
 			case 'p' :
-			pdb = strdup(optarg);
+			cellfile = strdup(optarg);
 			break;
 
 			case 'b' :
@@ -869,6 +880,13 @@ int main(int argc, char *argv[])
 			add_ring(&rings, res);
 			break;
 
+			case 6 :
+			if ( sscanf(optarg, "%e", &highres) != 1 ) {
+				ERROR("Invalid value for --highres\n");
+				return 1;
+			}
+			break;
+
 			case 0 :
 			break;
 
@@ -888,8 +906,8 @@ int main(int argc, char *argv[])
 		      " any longer (I ignored it for you).\n");
 	}
 
-	if ( (pdb == NULL) && !config_colkey ) {
-		ERROR("You must specify the PDB containing the unit cell.\n");
+	if ( (cellfile == NULL) && !config_colkey ) {
+		ERROR("You must specify the unit cell.\n");
 		return 1;
 	}
 
@@ -974,9 +992,9 @@ int main(int argc, char *argv[])
 
 	infile = argv[optind];
 
-	cell = load_cell_from_pdb(pdb);
+	cell = load_cell_from_file(cellfile);
 	if ( cell == NULL ) {
-		ERROR("Couldn't load unit cell from %s\n", pdb);
+		ERROR("Couldn't load unit cell from %s\n", cellfile);
 		return 1;
 	}
 	list = read_reflections(infile);
@@ -987,13 +1005,23 @@ int main(int argc, char *argv[])
 	if ( check_list_symmetry(list, sym) ) {
 		ERROR("The input reflection list does not appear to"
 		      " have symmetry %s\n", symmetry_name(sym));
+		if ( cell_get_lattice_type(cell) == L_MONOCLINIC ) {
+			ERROR("You may need to specify the unique axis in your "
+			      "point group.  The default is unique axis c.\n");
+			ERROR("See 'man crystfel' for more details.\n");
+		}
 		return 1;
 	}
 
-	render_za(cell, list, boost, sym, wght, colscale,
-	          rh, rk, rl, dh, dk, dl, outfile, scale_top, zone, &rings);
+	if ( highres > 0.0 ) {
+		list = res_cutoff(list, cell, 0.0, 1.0/(highres*1e-10));
+	}
 
-	free(pdb);
+	render_za(cell, list, boost, sym, wght, colscale,
+	          rh, rk, rl, dh, dk, dl, outfile, scale_top, zone, &rings,
+	          config_noaxes);
+
+	free(cellfile);
 	free_symoplist(sym);
 	reflist_free(list);
 	if ( outfile != NULL ) free(outfile);
