@@ -45,7 +45,8 @@
 
 
 static void scan_partialities(RefList *reflections, RefList *compare,
-                              int *valid, long double *vals[3], int idx)
+                              int *valid, long double *vals[3], int idx,
+                              PartialityModel pmodel)
 {
 	int i;
 	Reflection *refl;
@@ -58,8 +59,7 @@ static void scan_partialities(RefList *reflections, RefList *compare,
 	{
 		signed int h, k, l;
 		Reflection *refl2;
-		double r1, r2, p;
-		int clamp_low, clamp_high;
+		double rlow, rhigh, p;
 
 		get_indices(refl, &h, &k, &l);
 		refl2 = find_refl(compare, h, k, l);
@@ -69,23 +69,13 @@ static void scan_partialities(RefList *reflections, RefList *compare,
 			continue;
 		}
 
-		get_partial(refl2, &r1, &r2, &p, &clamp_low, &clamp_high);
-		if ( clamp_low && clamp_high ) {
-			if ( !within_tolerance(p, 1.0, 0.001) ) {
-
-				signed int h, k, l;
-
-				get_indices(refl, &h, &k, &l);
-
-				ERROR("%3i %3i %3i - double clamped but"
-				      " partiality not close to 1.0 (%5.2f)\n",
-				      h, k, l, p);
-
-			}
-			valid[i] = 0;
+		get_partial(refl2, &rlow, &rhigh, &p);
+		vals[idx][i] = p;
+		if ( unlikely(p < 0.0) ) {
+			ERROR("Negative partiality! %3i %3i %3i  %f\n",
+			      h, k, l, p);
 		}
 
-		vals[idx][i] = p;
 		i++;
 	}
 }
@@ -103,15 +93,15 @@ static UnitCell *new_shifted_cell(UnitCell *input, int k, double shift)
 	                    &csx, &csy, &csz);
 	switch ( k )
 	{
-		case REF_ASX :  asx += shift;  break;
-		case REF_ASY :  asy += shift;  break;
-		case REF_ASZ :  asz += shift;  break;
-		case REF_BSX :  bsx += shift;  break;
-		case REF_BSY :  bsy += shift;  break;
-		case REF_BSZ :  bsz += shift;  break;
-		case REF_CSX :  csx += shift;  break;
-		case REF_CSY :  csy += shift;  break;
-		case REF_CSZ :  csz += shift;  break;
+		case GPARAM_ASX :  asx += shift;  break;
+		case GPARAM_ASY :  asy += shift;  break;
+		case GPARAM_ASZ :  asz += shift;  break;
+		case GPARAM_BSX :  bsx += shift;  break;
+		case GPARAM_BSY :  bsy += shift;  break;
+		case GPARAM_BSZ :  bsz += shift;  break;
+		case GPARAM_CSX :  csx += shift;  break;
+		case GPARAM_CSY :  csy += shift;  break;
+		case GPARAM_CSZ :  csz += shift;  break;
 	}
 	cell_set_reciprocal(cell, asx, asy, asz, bsx, bsy, bsz, csx, csy, csz);
 
@@ -123,7 +113,7 @@ static void shift_parameter(struct image *image, int k, double shift)
 {
 	switch ( k )
 	{
-		case REF_DIV : image->div += shift;  break;
+		case GPARAM_DIV : image->div += shift;  break;
 	}
 }
 
@@ -145,21 +135,21 @@ static Crystal *new_shifted_crystal(Crystal *cr, int refine, double incr_val)
 
 	switch ( refine ) {
 
-		case REF_ASX :
-		case REF_ASY :
-		case REF_ASZ :
-		case REF_BSX :
-		case REF_BSY :
-		case REF_BSZ :
-		case REF_CSX :
-		case REF_CSY :
-		case REF_CSZ :
+		case GPARAM_ASX :
+		case GPARAM_ASY :
+		case GPARAM_ASZ :
+		case GPARAM_BSX :
+		case GPARAM_BSY :
+		case GPARAM_BSZ :
+		case GPARAM_CSX :
+		case GPARAM_CSY :
+		case GPARAM_CSZ :
 		cell = new_shifted_cell(crystal_get_cell(cr), refine,
 		                        incr_val);
 		crystal_set_cell(cr_new, cell);
 		break;
 
-		case REF_R :
+		case GPARAM_R :
 		cell = cell_new_from_cell(crystal_get_cell(cr));
 		crystal_set_cell(cr_new, cell);
 		crystal_set_profile_radius(cr_new, r + incr_val);
@@ -182,7 +172,7 @@ static void calc_either_side(Crystal *cr, double incr_val,
 	RefList *compare;
 	struct image *image = crystal_get_image(cr);
 
-	if ( (refine != REF_DIV) ) {
+	if ( (refine != GPARAM_DIV) ) {
 
 		Crystal *cr_new;
 
@@ -190,7 +180,7 @@ static void calc_either_side(Crystal *cr, double incr_val,
 		cr_new = new_shifted_crystal(cr, refine, -incr_val);
 		compare = find_intersections(image, cr_new, pmodel);
 		scan_partialities(crystal_get_reflections(cr), compare, valid,
-		                  vals, 0);
+		                  vals, 0, pmodel);
 		cell_free(crystal_get_cell(cr_new));
 		crystal_free(cr_new);
 		reflist_free(compare);
@@ -198,7 +188,7 @@ static void calc_either_side(Crystal *cr, double incr_val,
 		cr_new = new_shifted_crystal(cr, refine, +incr_val);
 		compare = find_intersections(image, cr_new, pmodel);
 		scan_partialities(crystal_get_reflections(cr), compare, valid,
-		                  vals, 2);
+		                  vals, 2, pmodel);
 		cell_free(crystal_get_cell(cr_new));
 		crystal_free(cr_new);
 		reflist_free(compare);
@@ -212,14 +202,14 @@ static void calc_either_side(Crystal *cr, double incr_val,
 		shift_parameter(&im_moved, refine, -incr_val);
 		compare = find_intersections(&im_moved, cr, pmodel);
 		scan_partialities(crystal_get_reflections(cr), compare,
-		                  valid, vals, 0);
+		                  valid, vals, 0, pmodel);
 		reflist_free(compare);
 
 		im_moved = *image;
 		shift_parameter(&im_moved, refine, +incr_val);
 		compare = find_intersections(&im_moved, cr, pmodel);
 		scan_partialities(crystal_get_reflections(cr), compare,
-		                  valid, vals, 2);
+		                  valid, vals, 2, pmodel);
 		reflist_free(compare);
 
 	}
@@ -271,7 +261,7 @@ static double test_gradients(Crystal *cr, double incr_val, int refine,
 	}
 	for ( i=0; i<nref; i++ ) valid[i] = 1;
 
-	scan_partialities(reflections, reflections, valid, vals, 1);
+	scan_partialities(reflections, reflections, valid, vals, 1, pmodel);
 
 	calc_either_side(cr, incr_val, valid, vals, refine, pmodel);
 
@@ -307,16 +297,15 @@ static double test_gradients(Crystal *cr, double incr_val, int refine,
 		} else {
 
 			double r1, r2, p;
-			int cl, ch;
 
 			grad1 = (vals[1][i] - vals[0][i]) / incr_val;
 			grad2 = (vals[2][i] - vals[1][i]) / incr_val;
 			grad = (grad1 + grad2) / 2.0;
 			i++;
 
-			cgrad = p_gradient(cr, refine, refl, pmodel);
+			cgrad = gradient(cr, refine, refl, pmodel);
 
-			get_partial(refl, &r1, &r2, &p, &cl, &ch);
+			get_partial(refl, &r1, &r2, &p);
 
 			if ( isnan(cgrad) ) {
 				n_nan++;
@@ -331,12 +320,12 @@ static double test_gradients(Crystal *cr, double incr_val, int refine,
 			vec2[n_line] = grad;
 			n_line++;
 
-			if ( (fabs(cgrad) < 5e-8) && (fabs(grad) < 5e-8) ) {
+			if ( (fabsl(cgrad) < 5e-8) && (fabsl(grad) < 5e-8) ) {
 				n_small++;
 				continue;
 			}
 
-			total += fabs(cgrad - grad);
+			total += fabsl(cgrad - grad);
 			ntot++;
 
 			if ( !within_tolerance(grad, cgrad, 5.0)
@@ -450,23 +439,18 @@ int main(int argc, char *argv[])
 
 	rng = gsl_rng_alloc(gsl_rng_mt19937);
 
-	for ( i=0; i<3; i++ ) {
+	for ( i=0; i<2; i++ ) {
 
 		UnitCell *rot;
 		double val;
 		PartialityModel pmodel;
 
 		if ( i == 0 ) {
-			pmodel = PMODEL_SPHERE;
-			STATUS("Testing flat sphere model:\n");
+			pmodel = PMODEL_SCSPHERE;
+			STATUS("Testing SCSphere model:\n");
 		} else if ( i == 1 ) {
-			pmodel = PMODEL_GAUSSIAN;
-			/* FIXME: Gradients for Gaussian model are not good */
-			STATUS("NOT testing Gaussian model.\n");
-			continue;
-		} else if ( i == 2 ) {
-			pmodel = PMODEL_THIN;
-			STATUS("Testing Thin Ewald Sphere model:\n");
+			pmodel = PMODEL_SCGAUSSIAN;
+			STATUS("Testing SCGaussian model.\n");
 		} else {
 			ERROR("WTF?\n");
 			return 1;
@@ -481,51 +465,51 @@ int main(int argc, char *argv[])
 			            &bz, &cx, &cy, &cz);
 
 		incr_val = incr_frac * image.div;
-		val =  test_gradients(cr, incr_val, REF_DIV, "div", "div",
+		val =  test_gradients(cr, incr_val, GPARAM_DIV, "div", "div",
 		                      pmodel, quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 
 		incr_val = incr_frac * crystal_get_profile_radius(cr);
-		val = test_gradients(cr, incr_val, REF_R, "R", "R", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_R, "R", "R", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 
 		incr_val = incr_frac * ax;
-		val = test_gradients(cr, incr_val, REF_ASX, "ax*", "x", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_ASX, "ax*", "x", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * bx;
-		val = test_gradients(cr, incr_val, REF_BSX, "bx*", "x", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_BSX, "bx*", "x", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * cx;
-		val = test_gradients(cr, incr_val, REF_CSX, "cx*", "x", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_CSX, "cx*", "x", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 
 		incr_val = incr_frac * ay;
-		val = test_gradients(cr, incr_val, REF_ASY, "ay*", "y", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_ASY, "ay*", "y", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * by;
-		val = test_gradients(cr, incr_val, REF_BSY, "by*", "y", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_BSY, "by*", "y", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * cy;
-		val = test_gradients(cr, incr_val, REF_CSY, "cy*", "y", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_CSY, "cy*", "y", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 
 		incr_val = incr_frac * az;
-		val = test_gradients(cr, incr_val, REF_ASZ, "az*", "z", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_ASZ, "az*", "z", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * bz;
-		val = test_gradients(cr, incr_val, REF_BSZ, "bz*", "z", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_BSZ, "bz*", "z", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 		incr_val = incr_frac * cz;
-		val = test_gradients(cr, incr_val, REF_CSZ, "cz*", "z", pmodel,
+		val = test_gradients(cr, incr_val, GPARAM_CSZ, "cz*", "z", pmodel,
 		                     quiet, plot);
 		if ( val < 0.99 ) fail = 1;
 
